<?php

namespace App\Http\Controllers;

use App\Business;
use App\BusinessCardReader;
use App\BusinessPrinter;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\OnlineOrder;
use App\ResellerCommission;
use App\ResellerPayoutInvoice;
use App\ResellerPayoutInvoiceGroup;
use App\StripeTransaction;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;

class AppResellerController extends Controller
{
    /**
     * Reseller Login for App
     * POST /reseller/login
     */
    public function login(Request $req)
    {
        $this->validate($req, [
            'login_email' => 'required',
            'password' => 'required',
        ]);
        $user = User::where('login_email', $req->input('login_email'))->first();
        if (empty($user)) Utils::error("Invalid login details", 400);

        if (!$user->reseller) {
            Utils::error("Username is not registered as reseller", 400);
        }
        if (!Hash::check($req->input('password'), $user->password)) {
            return Utils::error('Invalid password.', 400);
        } else {
            $tokens = (new UserController())->getNewTokens($user->id);

            return response()->json($user, 200, [
                'x-token' => (string) $tokens['aToken'],
                'x-refresh-token' => (string) $tokens['rToken'],
            ]);
        }
    }

    /**
     * Reseller dashboard for App
     * POST /reseller/dasboard
     */
    public function dashboard(Request $req)
    {
        $loggedInUser = Auth::user();
        if (!$loggedInUser->reseller) Utils::error("Unauthorised access", 401);

        $carbon = Carbon::today();
        $currentDayOfWeek = $carbon->dayOfWeek;

        if ($currentDayOfWeek == 1) {
            $from_date = date('Y-m-d', strtotime('tuesday previous week', $carbon->getTimestamp()));
            $to_date = date('Y-m-d', strtotime('monday this week', $carbon->getTimestamp()));
        } else {
            $from_date = date('Y-m-d', strtotime('tuesday this week', $carbon->getTimestamp()));
            $to_date = date('Y-m-d', strtotime('monday next week', $carbon->getTimestamp()));
        }
        $resellerPayoutInvoices = ResellerPayoutInvoice::select("reseller_payout_invoices.*",  "users.first_name", "users.last_name", "businesses.name", "businesses.country_id")->selectRaw('COUNT(*) as total_transactions')->leftJoin('reseller_invoice_transactions', 'reseller_invoice_transactions.reseller_payout_invoice_id', '=', 'reseller_payout_invoices.id')->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->orderByRaw('to_date DESC, reseller_payout_invoices.created_at DESC')->groupBy('reseller_payout_invoices.id');
        // $resellerPayoutInvoices->where('from_date', '>=', $from_date);
        // $resellerPayoutInvoices->where('to_date', '>=', $to_date);
        $resellerPayoutInvoices = $resellerPayoutInvoices->get();

        $data = [
            'reseller_payout_invoices' => $resellerPayoutInvoices,
        ];
        return $data;
    }

    /**
     * Get Reseller businesses
     * GET /reseller/businesses
     */
    public function businesses_index(Request $req)
    {
        return (new BusinessController())->index($req);
    }
    /**
     * Get Reseller businesses by id
     * GET /reseller/businesses/{business_id}
     */
    public function businesses_show(Request $req, $business_id)
    {
        $business = Business::selectRaw("id,online_service,merchant_service,name,address,email,contact_numbers,joining_date,restaurant_name,domain,domain_valid_till,type,country_id,bank_detail_id,paypal_detail_id,payout_mode,payout_method,archived_payout_mode,archived_payout_method,disabled")->find($business_id);
        if (empty($business)) Utils::error('Business not found!', 404);
        $business->reseller_commissions = ResellerCommission::where('business_id', $business->id)->where('user_id', Auth::id())->latest()->first();
        // $business->makeHidden(['paypal_mode', 'paypal_public_key_test', 'paypal_private_key_test', 'paypal_public_key_live', 'paypal_private_key_live', 'stripe_mode', 'stripe_public_key_test', 'stripe_private_key_test', 'stripe_public_key_live', 'stripe_private_key_live', 'footer_a', 'footer_b', 'header_a', 'header_b']);

        $from_date = Carbon::now()->subDays(7)->startOfDay();
        $to_date = Carbon::now()->endOfDay();
        $all_transactions = [];

        if ($business->online_service) {
            $onlienOrders = OnlineOrder::where('business_id', $business->id)->where('order_date', '>=', $from_date)->where('order_date', '<=', $to_date)->orderByRaw('order_date DESC')->get();
            if (!$onlienOrders->isEmpty()) {
                $onlienOrders->each(function ($order) {
                    $order->makeHidden('commissions');
                });
                $all_transactions = array_merge($all_transactions, $onlienOrders->toArray());
            }
        }
        if ($business->merchant_service) {
            $stipeTransactions = StripeTransaction::where('business_id', $business->id)->where('service', 'merchant')->where('created', '>=', $from_date->getTimestamp())->where('created', '<=', $to_date->getTimestamp())->orderByRaw('created DESC')->get();
            if (!$stipeTransactions->isEmpty()) {
                $stipeTransactions->each(function ($order) {
                    $order->makeHidden(['cash_handling_fee_type', 'cash_handling_fee', 'management_fee_type', 'management_fee', 'merchant_card_processing_fee_type', 'merchant_card_processing_fee', 'merchant_card_auth_fee_type', 'merchant_card_auth_fee', 'charge_back_fee_type', 'charge_back_fee']);
                });
                $all_transactions = array_merge($all_transactions, $stipeTransactions->toArray());
            }
        }
        if (!empty($all_transactions)) {
            usort($all_transactions, $this->date_compare('created_at'));
        }
        $business->transactions = $all_transactions;
        return $business;
    }
    /**
     * Show reseller profile
     * GET /reseller/me
     */
    public function show(Request $req)
    {
        return (new UserController())->show($req, 'me');
    }

    /**
     * Get grouped reseller payout invoices
     * GET /reseller/payout-invoices/groups
     */
    public function invoice_groups(Request $req)
    {
        $resellerPayoutInvoiceGroups = ResellerPayoutInvoiceGroup::where('user_id', Auth::id());
        if ($req->query('nopaginate', false) == '1') {
            $resellerPayoutInvoiceGroups = $resellerPayoutInvoiceGroups->get();
        } else {
            $resellerPayoutInvoiceGroups = $resellerPayoutInvoiceGroups->paginate($req->query('per_page', 20));
        }
        if ($resellerPayoutInvoiceGroups->isEmpty()) Utils::error("No data found", 400);
        return $resellerPayoutInvoiceGroups;
    }

    /**
     * Get grouped reseller payout invoices
     * GET /reseller/payout-invoices/groups/{id}
     */
    public function invoice_group_show(Request $req, $id)
    {
        $resellerPayoutInvoiceGroup = ResellerPayoutInvoiceGroup::find($id);
        if (empty($resellerPayoutInvoiceGroup)) Utils::error("No data found", 400);
        $resellerPayoutInvoiceGroup->invoices = ResellerPayoutInvoice::select("reseller_payout_invoices.*", "users.first_name", "users.last_name", "businesses.name", "businesses.country_id")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->where('from_date', $resellerPayoutInvoiceGroup->from_date)->where('to_date', $resellerPayoutInvoiceGroup->to_date)->where('user_id', $resellerPayoutInvoiceGroup->user_id)->get();
        return $resellerPayoutInvoiceGroup;
    }

    /**
     * Get grouped reseller payout invoices
     * GET /reseller/payout-invoices/grouped
     */
    public function invoices_group(Request $req)
    {
        $resellerPayoutInvoices = ResellerPayoutInvoice::selectRaw("id,from_date,to_date,SUM(net_fee) as net_fee")->where('user_id', Auth::id())->groupBy(['to_date', 'from_date'])->orderByRaw('to_date DESC, created_at DESC')->get();
        return $resellerPayoutInvoices;
    }

    /**
     * Get details reseller payout invoices
     * GET /reseller/payout-invoices/details?from_date=Y-m-d&to_date=Y-m-d
     */
    public function invoices_details(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'required|date',
            'to_date' => 'required|date|after:from_date',
        ]);

        $resellerPayoutInvoices = ResellerPayoutInvoice::select("reseller_payout_invoices.*", "users.first_name", "users.last_name", "businesses.name", "businesses.country_id")->leftJoin('users', 'users.id', '=', 'reseller_payout_invoices.user_id')->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->where('user_id', Auth::id())->orderByRaw('to_date DESC, reseller_payout_invoices.created_at DESC');

        if ($query = $req->query('from_date', false)) {
            $resellerPayoutInvoices->where('from_date', '=', $query);
        }
        if ($query = $req->query('to_date', false)) {
            $resellerPayoutInvoices->where('to_date', '=', $query);
        }
        $resellerPayoutInvoices = $resellerPayoutInvoices->get();
        return $resellerPayoutInvoices;
    }


    /**
     * show reseller payout invoice by id
     * GET /reseller/payout-invoices/{id}
     */
    public function invoices_show(Request $req, $id)
    {
        $payoutInvoice = ResellerPayoutInvoice::select("reseller_payout_invoices.*", "businesses.country_id")->leftJoin('businesses', 'businesses.id', '=', 'reseller_payout_invoices.business_id')->find($id);
        if (empty($payoutInvoice)) Utils::error('ResellerPayoutInvoice not found!', 404);
        $payoutInvoice->append('transactions');
        $payoutInvoice->append('business');
        $payoutInvoice->append('reseller');
        return $payoutInvoice;
    }

    public function date_compare($key)
    {
        return function ($a, $b) use ($key) {
            $t1 = strtotime($a[$key]);
            $t2 = strtotime($b[$key]);
            return $t2 - $t1;
        };
    }
}
