<?php

namespace App\Http\Controllers;

use App\Business;
use App\Helpers\ImageHelper;
use App\BusinessCardReader;
use App\BusinessTip;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Exception;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Stripe\StripeClient;

class BusinessCardReaderController extends Controller
{
    /**
     * List businessCardReaders.
     * GET /api/business-card-readers
     */
    public function index(Request $req)
    {
        $businessCardReaders = BusinessCardReader::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $businessCardReaders->whereRaw('(name LIKE ? OR ip LIKE ? OR port LIKE ? OR connectivity LIKE ? OR merchant_key LIKE ? OR merchant_secret LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('connectivity', false)) {
            $businessCardReaders->where('connectivity', $query);
        }
        if ($query = $req->query('business_id', false)) {
            $businessCardReaders->where('business_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $businessCardReaders = $businessCardReaders->get();
        } else {
            $businessCardReaders = $businessCardReaders->paginate($req->query('per_page', 20));
        }

        if ($businessCardReaders->isEmpty()) Utils::error('No businessCardReaders found!', 404);


        return ($businessCardReaders);
    }

    /**
     * Create businessCardReader.
     * POST /api/business-card-readers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string',
            'connectivity' => 'required|in:Wifi,Bluetooth,Internet,StripeBluetooth,POSBluetooth,TAPTOPAY',
            'ip' => 'required_if:connectivity,Wifi',
            'port' => 'required_if:connectivity,Wifi',
            'card_reader_id' => 'required|exists:card_readers,id',
            'business_id' => 'required|exists:businesses,id',
            'merchant_key' => 'required_if:connectivity,Wifi,Bluetooth|string',
            'merchant_secret' => 'required_if:connectivity,Wifi,Bluetooth|string',
            's_registration_code' => 'required_if:connectivity,Internet',
            's_location_id' => 'string',
            'serial_number' => 'string'
        ]);

        $business = Business::find($req->input('business_id'));
        if ($business->name == '') {
            Utils::error("Please fill business name first", 400);
        }
        if ($business->address == '') {
            Utils::error("Please fill business address first", 400);
        }
        if ($business->city == '') {
            Utils::error("Please fill business city first", 400);
        }
        if ($business->postcode == '') {
            Utils::error("Please fill business postcode first", 400);
        }

        // create businessCardReader
        $businessCardReader = new BusinessCardReader;
        $businessCardReader->fill($req->all());
        if ($req->has('merchant_key')) $businessCardReader->merchant_key = trim($req->input('merchant_key'));
        if ($req->has('merchant_secret')) $businessCardReader->merchant_secret =  trim($req->input('merchant_secret'));
        $businessCardReader->save();

        if (($req->input('connectivity') == 'Internet' || $req->input('connectivity') == 'StripeBluetooth' || $req->input('connectivity') == 'POSBluetooth' || $req->input('connectivity') == 'TAPTOPAY')) {
            if ($business->s_account_id != null && $business->connect_service == 1) {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
            } else {
                $mode = SiteSettingController::findByKey('stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'stripe_private_key_live';
                }
            }

            $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
            $locationParams = [
                'display_name' => $business->name,
                'address' => [
                    'line1' => $business->door_no . " " . $business->address,
                    'city' => $business->city,
                    'state' => $business->state,
                    'country' => $business->country->iso,
                    'postal_code' => $business->postcode
                ]
            ];
            if ($business->s_account_id != null && $business->connect_service == 1) {
                $locationParams['metadata'] = [
                    'connected_account' => $business->s_account_id
                ];
            }

            if ($req->has('s_location_id')) {
                try {
                    $tips = BusinessTip::where('business_id', $business->id)->select('tip')->toBase()->get()->pluck('tip')->toArray();
                    if (count($tips) > 3) {
                        $configuration = $stripe->terminal->configurations->create([
                            'tipping' => ['gbp' => ['fixed_amounts' => array_map('intval', array_values($tips))]],
                        ]);

                        $stripe->terminal->locations->update(
                            $businessCardReader->s_location_id,
                            [
                                'configuration_overrides' => $configuration->id
                            ]
                        );
                    }
                } catch (Exception $e) {
                    Log::debug($e);
                }
            }

            if ($businessCardReader->s_location_id == null) {
                // create location
                $stripeLocation = $stripe->terminal->locations->create($locationParams);

                $tips = BusinessTip::where('business_id', $business->id)->select('tip')->toBase()->get()->pluck('tip')->toArray();
                if (count($tips) > 3) {
                    $configuration = $stripe->terminal->configurations->create([
                        'tipping' => ['gbp' => ['fixed_amounts' => array_map('intval', array_values($tips))]],
                    ]);

                    $stripe->terminal->locations->update(
                        $stripeLocation->id,
                        [
                            'configuration_overrides' => $configuration->id
                        ]
                    );
                }

                $businessCardReader->s_location_id = $stripeLocation->id;
                $businessCardReader->save();
                $business->s_location_id = $stripeLocation->id;
                $business->save();
            }
            if ($businessCardReader->connectivity == 'Internet') {
                // assign card reader to location
                $stripeCardReader = $stripe->terminal->readers->create([
                    'registration_code' => $businessCardReader->s_registration_code,
                    'label' => $businessCardReader->name,
                    'location' => $businessCardReader->s_location_id,
                ]);
                $businessCardReader->s_terminal_id = $stripeCardReader->id;
                $businessCardReader->save();
            }
        }
        return $businessCardReader;
    }

    /**
     * Show businessCardReader.
     * GET /api/business-card-readers/{id}
     */
    public function show(Request $req, $id)
    {
        $businessCardReader = BusinessCardReader::find($id);
        if (empty($businessCardReader)) Utils::error('BusinessCardReader not found!', 404);
        return $businessCardReader;
    }

    /**
     * Update businessCardReader.
     * POST /api/business-card-readers/{id}
     */
    public function update(Request $req, $id)
    {

        $dbBusinessCardReader = BusinessCardReader::find($id);
        $businessCardReader = clone ($dbBusinessCardReader);
        if (empty($businessCardReader)) Utils::error('BusinessCardReader not found!', 404);

        $this->validate($req, [
            'name' => 'string',
            'connectivity' => 'required|in:Wifi,Bluetooth,Internet,StripeBluetooth,POSBluetooth,TAPTOPAY',
            'ip' => 'required_if:connectivity,Wifi',
            'port' => 'required_if:connectivity,Wifi',
            'card_reader_id' => 'exists:card_readers,id',
            'business_id' => 'exists:businesses,id',
            'merchant_key' => 'required_if:connectivity,Wifi,Bluetooth|string',
            'merchant_secret' => 'required_if:connectivity,Wifi,Bluetooth|string',
            's_registration_code' => 'required_if:connectivity,Internet',
            's_location_id' => 'string',
            'serial_number' => 'string',

        ]);

        // update businessCardReader
        if ($req->has('name')) $businessCardReader->name = $req->input('name');
        if ($req->has('connectivity')) $businessCardReader->connectivity = $req->input('connectivity');
        if ($req->has('ip')) $businessCardReader->ip = $req->input('ip');
        if ($req->has('port')) $businessCardReader->port = $req->input('port');
        if ($req->has('card_reader_id')) $businessCardReader->card_reader_id = $req->input('card_reader_id');
        if ($req->has('business_id')) $businessCardReader->business_id = $req->input('business_id');
        if ($req->has('merchant_key')) $businessCardReader->merchant_key = trim($req->input('merchant_key'));
        if ($req->has('merchant_secret')) $businessCardReader->merchant_secret =  trim($req->input('merchant_secret'));
        if ($req->has('s_registration_code')) $businessCardReader->s_registration_code = $req->input('s_registration_code');
        if ($req->has('s_location_id')) $businessCardReader->s_location_id = $req->input('s_location_id');
        $businessCardReader->serial_number = $req->input('serial_number');

        $businessCardReader->save();

        if ($req->input('connectivity') != $dbBusinessCardReader->connectivity && ($req->input('connectivity') == 'Internet' || $req->input('connectivity') == 'StripeBluetooth' || $req->input('connectivity') == 'POSBluetooth' || $req->input('connectivity') == 'TAPTOPAY')) {
            $business = Business::find($req->input('business_id'));
            if ($business->s_account_id != null && $business->connect_service == 1) {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
            } else {
                $mode = SiteSettingController::findByKey('stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'stripe_private_key_live';
                }
            }

            $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);
            $locationParams = [
                'display_name' => $business->name,
                'address' => [
                    'line1' => $business->door_no . " " . $business->address,
                    'city' => $business->city,
                    'state' => $business->state,
                    'country' => $business->country->iso,
                    'postal_code' => $business->postcode
                ]
            ];
            if ($business->s_account_id != null && $business->connect_service == 1) {
                $locationParams['metadata'] = [
                    'connected_account' => $business->s_account_id
                ];
            }
            if ($businessCardReader->s_location_id == null) {
                // create location
                $stripeLocation = $stripe->terminal->locations->create($locationParams);
                $businessCardReader->s_location_id = $stripeLocation->id;
                $businessCardReader->save();
                $business->s_location_id = $stripeLocation->id;
                $business->save();
            }
            // assign card reader to location
            if ($businessCardReader->connectivity == 'Internet') {
                $stripeCardReader = $stripe->terminal->readers->create([
                    'registration_code' => $businessCardReader->s_registration_code,
                    'label' => $businessCardReader->name,
                    'location' => $businessCardReader->s_location_id,
                ]);
                $businessCardReader->s_terminal_id = $stripeCardReader->id;
                $businessCardReader->save();
            }
        }
        return $businessCardReader;
    }

    /**
     * Enable businessCardReader.
     * POST /api/business-card-readers/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $businessCardReader = BusinessCardReader::find($id);
        if (empty($businessCardReader)) Utils::error('BusinessCardReader not found!', 404);
        $businessCardReader->disabled = 0;
        $businessCardReader->save();
        return $businessCardReader;
    }

    /**
     * Disable businessCardReader.
     * POST /api/business-card-readers/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $businessCardReader = BusinessCardReader::find($id);
        if (empty($businessCardReader)) Utils::error('BusinessCardReader not found!', 404);
        $businessCardReader->disabled = 1;
        $businessCardReader->save();
        return $businessCardReader;
    }
    /**
     * Delete businessCardReader by id.
     * DELETE /api/business-card-readers/{id}
     */
    public function delete(Request $req, $id)
    {
        $businessCardReader = BusinessCardReader::find($id);
        if (empty($businessCardReader)) Utils::error('BusinessCardReader not found!', 404);
        $businessCardReader->delete();
        return $businessCardReader;
    }


    /**
     * Enable USB businessCardReader.
     * POST /api/business-card-readers/{id}/enable-usb
     */
    public function enable_usb(Request $req, $id)
    {
        $businessCardReader = BusinessCardReader::find($id);
        if (empty($businessCardReader)) Utils::error('BusinessCardReader not found!', 404);
        $businessCardReader->isUsb = 0;
        $businessCardReader->save();
        return $businessCardReader;
    }

    /**
     * Disable USB businessCardReader.
     * POST /api/business-card-readers/{id}/disable-usb
     */
    public function disable_usb(Request $req, $id)
    {
        $businessCardReader = BusinessCardReader::find($id);
        if (empty($businessCardReader)) Utils::error('BusinessCardReader not found!', 404);
        $businessCardReader->isUsb = 1;
        $businessCardReader->save();
        return $businessCardReader;
    }
}
