<?php

namespace App\Http\Controllers;

use App\BusinessPlan;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class BusinessPlanController extends Controller
{
    /**
     * List businessPlans.
     * GET /api/business-plans
     */
    public function index(Request $req)
    {
        $businessPlans = BusinessPlan::selectRaw('business_plans.*,plans.name as plan_name')->leftJoin('plans', 'plans.id', '=', 'plan_id')->orderByRaw('created_at DESC');

        if ($query = $req->query('business_id', false)) {
            $businessPlans->where('business_id', $query);
        }
        if ($query = $req->query('plan_id', false)) {
            $businessPlans->where('plan_id', $query);
        }
        $businessPlans = $businessPlans->get();

        if ($businessPlans->isEmpty()) Utils::error('No plans found!', 404);

        return ($businessPlans);
    }

    /**
     * Create businessPlan.
     * POST /api/business-plans
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'business_id' =>  ['required', 'exists:businesses,id', Rule::unique('business_plans')->where(function ($query) use ($req) {
                return $query->where('business_id', $req->input('business_id'))->where('service', $req->input('service')); //where('plan_id', $req->input('plan_id'))->
            })],
            'plan_id' =>  ['required', 'exists:plans,id', Rule::unique('business_plans')->where(function ($query) use ($req) {
                return $query->where('business_id', $req->input('business_id'))->where('service', $req->input('service')); //where('plan_id', $req->input('plan_id'))->
            })],
            'service' => 'in:Epos,Online,Merchant,Card,Connect',
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after_or_equal:from_date',
        ], [
            'business_id.unique' => "Plan already assigned same service with the business",
            'plan_id.unique' => "Business already assigned same service with the plan",
        ]);

        // create businessPlan
        $businessPlan = new BusinessPlan;
        $businessPlan->fill($req->all());
        $businessPlan->save();

        return $businessPlan;
    }

    /**
     * Show businessPlan.
     * GET /api/business-plans/{id}
     */
    public function show(Request $req, $id)
    {
        $businessPlan = BusinessPlan::find($id);
        if (empty($businessPlan)) Utils::error('Plan not found!', 404);
        return $businessPlan;
    }

    /**
     * Update businessPlan.
     * POST /api/business-plans/{id}
     */
    public function update(Request $req, $id)
    {
        $businessPlan = BusinessPlan::find($id);
        if (empty($businessPlan)) Utils::error('Plan not found!', 404);

        $this->validate($req, [
            'business_id' =>  ['exists:businesses,id', Rule::unique('business_plans')->where(function ($query) use ($req, $id) {
                return $query->where('business_id', $req->input('business_id'))->whereNotIn('id', [$id]); // where('plan_id', $req->input('plan_id'))->
            })],
            'plan_id' =>  ['exists:plans,id', Rule::unique('business_plans')->where(function ($query) use ($req, $id) {
                return $query->where('business_id', $req->input('business_id'))->whereNotIn('id', [$id]); // where('plan_id', $req->input('plan_id'))->
            })],
            'service' => 'in:Epos,Online,Merchant,Card,Connect',
            'from_date' => 'required|date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d|after_or_equal:from_date',
        ], [
            'business_id.unique' => "Plan already assigned with the business",
            'plan_id.unique' => "Business already assigned with the plan",
        ]);

        // update businessPlan
        if ($req->has('business_id')) $businessPlan->business_id = $req->input('business_id');
        if ($req->has('plan_id')) $businessPlan->plan_id = $req->input('plan_id');
        if ($req->has('service')) $businessPlan->service = $req->input('service');
        if ($req->has('from_date')) $businessPlan->from_date = $req->input('from_date');
        if ($req->has('to_date')) $businessPlan->to_date = $req->input('to_date');

        $businessPlan->save();

        return $businessPlan;
    }

    /**
     * Delete businessPlan by id.
     * DELETE /api/business-plans/{id}
     */
    public function delete(Request $req, $id)
    {
        $businessPlan = BusinessPlan::find($id);
        if (empty($businessPlan)) Utils::error('Plan not found!', 404);
        $businessPlan->delete();
        return $businessPlan;
    }
}
