<?php

namespace App\Http\Controllers;

use App\Business;
use App\Helpers\ImageHelper;
use App\BusinessPrinter;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Epos\EposPrinterController;
use App\ModelsEpos\EposPrinter;
use App\ModelsEpos\EposPrinterModel;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class BusinessPrinterController extends Controller
{
    /**
     * List businessPrinters.
     * GET /api/business-printers
     */
    public function index(Request $req)
    {
        $businessPrinters = BusinessPrinter::selectRaw('business_printers.*,printers.name as printer_model_name')->leftJoin('printers', 'printers.id', '=', 'printer_id')->orderByRaw('business_printers.created_at DESC');

        if ($query = $req->query('query', false)) {
            $businessPrinters->whereRaw('(business_printers.name LIKE ? OR business_printers.ip LIKE ? OR business_printers.port LIKE ? OR business_printers.connectivity LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $businessPrinters->where('business_printers.business_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $businessPrinters = $businessPrinters->get();
        } else {
            $businessPrinters = $businessPrinters->paginate($req->query('per_page', 20));
        }

        if ($businessPrinters->isEmpty()) Utils::error('No businessPrinters found!', 404);


        return ($businessPrinters);
    }

    /**
     * Create businessPrinter.
     * POST /api/business-printers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string',
            'ip' => 'required_if:connectivity,Wifi',
            'port' => 'required_if:connectivity,Wifi',
            'printer_id' => 'required|exists:printers,id',
            'business_id' => 'required|exists:businesses,id',
        ]);

        // create businessPrinter
        $businessPrinter = new BusinessPrinter;
        $businessPrinter->fill($req->all());

        $businessPrinter->save();

        $business = Business::find($businessPrinter->business_id);
        $this->establishEposConnection($business);
        if ($business->epos_service == 1) {
            $printerModel = new EposPrinter();
            $printerModel->name = $businessPrinter->name;
            $printerModel->printer_model_id = $businessPrinter->printer_id;
            if ($req->input('ip')) $printerModel->ip = $businessPrinter->ip;
            $printerModel->id = $businessPrinter->id;
            $printerModel->save();
        }
        return $businessPrinter;
    }

    /**
     * Show businessPrinter.
     * GET /api/business-printers/{id}
     */
    public function show(Request $req, $id)
    {
        $businessPrinter = BusinessPrinter::find($id);
        if (empty($businessPrinter)) Utils::error('BusinessPrinter not found!', 404);
        return $businessPrinter;
    }

    /**
     * Update businessPrinter.
     * POST /api/business-printers/{id}
     */
    public function update(Request $req, $id)
    {
        $businessPrinter = BusinessPrinter::find($id);
        if (empty($businessPrinter)) Utils::error('BusinessPrinter not found!', 404);

        $this->validate($req, [
            'name' => 'string',
            'ip' => 'required_if:connectivity,Wifi',
            'port' => 'required_if:connectivity,Wifi',
            'printer_id' => 'exists:printers,id',
            'business_id' => 'exists:businesses,id',

        ]);

        // update businessPrinter
        if ($req->has('name')) $businessPrinter->name = $req->input('name');
        $businessPrinter->ip = $req->input('ip');
        $businessPrinter->port = $req->input('port');
        if ($req->has('printer_id')) $businessPrinter->printer_id = $req->input('printer_id');
        if ($req->has('business_id')) $businessPrinter->business_id = $req->input('business_id');

        $businessPrinter->save();

        $business = Business::find($businessPrinter->business_id);
        $this->establishEposConnection($business);
        if ($business->epos_service == 1) {
            // $printerModel = new EposPrinter();
            $printerModel = EposPrinter::find($id);
            $printerModel->name = $businessPrinter->name;
            $printerModel->printer_model_id = $businessPrinter->printer_id;
            if ($req->input('ip')) $printerModel->ip = $businessPrinter->ip;
            $printerModel->id = $businessPrinter->id;
            $printerModel->save();
        }

        return $businessPrinter;
    }

    /**
     * Enable businessPrinter.
     * POST /api/business-printers/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $businessPrinter = BusinessPrinter::find($id);
        if (empty($businessPrinter)) Utils::error('BusinessPrinter not found!', 404);
        $businessPrinter->disabled = 0;
        $businessPrinter->save();
        return $businessPrinter;
    }

    /**
     * Disable businessPrinter.
     * POST /api/business-printers/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $businessPrinter = BusinessPrinter::find($id);
        if (empty($businessPrinter)) Utils::error('BusinessPrinter not found!', 404);
        $businessPrinter->disabled = 1;
        $businessPrinter->save();
        return $businessPrinter;
    }
    /**
     * Delete businessPrinter by id.
     * DELETE /api/business-printers/{id}
     */
    public function delete(Request $req, $id)
    {
        $businessPrinter = BusinessPrinter::find($id);
        if (empty($businessPrinter)) Utils::error('BusinessPrinter not found!', 404);
        $businessPrinter->delete();

        $business = Business::find($businessPrinter->business_id);
        $this->establishEposConnection($business);
        if ($business->epos_service == 1) {
            $printer = EposPrinter::find($id);
            $printer->delete();
        }

        return $businessPrinter;
    }
    /**
     * Disable businessPrinter.
     * POST /api/business-printers/{id}/disable
     */
    public function select(Request $req, $id)
    {
        $businessPrinters = BusinessPrinter::where('is_default', 1)->where('business_id', $req->input('business_id'))->get();
        if (!empty($businessPrinters)) {
            foreach ($businessPrinters as $businessPrinter) {
                $business_printer = BusinessPrinter::find($businessPrinter['id']);
                $business_printer->is_default = 0;
                $business_printer->save();
            }
        }

        $businessPrinter = BusinessPrinter::find($id);
        if (empty($businessPrinter)) Utils::error('BusinessPrinter not found!', 404);
        $businessPrinter->is_default = 1;
        $businessPrinter->save();
        return $businessPrinter;
    }

    //Connection Check on Epos
    public function establishEposConnection($business)
    {
        if ($business->epos_service == 0) Utils::error("EPOS Service not enabled for $business->name", 400);
        if ($business->db_name == null) Utils::error("No database set for $business->name", 400);
        if ($business->database_server_id == null) Utils::error("No server assigned for $business->name", 400);
        $business->append('database_server');
        $databaseServer = $business->database_server;

        if (empty($databaseServer)) Utils::error("Database server not found", 400);


        $config = app()->make('config');
        $config->set('database.connections.epos_connection', [
            'driver'   => 'mysql',
            'host'   => $databaseServer->local_ip,
            'database'   => $business->db_name,
            'username'   => $databaseServer->username,
            'password'   => $databaseServer->password,
            'port'   => $databaseServer->port,
        ]);
    }
}
