<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\Country;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class CountryController extends Controller
{
    /**
     * List countries.
     * GET /api/countries
     */
    public function index(Request $req)
    {
        $countries = Country::orderByRaw('name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $countries->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $countries = $countries->get();
        } else {
            $countries = $countries->paginate($req->query('per_page', 20));
        }

        if ($countries->isEmpty()) Utils::error('No countries found!', 404);


        return ($countries);
    }

    /**
     * Create country.
     * POST /api/countries
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'iso' => 'required|string|max:255|unique:countries,iso',
            'iso3' => 'required|string|unique:countries,iso3',
            'phone_code' => 'required|string',
            'currency_name' => 'required|string',
            'currency_code' => 'required|string',
            'currency_symbol' => 'required|string',
            'image' => 'image'
        ]);

        // create country
        $country = new Country;
        $country->fill($req->all());

        $country->save();

        return $country;
    }

    /**
     * Show country.
     * GET /api/countries/{id}
     */
    public function show(Request $req, $id)
    {
        $country = Country::find($id);
        if (empty($country)) Utils::error('Country not found!', 404);
        return $country;
    }

    /**
     * Update country.
     * POST /api/countries/{id}
     */
    public function update(Request $req, $id)
    {
        $country = Country::find($id);
        if (empty($country)) Utils::error('Country not found!', 404);

        $this->validate($req, [
            'name' => 'string|max:255',
            'iso' => ['string', 'max:255', Rule::unique('countries')->ignore($country->id)],
            'iso3' => ['string', 'max:255', Rule::unique('countries')->ignore($country->id)],
            'phone_code' => 'string',
            'currency_name' => 'string',
            'currency_code' => 'string',
            'currency_symbol' => 'string',
            'image' => 'image'
        ]);

        // update country
        if ($req->has('name')) $country->name = $req->input('name');
        if ($req->has('phone_code')) $country->phone_code = $req->input('phone_code');
        if ($req->has('iso')) $country->iso = $req->input('iso');
        if ($req->has('iso3')) $country->iso3 = $req->input('iso3');
        if ($req->has('currency_name')) $country->currency_name = $req->input('currency_name');
        if ($req->has('currency_code')) $country->currency_code = $req->input('currency_code');
        if ($req->has('currency_symbol')) $country->currency_symbol = $req->input('currency_symbol');

        $country->save();

        return $country;
    }


    /**
     * Delete country by id.
     * DELETE /api/countries/{id}
     */
    public function delete(Request $req, $id)
    {
        $country = Country::find($id);
        if (empty($country)) Utils::error('Country not found!', 404);
        $country->delete();
        return $country;
    }
}
