<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\ImageHelper;
use App\ModelsEpos\EposAddon;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsEpos\EposProductAddon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposAddonController extends Controller
{
    /**
     * List addons.
     * GET /epos/addons
     */
    public function index(Request $req)
    {
        $addons = EposAddon::orderByRaw('sequence ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $addons->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('disabled', false)) {
            $addons->where('disabled', $query);
        }
        if ($query = $req->query('product_id', false)) {
            if ($requirement = $req->query('requirement', false)) {
                if ($requirement == 'with') {
                    $addons->whereRaw('(id NOT IN (SELECT addon_id from product_addons where product_id = ? AND with=1))', [$req->input('product_id')]);
                }
                if ($requirement == 'without') {
                    $addons->whereRaw('(id NOT IN (SELECT addon_id from product_addons where product_id = ? AND with=0))', [$req->input('product_id')]);
                }
            } else {
                $addons->whereRaw('(id NOT IN (SELECT addon_id from product_addons where product_id = ?))', [$req->input('product_id')]);
            }
        }
        if (!$query = $req->query('all', false)) {
            if ($parentId = $req->query('parent_id', false)) {
                $addons->where('parent_id', $parentId);
            } else {
                $addons->whereNull('parent_id');
            }
        }
        if (!$prefilled = $req->query('prefilled', false)) {
            $addons->each(function ($items) {
                $items->makeHidden('children');
            });
        }
        if ($req->query('nopaginate', false) == '1') {
            $addons = $addons->get();
        } else {
            $addons = $addons->paginate($req->query('per_page', 20));
        }

        if ($addons->isEmpty()) Utils::error('No addons found!', 404);


        return ($addons);
    }

    /**
     * Create addon.
     * POST /epos/addons
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'image' => 'image',
            'parent_id' => 'exists:epos_connection.addons,id',
            'price' => 'numeric',
            'takeaway_price' => 'numeric',
            'web_price' => 'numeric',
            'delivery_price' => 'numeric',
            'waiting_price' => 'numeric',
            'sequence' => 'numeric',
            'top_color' => 'string|max:9|min:7',
            'bottom_color' => 'string|max:9|min:7',
        ]);

        // create addon
        $addon = new EposAddon;
        $addon->fill($req->all());
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_EPOS_ADDON);
            $addon->image = $fileName;
        }
        $addon->name = strtoupper($req->input('name'));
        $addon->save();

        // Sequence add
        $addon->sequence = $addon->id;
        $addon->save();

        return $addon;
    }

    /**
     * Show addon.
     * GET /epos/addons/{id}
     */
    public function show(Request $req, $id)
    {
        $addon = EposAddon::find($id);
        if (empty($addon)) Utils::error('EposAddon not found!', 404);
        return $addon;
    }

    /**
     * Update addon.
     * POST /epos/addons/{id}
     */
    public function update(Request $req, $id)
    {
        $addon = EposAddon::find($id);
        if (empty($addon)) Utils::error('EposAddon not found!', 404);

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'image' => 'image',
            'parent_id' => 'exists:epos_connection.addons,id',
            'price' => 'numeric',
            'takeaway_price' => 'numeric',
            'web_price' => 'numeric',
            'delivery_price' => 'numeric',
            'waiting_price' => 'numeric',
            'sequence' => 'numeric',
            'top_color' => 'string|max:9|min:7',
            'bottom_color' => 'string|max:9|min:7',
        ]);

        // update addon
        if ($req->has('name')) $addon->name = strtoupper($req->input('name'));
        $addon->parent_id = $req->input('parent_id');
        $addon->price = $req->input('price');
        $addon->takeaway_price = $req->input('takeaway_price');
        $addon->web_price = $req->input('web_price');
        $addon->delivery_price = $req->input('delivery_price');
        $addon->waiting_price = $req->input('waiting_price');
        $addon->sequence = $req->input('sequence');
        if ($req->has('top_color')) $addon->top_color = $req->input('top_color');
        if ($req->has('bottom_color')) $addon->bottom_color = $req->input('bottom_color');
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_EPOS_ADDON);
            $addon->image = $fileName;
        }
        $addon->save();

        return $addon;
    }


    /**
     * Delete addon by id.
     * DELETE /epos/addons/{id}
     */
    public function delete(Request $req, $id)
    {
        $addon = EposAddon::find($id);
        if (empty($addon)) Utils::error('EposAddon not found!', 404);
        $addon->delete();
        return $addon;
    }

    /**
     * Enable addon.
     * POST /epos/addons/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $addon = EposAddon::find($id);
        if (empty($addon)) Utils::error('EposAddon not found!', 404);
        $addon->disabled = 0;
        $addon->save();

        $productAddons = EposProductAddon::where('addon_id', $id)->get();
        if (!empty($productAddons)) {
            foreach ($productAddons as $productaddon) {
                $productaddonObj = EposProductAddon::find($productaddon->id);
                $productaddonObj->disabled = 0;
                $productaddonObj->save();
            }
        }

        return $addon;
    }

    /**
     * Disable addon.
     * POST /epos/addons/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $addon = EposAddon::find($id);
        if (empty($addon)) Utils::error('EposAddon not found!', 404);
        $addon->disabled = 1;
        $addon->save();

        $productAddons = EposProductAddon::where('addon_id', $id)->get();
        if (!empty($productAddons)) {
            foreach ($productAddons as $productaddon) {
                $productaddonObj = EposProductAddon::find($productaddon->id);
                $productaddonObj->disabled = 1;
                $productaddonObj->save();
            }
        }
        return $addon;
    }

    public function addon_bulk(Request $req)
    {
        if ($req->has('addons') && !is_array($req->input('addons'))) {
            $req->merge(['addons' => json_decode($req->input('addons'))]);
        }
        $this->validate($req, [
            'addons' => 'required|array'
        ],);
        $addons = ($req->input('addons'));
        $count = 1;
        foreach ($addons as $addon) {
            $addonObj = EposAddon::find($addon->id);
            $addonObj->sequence = $count;
            $addonObj->save();
            $count++;
        }
        return $addons;
    }
}