<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\ImageHelper;
use App\ModelsEpos\EposCategory;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposCategoryController extends Controller
{
    /**
     * List categories.
     * GET /epos/categories
     */
    public function index(Request $req)
    {
        $categories = EposCategory::orderByRaw('sequence ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $categories->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }

        if ($query = $req->query('print_block_id', false)) {
            if ($requirement = $req->query('requirement', false)) {
                $categories->whereRaw('(id NOT IN (SELECT category_id from print_block_assigns where print_block_id = ?))', [$req->input('print_block_id')]);
            }
        }

        if ($query = $req->query('is_dinein', false)) {
            $categories->where('is_dinein', 1);
        }
        if ($query = $req->query('is_delivery', false)) {
            $categories->where('is_delivery', 1);
        }
        if ($query = $req->query('is_collection', false)) {
            $categories->where('is_collection', 1);
        }
        if ($query = $req->query('is_banquet', false)) {
            $categories->where('is_banquet', 1);
        }

        if ($req->query('disabled') == '0' || $req->query('disabled') == '1') {
            $categories->where('disabled', $req->query('disabled'));
        }

        if (!$query = $req->query('all', false)) {
            if ($parentId = $req->query('parent_id', false)) {
                $categories->where('parent_id', $parentId);
            } else {
                $categories->whereNull('parent_id');
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $categories = $categories->get();
        } else {
            $categories = $categories->paginate($req->query('per_page', 20));
        }

        if ($categories->isEmpty()) Utils::error('More categories coming soon!', 404);

        if (!$prefilled = $req->query('prefilled', false)) {
            $categories->each(function ($items) {
                $items->makeHidden('children');
            });
        }
        return $categories;
    }


    /**
     * List categories.
     * GET /api/categories/mini
     */
    public function mini(Request $req)
    {
        $categories = EposCategory::orderByRaw('id ASC');
        // $categories->where('disabled', 0);

        if ($query = $req->query('print_block_id', false)) {
            if ($requirement = $req->query('requirement', false)) {
                $categories->whereRaw('(id NOT IN (SELECT category_id from print_block_assigns where print_block_id = ?))', [$req->input('print_block_id')]);
                $categories->whereRaw('(id NOT IN (SELECT category_id from print_block_assigns))', []);
            }
        }

        $categories = $categories->get();
        if ($categories->isEmpty()) Utils::error('No categories found!', 404);
        $categories->each(function ($category) {
            $category->makeHidden('children');
        });

        return ($categories);
    }

    private function buildTree($categories)
    {
        $children = array();
        foreach ($categories as $item)
            $children[$item->parent_id ?? 'root'][] = $item;

        foreach ($categories as $item) {
            if (isset($children[$item->id])) {
                $item->children = $children[$item->id];
            }
        }

        return $children[$item->parent_id ?? 'root'];
    }
    /**
     * Create category.
     * POST /epos/categories
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'image' => 'image',
            'description' => 'string|max:255',
            'sequence' => 'numeric',
            'parent_id' => 'exists:epos_connection.categories,id',
            'top_color' => 'required|string|max:9|min:7',
            'bottom_color' => 'required|string|max:9|min:7',
            'font_color' => 'required|string|max:9|min:7',
            'is_dinein' => 'required|boolean',
            'is_delivery' => 'required|boolean',
            'is_collection' => 'required|boolean',
            'is_banquet' => 'required|boolean',
        ]);

        // create category
        $category = new EposCategory;
        $category->fill($req->all());
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_EPOS_CATEGORY);
            $category->image = $fileName;
        }
        if ($req->has('name')) $category->name = (new ExtraController())->convertToTitleCase($req->input('name'));
        $category->save();

        // Sequence add
        $category->sequence = $category->id;
        $category->save();

        return $category;
    }

    /**
     * Show category.
     * GET /epos/categories/{id}
     */
    public function show(Request $req, $id)
    {
        $category = EposCategory::find($id);
        if (empty($category)) Utils::error('EposCategory not found!', 404);

        return $category;
    }

    /**
     * Update category.
     * POST /epos/categories/{id}
     */
    public function update(Request $req, $id)
    {
        $category = EposCategory::find($id);
        if (empty($category)) Utils::error('EposCategory not found!', 404);

        $this->validate($req, [
            'name' => 'string|max:255',
            'image' => 'image',
            'description' => 'string|max:255',
            'sequence' => 'numeric',
            'parent_id' => 'exists:epos_connection.categories,id',
            'top_color' => 'string|max:9|min:7',
            'bottom_color' => 'string|max:9|min:7',
            'font_color' => 'string|max:9|min:7',
            'is_dinein' => 'required|boolean',
            'is_delivery' => 'required|boolean',
            'is_collection' => 'required|boolean',
            'is_banquet' => 'required|boolean',
        ]);

        // update category
        if ($req->has('name')) $category->name = (new ExtraController())->convertToTitleCase($req->input('name'));
        if ($req->has('description')) $category->description = $req->input('description');
        if ($req->has('sequence')) $category->sequence = $req->input('sequence');
        $category->parent_id = $req->input('parent_id');
        if ($req->has('top_color')) $category->top_color = $req->input('top_color');
        if ($req->has('bottom_color')) $category->bottom_color = $req->input('bottom_color');
        if ($req->has('font_color')) $category->font_color = $req->input('font_color');
        $category->is_dinein = $req->input('is_dinein');
        $category->is_delivery = $req->input('is_delivery');
        $category->is_collection = $req->input('is_collection');
        $category->is_banquet = $req->input('is_banquet');

        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_EPOS_CATEGORY);
            $category->image = $fileName;
        }

        $category->save();

        return $category;
    }


    /**
     * Delete category by id.
     * DELETE /epos/categories/{id}
     */
    public function delete(Request $req, $id)
    {
        $category = EposCategory::find($id);
        if (empty($category)) Utils::error('EposCategory not found!', 404);
        $category->delete();
        return $category;
    }

    /**
     * Enable category.
     * POST /epos/categories/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $category = EposCategory::find($id);
        if (empty($category)) Utils::error('EposCategory not found!', 404);
        $category->disabled = 0;
        $category->save();
        return $category;
    }

    /**
     * Disable category.
     * POST /epos/categories/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $category = EposCategory::find($id);
        if (empty($category)) Utils::error('EposCategory not found!', 404);
        $category->disabled = 1;
        $category->save();
        return $category;
    }


    public function category_bulk(Request $req)
    {
        if ($req->has('categories') && !is_array($req->input('categories'))) {
            $req->merge(['categories' => json_decode($req->input('categories'))]);
        }
        $this->validate($req, [
            'categories' => 'required|array'
        ],);
        $categories = ($req->input('categories'));
        $count = 1;
        foreach ($categories as $category) {
            $categoryObj = EposCategory::find($category->id);
            $categoryObj->sequence = $count;
            $categoryObj->save();
            $count++;
        }
        return $categories;
    }
}
