<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposDeliveryCharge;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsEpos\EposSetting;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposDeliveryChargeController extends Controller
{
    /**
     * List deliveryCharges.
     * GET /epos/delivery-charges
     */
    public function index(Request $req)
    {
        $deliveryCharges = EposDeliveryCharge::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $deliveryCharges->whereRaw('(distance ?  )', ["%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $deliveryCharges = $deliveryCharges->get();
        } else {
            $deliveryCharges = $deliveryCharges->paginate($req->query('per_page', 20));
        }

        if ($deliveryCharges->isEmpty()) Utils::error('No deliveryCharges found!', 404);


        return ($deliveryCharges);
    }

    /**
     * Create deliveryCharge.
     * POST /epos/delivery-charges
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'distance' => 'required|numeric',
            'charge' => 'required|numeric',
        ]);


        $latestLesserDistance = EposDeliveryCharge::where('distance', '<=', $req->input('distance'))->orderByRaw('distance DESC')->first();
        if (!empty($latestLesserDistance)) {
            if ($latestLesserDistance->charge > $req->input('charge')) {
                Utils::error("Charge should be greater than latest small distance", 400);
            }
        } else {
            $latestLesserCharge = EposDeliveryCharge::where('charge', '<', $req->input('charge'))->orderByRaw('charge DESC')->first();
            if (!empty($latestLesserCharge)) {
                if ($latestLesserCharge->distance < $req->input('distance')) {
                    Utils::error("Distance should be greater than latest low charge", 400);
                }
            }
        }

        // create deliveryCharge
        $deliveryCharge = new EposDeliveryCharge;
        $deliveryCharge->fill($req->all());

        $deliveryCharge->save();

        return $deliveryCharge;
    }

    /**
     * Show deliveryCharge.
     * GET /epos/delivery-charges/{id}
     */
    public function show(Request $req, $id)
    {
        $deliveryCharge = EposDeliveryCharge::find($id);
        if (empty($deliveryCharge)) Utils::error('EposDeliveryCharge not found!', 404);
        return $deliveryCharge;
    }

    /**
     * Update deliveryCharge.
     * POST /epos/delivery-charges/{id}
     */
    public function update(Request $req, $id)
    {
        $deliveryCharge = EposDeliveryCharge::find($id);
        if (empty($deliveryCharge)) Utils::error('EposDeliveryCharge not found!', 404);

        $this->validate($req, [
            'distance' => 'required|numeric',
            'charge' => 'required|numeric',
        ]);


        $latestLesserDistance = EposDeliveryCharge::where('distance', '<=', $req->input('distance'))->orderByRaw('distance DESC')->first();
        if (!empty($latestLesserDistance)) {
            if ($latestLesserDistance->charge > $req->input('charge')) {
                Utils::error("Charge should be greater than latest small distance", 400);
            }
        } else {
            $latestLesserCharge = EposDeliveryCharge::where('charge', '<', $req->input('charge'))->orderByRaw('charge DESC')->first();
            if (!empty($latestLesserCharge)) {
                if ($latestLesserCharge->distance < $req->input('distance')) {
                    Utils::error("Distance should be greater than latest low charge", 400);
                }
            }
        }

        // update deliveryCharge
        if ($req->has('distance')) $deliveryCharge->distance = $req->input('distance');
        if ($req->has('charge')) $deliveryCharge->charge = $req->input('charge');

        $deliveryCharge->save();

        return $deliveryCharge;
    }


    /**
     * Delete deliveryCharge by id.
     * DELETE /epos/delivery-charges/{id}
     */
    public function delete(Request $req, $id)
    {
        $deliveryCharge = EposDeliveryCharge::find($id);
        if (empty($deliveryCharge)) Utils::error('EposDeliveryCharge not found!', 404);
        $deliveryCharge->delete();
        return $deliveryCharge;
    }


    /**
     * Enable deliveryCharge.
     * POST /epos/deliveryCharges/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $deliveryCharge = EposDeliveryCharge::find($id);
        if (empty($deliveryCharge)) Utils::error('EposDeliveryCharge not found!', 404);
        $deliveryCharge->disabled = 0;
        $deliveryCharge->save();
        return $deliveryCharge;
    }

    /**
     * Disable deliveryCharge.
     * POST /epos/deliveryCharges/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $deliveryCharge = EposDeliveryCharge::find($id);
        if (empty($deliveryCharge)) Utils::error('EposDeliveryCharge not found!', 404);
        $deliveryCharge->disabled = 1;
        $deliveryCharge->save();
        return $deliveryCharge;
    }

    /**
     * Calculate deliveryCharge.
     * POST /epos/deliveryCharges/calculate
     */
    public function delivery_charge_calculation(Request $req)
    {
        $this->validate($req, [
            'latitude' => 'string',
            'longitude' => 'string',
            'distance' => 'string',
        ]);

        if ($req->has('latitude') && $req->has('longitude')) {
            $longitudeRest = (new EposSettingController())->findByKey('address_longitude');
            $latitudeRest = (new EposSettingController())->findByKey('address_latitude');
            if (empty($longitudeRest) || empty($latitudeRest)) Utils::error('latitude or longitude settings not found!', 404);

            $longitudeRest = (float)$longitudeRest->value;
            $latitudeRest = (float)$latitudeRest->value;
            if ($longitudeRest == '' || $latitudeRest == '') Utils::error('latitude or longitude settings not found!', 404);

            $longitudeUser = (float)$req->input('longitude');
            $latitudeUser = (float)$req->input('latitude');

            $theta = $longitudeRest - $longitudeUser;
            $dist = sin(deg2rad((float)$latitudeRest)) * sin(deg2rad((float)$latitudeUser)) +  cos(deg2rad((float)$latitudeRest)) * cos(deg2rad((float)$latitudeUser)) * cos(deg2rad((float)$theta));
            $dist = acos($dist);
            $dist = rad2deg($dist);
            $miles = number_format($dist * 60 * 1.1515, 2);
        } else {
            $miles = $req->input('distance');
        }

        $deliveryCharges = EposDeliveryCharge::orderByRaw('distance DESC')->where('distance', '<=', $miles)->where('disabled', '0')->limit('1')->first();
        if (empty($deliveryCharges)) Utils::error('EposDeliveryCharge not found!', 404);
        $deliveryCharges->current_miles = $miles;
        return $deliveryCharges;
    }
}
