<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\Auth;
use App\ModelsEpos\EposDiscount;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposDiscountController extends Controller
{
    /**
     * List discounts.
     * GET /api/discounts
     */
    public function index(Request $req)
    {
        // $discounts = EposDiscount::orderByRaw('created_at DESC');
        $discounts = EposDiscount::selectRaw('discounts.*,order_types.type as order_type')->leftJoin('order_types', 'order_types.id', '=', 'order_type_id')->orderByRaw('discounts.created_at DESC');

        if ($query = $req->query('order_type_id', false)) {
            $discounts->where('order_type_id', $query);
        }
        if ($query = $req->query('week_day', false)) {
            $discounts->where('week_day', $query);
        }
        if ($query = $req->query('discount', false)) {
            $discounts->where('discount', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $discounts = $discounts->get();
        } else {
            $discounts = $discounts->paginate($req->query('per_page', 20));
        }

        if ($discounts->isEmpty()) Utils::error('No discounts found!', 404);


        return ($discounts);
    }

    /**
     * Create discount.
     * POST /api/discounts
     */
    public function create(Request $req)
    {
        if (Auth::role() == 'epos_user')
            $req->merge(['business_id' => $req->header('business_id')]);
        $this->validate($req, [
            'order_type_id' => 'required|string',
            'week_day' => 'required|string',
            'discount' => 'required|numeric|gt:0',
            'discount_type' => 'required|in:enum,flat,percentage',
            'minimum_value' => 'required|numeric',
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date'
        ]);

        // create discount
        $discount = new EposDiscount;
        $discount->fill($req->all());

        $discount->save();
        $discount->refresh();

        return $discount;
    }

    /**
     * Show discount.
     * GET /api/discounts/{id}
     */
    public function show(Request $req, $id)
    {
        $discount = EposDiscount::find($id);
        if (empty($discount)) Utils::error('EposDiscount not found!', 404);
        return $discount;
    }
    /**
     * validate discount.
     * GET /api/discounts/validate/{discount_id}
     */
    public function validate_discount(Request $req, $discount_id)
    {
        $discount = EposDiscount::where('discount_id', $discount_id)->where('business_id', $req->header('business_id'))->orderBy('created_at', 'DESC')->first();
        if (empty($discount)) Utils::error('EposDiscount not found!', 404);
        return $discount;
    }
    /**
     * Update discount.
     * POST /api/discounts/{id}
     */
    public function update(Request $req, $id)
    {
        $discount = EposDiscount::find($id);
        if (empty($discount)) Utils::error('EposDiscount not found!', 404);
        if (Auth::role() == 'epos_user')
            $req->merge(['business_id' => $req->header('business_id')]);
        $this->validate($req, [
            'order_type_id' => 'required|string',
            'week_day' => 'required|string',
            'discount' => 'required|numeric|gt:0',
            'discount_type' => 'required|in:enum,flat,percentage',
            'minimum_value' => 'required|numeric',
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date',
        ]);

        // update discount
        if ($req->has('order_type_id')) $discount->order_type_id = $req->input('order_type_id');
        if ($req->has('week_day')) $discount->week_day = $req->input('week_day');
        if ($req->has('discount')) $discount->discount = $req->input('discount');
        if ($req->has('discount_type')) $discount->discount_type = $req->input('discount_type');
        $discount->minimum_value = $req->input('minimum_value');
        if ($req->has('start_date')) $discount->start_date = $req->input('start_date');
        if ($req->has('end_date')) $discount->end_date = $req->input('end_date');

        $discount->save();
        $discount->refresh();

        return $discount;
    }


    /**
     * Delete discount by id.
     * DELETE /api/discounts/{id}
     */
    public function delete(Request $req, $id)
    {
        $discount = EposDiscount::find($id);
        if (empty($discount)) Utils::error('EposDiscount not found!', 404);
        $discount->delete();
        return $discount;
    }
}
