<?php

namespace App\Http\Controllers\Epos;

use App\ModelsEpos\EposIngredient;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsEpos\EposProductIngredient;
use Illuminate\Http\Request;

class EposIngredientController extends Controller
{
    /**
     * List ingredients.
     * GET /epos/ingredients
     */
    public function index(Request $req)
    {
        $ingredients = EposIngredient::orderByRaw('sequence ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $ingredients->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('parent_id', false)) {
            $ingredients->where('parent_id', $query);
        }
        if ($query = $req->query('disabled', false)) {
            $ingredients->where('disabled', $query);
        }
        if ($query = $req->query('product_id', false)) {
            if ($requirement = $req->query('requirement', false)) {
                if ($requirement == 'with') {
                    $ingredients->whereRaw('(id NOT IN (SELECT ingredient_id from product_ingredients where product_id = ? AND product_ingredients.with=1))', [$req->input('product_id')]);
                }
                if ($requirement == 'without') {
                    $ingredients->whereRaw('(id NOT IN (SELECT ingredient_id from product_ingredients where product_id = ? AND product_ingredients.without=1))', [$req->input('product_id')]);
                }
            } else {
                $ingredients->whereRaw('(id NOT IN (SELECT ingredient_id from product_ingredients where product_id = ?))', [$req->input('product_id')]);
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $ingredients = $ingredients->get();
        } else {
            $ingredients = $ingredients->paginate($req->query('per_page', 20));
        }

        if ($ingredients->isEmpty()) Utils::error('No ingredients found!', 404);


        return ($ingredients);
    }

    /**
     * Create ingredient.
     * POST /epos/ingredients
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'price' => 'numeric',
            'price_without' => 'numeric',
            'sequence' => 'numeric',
        ]);

        // create ingredient
        $ingredient = new EposIngredient;
        $ingredient->fill($req->all());
        $ingredient->price = number_format($req->input('price'), 2);
        $ingredient->price_without = number_format($req->input('price_without'), 2);
        $ingredient->name = strtoupper($req->input('name'));
        $ingredient->save();

        // Sequence add
        $ingredient->sequence = $ingredient->id;
        $ingredient->save();

        return $ingredient;
    }

    /**
     * Show ingredient.
     * GET /epos/ingredients/{id}
     */
    public function show(Request $req, $id)
    {
        $ingredient = EposIngredient::find($id);
        if (empty($ingredient)) Utils::error('EposIngredient not found!', 404);
        return $ingredient;
    }

    /**
     * Update ingredient.
     * POST /epos/ingredients/{id}
     */
    public function update(Request $req, $id)
    {
        $ingredient = EposIngredient::find($id);
        if (empty($ingredient)) Utils::error('EposIngredient not found!', 404);

        $this->validate($req, [
            'name' => 'string|max:255',
            'price' => 'numeric',
            'price_without' => 'numeric',
            'sequence' => 'numeric',
        ]);

        // update ingredient
        if ($req->has('name')) $ingredient->name = strtoupper($req->input('name'));
        $ingredient->price = number_format($req->input('price'), 2);
        $ingredient->price_without = number_format($req->input('price_without'), 2);
        $ingredient->sequence = $req->input('sequence');

        // Epos product ingredient sequence assign
        if ($req->has('sequence')) EposProductIngredient::where('ingredient_id', $ingredient->id)->update(['sequence' => $ingredient->sequence]);

        $ingredient->save();

        return $ingredient;
    }


    /**
     * Delete ingredient by id.
     * DELETE /epos/ingredients/{id}
     */
    public function delete(Request $req, $id)
    {
        $ingredient = EposIngredient::find($id);
        if (empty($ingredient)) Utils::error('EposIngredient not found!', 404);
        $ingredient->delete();
        return $ingredient;
    }
    /**
     * Enable ingredient.
     * POST /epos/ingredients/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $ingredient = EposIngredient::find($id);
        if (empty($ingredient)) Utils::error('EposIngredient not found!', 404);
        $ingredient->disabled = 0;
        $ingredient->save();

        $productIngredients = EposProductIngredient::where('ingredient_id', $id)->get();
        if (!empty($productIngredients)) {
            foreach ($productIngredients as $productingredient) {
                $productingredientObj = EposProductIngredient::find($productingredient->id);
                $productingredientObj->disabled = 0;
                $productingredientObj->save();
            }
        }

        return $ingredient;
    }

    /**
     * Disable ingredient.
     * POST /epos/ingredients/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $ingredient = EposIngredient::find($id);
        if (empty($ingredient)) Utils::error('EposIngredient not found!', 404);
        $ingredient->disabled = 1;
        $ingredient->save();

        $productIngredients = EposProductIngredient::where('ingredient_id', $id)->get();
        if (!empty($productIngredients)) {
            foreach ($productIngredients as $productingredient) {
                $productingredientObj = EposProductIngredient::find($productingredient->id);
                $productingredientObj->disabled = 1;
                $productingredientObj->save();
            }
        }

        return $ingredient;
    }

    public function ingredient_bulk(Request $req)
    {
        if ($req->has('ingredients') && !is_array($req->input('ingredients'))) {
            $req->merge(['ingredients' => json_decode($req->input('ingredients'))]);
        }
        $this->validate($req, [
            'ingredients' => 'required|array'
        ],);
        $ingredients = ($req->input('ingredients'));
        $count = 1;
        foreach ($ingredients as $ingredient) {
            $ingredientObj = EposIngredient::find($ingredient->id);
            $ingredientObj->sequence = $count;
            $ingredientObj->save();
            $count++;
        }
        return $ingredientObj;
    }
}
