<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\Auth;
use App\ModelsEpos\EposOrderPayment;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ModelsEpos\EposCustomer;
use App\ModelsEpos\EposLoyaltyPoint;
use App\ModelsEpos\EposOrder;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposOrderPaymentController extends Controller
{
    /**
     * List order payments.
     * GET /epos/order payments
     */
    public function index(Request $req)
    {
        $order_payments = EposOrderPayment::selectRaw('order_payments.*,payment_methods.name as payment_method_name')->leftJoin('payment_methods', 'payment_methods.id', '=', 'payment_method_id')->orderByRaw('created_at DESC');

        if ($query = $req->query('order_id', false)) {
            $order_payments->where('order_id', $query);
        }
        if ($query = $req->query('order_split_id', false)) {
            $order_payments->where('order_split_id', $query);
        }

        if ($req->query('nopaginate', false) == '1') {
            $order_payments = $order_payments->get();
        } else {
            $order_payments = $order_payments->paginate($req->query('per_page', 20));
        }

        if ($order_payments->isEmpty()) Utils::error('No order payments found!', 404);


        return ($order_payments);
    }

    /**
     * Create table.
     * POST /epos/order_payments
     */
    public function create(Request $req)
    {

        if (!is_array($req->order_items))
            $req->merge(['order_items' => (json_decode($req->order_items, true))]);

        $this->validate($req, [
            'order_split_id' => 'nullable|exists:epos_connection.order_splits,id',
            'order_id' => 'required',
            'payment_method_id' => 'required|exists:epos_connection.payment_methods,id',
            'amount' => 'numeric',
            'cheque_number' => 'string',
            'cc_number' => 'string',
            'txn_id' => 'string',
            'mode' => 'string',
        ]);
        $order = EposOrder::find($req->input('order_id'));
        if (empty($order)) {
            Utils::error("Order not found", 404);
        }
        $totalPaidAmount = (EposOrderPayment::where('order_id', $req->input('order_id'))->sum('amount'));
        $toBePaidAmount = ($req->input('amount'));


        // create order_payment
        $order_payment = new EposOrderPayment;
        $order_payment->fill($req->all());
        $order_payment->save();

        $order->total_paid = $totalPaidAmount + $toBePaidAmount;

        // if ($order->total_paid >= $order->total) {
        //     $order->order_status_id = 4; // Order Paid
        // } else {
        //     $order->order_status_id = 7; // Part Payment Received
        // }

        $order->save();
        if ($order->total_paid >= $order->total) {

            $loyalty_points = EposLoyaltyPoint::where('order_type_id', $order->order_type_id)->first();
            if (!empty($loyalty_points)) {
                $membership_points = ($order->sub_total * $loyalty_points['earn_point']) / $loyalty_points['earn_amount'];
                $customer = EposCustomer::find($order->customer_id);
                $customer->membership_points = $customer->membership_points + $membership_points;
                $customer->save();
            }
        }

        if ($req->input('payment_method_id') == 6) {
            $customer = EposCustomer::find($order->customer_id);
            $customer->membership_points = $customer->membership_points - $req->input('spent_loyalty_point');
            $customer->save();
        }

        $order_payment = EposOrderPayment::selectRaw('order_payments.*,payment_methods.name as payment_method_name')->leftJoin('payment_methods', 'payment_methods.id', '=', 'payment_method_id')->find($order_payment->id);
        return $order_payment;
    }

    /**
     * Show table.
     * GET /epos/order_payments/{id}
     */
    public function show(Request $req, $id)
    {
        $order_payment = EposOrderPayment::selectRaw('order_payments.*,payment_methods.name as payment_method_name')->leftJoin('payment_methods', 'payment_methods.id', '=', 'payment_method_id')->find($id);
        if (empty($order_payment)) Utils::error('EposOrderPayment not found!', 404);
        return $order_payment;
    }

    /**
     * Update order.
     * POST /epos/order_payments/{id}
     */
    public function update(Request $req, $id)
    {
        $order_payment = EposOrderPayment::find($id);
        if (empty($order_payment)) Utils::error('EposOrderPayment not found!', 404);

        $this->validate($req, [
            'order_id' => 'required|exists:epos_connection.orders,id',
            'order_split_id' => 'nullable|exists:epos_connection.order_splits,id',
            'payment_method_id' => 'required|exists:epos_connection.payment_methods,id',
            'amount' => 'required|numeric|gt:0',
            'cheque_number' => 'string',
            'cc_number' => 'string',
            'txn_id' => 'string',
            'mode' => 'string',
        ]);

        // update order_payment
        if ($req->has('order_id')) $order_payment->order_id = $req->input('order_id');
        if ($req->has('order_split_id')) $order_payment->order_split_id = $req->input('order_split_id');
        if ($req->has('payment_method_id')) $order_payment->payment_method_id = $req->input('payment_method_id');
        if ($req->has('voucher_id')) $order_payment->voucher_id = $req->input('voucher_id');
        if ($req->has('updater_id')) $order_payment->table_id = Auth::id();
        if ($req->has('amount')) $order_payment->amount = $req->input('amount');
        if ($req->has('cheque_number')) $order_payment->cheque_number = $req->input('cheque_number');
        if ($req->has('cc_number')) $order_payment->cc_number = $req->input('cc_number');
        if ($req->has('txn_id')) $order_payment->txn_id = $req->input('txn_id');
        if ($req->has('mode')) $order_payment->mode = $req->input('mode');

        $order_payment->save();

        $order_payment = EposOrderPayment::selectRaw('order_payments.*,payment_methods.name as payment_method_name')->leftJoin('payment_methods', 'payment_methods.id', '=', 'payment_method_id')->find($order_payment->id);

        return $order_payment;
    }


    /**
     * Delete order by id.
     * DELETE /epos/order_payments/{id}
     */
    public function delete(Request $req, $id)
    {
        $order_payment = EposOrderPayment::find($id);
        if (empty($order_payment)) Utils::error('EposOrderPayment not found!', 404);
        $order_payment->delete();
        return $order_payment;
    }

    public function payment_bulk(Request $req)
    {
        if ($req->has('payments') && !is_array($req->input('payments'))) {
            $req->merge(['payments' => json_decode($req->input('payments'))]);
        }
        $this->validate($req, [
            'payments' => 'required|array'
        ]);

        $payments = ($req->input('payments'));
        $orderPayments = [];
        foreach ($payments as $payment) {
            $order = EposOrder::find($payment->order_id);
            if (empty($order)) {
                Utils::error("Order not found", 404);
            }

            $totalPaidAmount = (EposOrderPayment::where('order_id', $payment->order_id)->sum('amount'));
            $toBePaidAmount = ($payment->amount);
            if (empty($payment->id)) {
                // create order_payment
                $orderPayment = new EposOrderPayment((array)$payment);
                $orderPayment->save();
            } else {
                // update order_payment
                $orderPayment = EposOrderPayment::find($payment->id);
                if (empty($orderPayment)) {
                    $orderPayment = new EposOrderPayment();
                }
                if (isset($payment->order_id)) $orderPayment->order_id = $payment->order_id;
                if (isset($payment->order_split_id)) $orderPayment->order_split_id = $payment->order_split_id;
                if (isset($payment->payment_method_id)) $orderPayment->payment_method_id = $payment->payment_method_id;
                if (isset($payment->voucher_id)) $orderPayment->voucher_id = $payment->voucher_id;
                if (isset($payment->updater_id)) $orderPayment->updater_id = $payment->updater_id;
                if (isset($payment->amount)) $orderPayment->amount = $payment->amount;
                if (isset($payment->cheque_number)) $orderPayment->cheque_number = $payment->cheque_number;
                if (isset($payment->cc_number)) $orderPayment->cc_number = $payment->cc_number;
                if (isset($payment->txn_id)) $orderPayment->txn_id = $payment->txn_id;
                if (isset($payment->mode)) $orderPayment->mode = $payment->mode;
                if (isset($payment->unique_id)) $orderPayment->unique_id = $payment->unique_id;
                $orderPayment->save();
            }

            $orderPayment->total_paid = $totalPaidAmount + $toBePaidAmount;

            $orderPayments[] = $orderPayment;
        }
        return $orderPayments;
    }
}
