<?php

namespace App\Http\Controllers\Epos;

use App\Business;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Passbook;
use App\PayoutRequest;
use App\User;
use Carbon\Carbon;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposPayoutRequestController extends Controller
{
    /**
     * List payoutRequests.
     * GET /api/payout-requests
     */
    public function index(Request $req)
    {
        $payoutRequests = PayoutRequest::select("payout_requests.*", "businesses.name as business_name", "users.first_name", "users.last_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->leftJoin('users', 'users.id', '=', 'user_id')->orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $payoutRequests->whereRaw('(businesses.name LIKE ? OR first_name LIKE ? OR last_name LIKE ? OR amount LIKE ? OR note LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('status', false)) {
            $payoutRequests->where('status', $query);
        }
        if ($query = $req->query('business_id', false)) {
            $payoutRequests->where('business_id', $query);
        }
        if ($query = $req->query('user_id', false)) {
            $payoutRequests->where('user_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $payoutRequests = $payoutRequests->get();
        } else {
            $payoutRequests = $payoutRequests->paginate($req->query('per_page', 20));
        }

        if ($payoutRequests->isEmpty()) Utils::error('No payoutRequests found!', 404);


        return ($payoutRequests);
    }

    /**
     * Create payoutRequest.
     * POST /api/payout-requests
     */
    public function create(Request $req)
    {
        $req->merge(['business_id' => Auth::$business_id]);

        $this->validate($req, [
            'amount' => 'required|numeric:gt,0',
            'note' => 'string',
            'status' => 'in:Pending,Accepted,Rejected',
            'business_id' => 'required_without:user_id|exists:businesses,id',
            'user_id' => 'required_without:business_id|exists:users,id',
        ]);

        // create payoutRequest
        $payoutRequest = new PayoutRequest;
        $payoutRequest->fill($req->all());
        if ($req->has('business_id')) {
            $business = Business::find($req->input('business_id'));
            if (!empty($business) && $business->balance < $payoutRequest->amount) {
                Utils::error("Unsufficeint balance", 400);
            }
        }
        if ($req->has('user_id')) {
            $user = User::find($req->input('user_id'));
            if (!empty($user) && $user->balance < $payoutRequest->amount) {
                Utils::error("Unsufficeint balance", 400);
            }
        }
        $payoutRequest->save();

        return $payoutRequest;
    }

    /**
     * Show payoutRequest.
     * GET /api/payout-requests/{id}
     */
    public function show(Request $req, $id)
    {
        $payoutRequest = PayoutRequest::find($id);
        if (empty($payoutRequest)) Utils::error('PayoutRequest not found!', 404);
        return $payoutRequest;
    }

    /**
     * Update payoutRequest.
     * POST /api/payout-requests/{id}
     */
    public function update(Request $req, $id)
    {
        $payoutRequest = PayoutRequest::find($id);
        $dbPayoutRequest = $payoutRequest->clone();
        if (empty($payoutRequest)) Utils::error('PayoutRequest not found!', 404);

        $req->merge(['business_id' => Auth::$business_id]);

        $this->validate($req, [
            'amount' => 'required|numeric:gt,0',
            'note' => 'string',
            'rejection_note' => 'string',
            'status' => 'in:Pending,Accepted,Rejected',
            'business_id' => 'required_without:user_id|exists:businesses,id',
            'user_id' => 'required_without:business_id|exists:users,id',
        ]);

        // update payoutRequest
        if ($req->has('amount')) $payoutRequest->amount = $req->input('amount');
        if ($req->has('note')) $payoutRequest->note = $req->input('note');
        if ($req->has('status')) $payoutRequest->status = $req->input('status');
        if ($req->has('business_id')) $payoutRequest->business_id = $req->input('business_id');
        if ($req->has('user_id')) $payoutRequest->user_id = $req->input('user_id');
        if ($req->has('rejection_note')) $payoutRequest->rejection_note = $req->input('rejection_note');
        if ($req->has('business_id')) {
            $business = Business::find($req->input('business_id'));
            if (!empty($business) && $business->balance < $payoutRequest->amount) {
                Utils::error("Unsufficeint balance", 400);
            }
        }
        if ($req->has('user_id')) {
            $user = User::find($req->input('user_id'));
            if (!empty($user) && $user->balance < $payoutRequest->amount) {
                Utils::error("Unsufficeint balance", 400);
            }
        }
        $passbook = Passbook::where('entity', 'payout_request')->where('entity_id', $payoutRequest->id);
        if ($req->has('user_id')) {
            $passbook->where('user_id', $payoutRequest->user_id);
        }
        if ($req->has('business_id')) {
            $passbook->where('business_id', $payoutRequest->business_id);
        }
        $passbook = $passbook->first();
        if ($payoutRequest->status == 'Accepted') {
            if (empty($passbook)) {
                $passbook = new Passbook;
            }
            $passbook->entity = "payout_request";
            $passbook->entity_id = $payoutRequest->id;
            $passbook->business_id = $payoutRequest->business_id;
            $passbook->user_id = $payoutRequest->user_id;
            $passbook->date = Carbon::today()->toDateString();
            $passbook->from_date = Carbon::today()->toDateString();
            $passbook->to_date = Carbon::today()->toDateString();
            $passbook->amount = -1 * $payoutRequest->amount;
            $passbook->description = "Withdrawal request accepted dated on " . Carbon::parse($payoutRequest->created_at)->toDateString();
            $passbook->save();
        } else {
            if (!empty($passbook))
                $passbook->delete();
        }
        $payoutRequest->save();

        return $payoutRequest;
    }


    /**
     * Delete payoutRequest by id.
     * DELETE /api/payout-requests/{id}
     */
    public function delete(Request $req, $id)
    {
        $payoutRequest = PayoutRequest::find($id);
        if (empty($payoutRequest)) Utils::error('PayoutRequest not found!', 404);
        $payoutRequest->delete();
        return $payoutRequest;
    }
}
