<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\ImageHelper;
use App\ModelsEpos\EposProduct;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use Illuminate\Http\Request;

class EposProductController extends Controller
{
    /**
     * List products.
     * GET /epos/products
     */
    public function index(Request $req)
    {
        $products = EposProduct::selectRaw("products.*,categories.name as category_name,preparation_locations.name as preparation_location_name,(SELECT COUNT(*) FROM product_addons WHERE product_addons.product_id = products.id AND product_addons.disabled=0) as total_addons")->leftJoin('categories', 'categories.id', '=', 'products.category_id')->leftJoin('preparation_locations', 'preparation_locations.id', '=', 'products.preparation_location_id')->orderByRaw('products.sequence ASC, products.created_at DESC');

        $products->where('products.misc', 0);

        if ($query = $req->query('query', false)) {
            $products->whereRaw('(products.name LIKE ?  )', ["%$query%"]);
        }

        if ($barcode = $req->query('barcode', false)) {
            $products->whereRaw('(products.barcode LIKE ?  )', ["%$barcode%"]);
        }
        if ($query = $req->query('parent_id', false)) {
            $products->where('products.parent_id', $query);
        }
        if ($req->query('disabled') == '0' || $req->query('disabled') == '1') {
            $products->where('products.disabled', $req->query('disabled'));
        }
        if ($query = $req->query('is_dinein', false)) {
            $products->where('products.is_dinein', 1);
        }
        if ($query = $req->query('is_delivery', false)) {
            $products->where('products.is_delivery', 1);
        }
        if ($query = $req->query('is_collection', false)) {
            $products->where('products.is_collection', 1);
        }
        if ($query = $req->query('is_banquet', false)) {
            $products->where('products.is_banquet', 1);
        }
        if ($query = $req->query('category_id', false)) {
            $products->where('products.category_id', $query);
        }
        if ($query = $req->query('preparation_location_id', false)) {
            $products->where('preparation_location_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $products = $products->get();
        } else {
            $products = $products->paginate($req->query('per_page', 20));
        }

        if ($products->isEmpty()) Utils::error('No products found!', 404);


        return ($products);
    }

    /**
     * Create product.
     * POST /epos/products
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'short_name' => 'string',
            'description' => 'string',
            'image' => 'image',
            'parent_id' => 'exists:epos_connection.products,id',
            'category_id' => 'exists:epos_connection.categories,id',
            'top_color' => 'string|max:9|min:7',
            'font_color' => 'string|max:9|min:7',
            'price' => 'numeric',
            'takeaway_price' => 'numeric',
            'web_price' => 'numeric',
            'delivery_price' => 'numeric',
            'waiting_price' => 'numeric',
            'is_dinein' => 'required|boolean',
            'is_delivery' => 'required|boolean',
            'is_collection' => 'required|boolean',
            'is_banquet' => 'required|boolean',
            'favourite' => 'required|boolean',
            'bar_favourite' => 'required|boolean',
            'tax_included' => 'required|boolean',
            'taxable' => 'required|boolean',
            'auto_modify' => 'required|boolean',
            'auto_addon' => 'required|boolean',
            'stockable' => 'required|boolean',
            'misc' => 'required|boolean',
            'barcode' => 'string|max:255',
            'sequence' => 'numeric',
        ]);

        // create product
        $product = new EposProduct;
        $product->fill($req->all());
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_EPOS_PRODUCT);
            $product->image = $fileName;
        }
        if ($req->has('name')) $product->name = (new ExtraController())->convertToTitleCase($req->input('name'));
        $product->save();

        // Sequence add
        $product->sequence = $product->id;
        $product->save();

        return $product;
    }

    /**
     * Show product.
     * GET /epos/products/{id}
     */
    public function show(Request $req, $id)
    {
        $product = EposProduct::selectRaw("products.*,categories.name as category_name,preparation_locations.name as preparation_location_name,(SELECT COUNT(*) FROM product_addons WHERE product_addons.product_id = products.id AND product_addons.disabled=0) as total_addons")->leftJoin('categories', 'categories.id', '=', 'products.category_id')->leftJoin('preparation_locations', 'preparation_locations.id', '=', 'products.preparation_location_id')->find($id);
        if (empty($product)) Utils::error('EposProduct not found!', 404);
        return $product;
    }

    /**
     * Update product.
     * POST /epos/products/{id}
     */
    public function update(Request $req, $id)
    {
        $product = EposProduct::find($id);
        if (empty($product)) Utils::error('EposProduct not found!', 404);

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'short_name' => 'string',
            'description' => 'string',
            'image' => 'image',
            'parent_id' => 'exists:epos_connection.products,id',
            'category_id' => 'exists:epos_connection.categories,id',
            'top_color' => 'string|max:9|min:7',
            'font_color' => 'string|max:9|min:7',
            'price' => 'numeric',
            'takeaway_price' => 'numeric',
            'web_price' => 'numeric',
            'delivery_price' => 'numeric',
            'waiting_price' => 'numeric',
            'is_dinein' => 'required|boolean',
            'is_delivery' => 'required|boolean',
            'is_collection' => 'required|boolean',
            'is_banquet' => 'required|boolean',
            'favourite' => 'required|boolean',
            'bar_favourite' => 'required|boolean',
            'tax_included' => 'required|boolean',
            'taxable' => 'required|boolean',
            'auto_modify' => 'required|boolean',
            'auto_addon' => 'required|boolean',
            'stockable' => 'required|boolean',
            'misc' => 'required|boolean',
            'barcode' => 'string|max:255',
            'sequence' => 'numeric',
        ]);

        // update product
        if ($req->has('name')) $product->name = (new ExtraController())->convertToTitleCase($req->input('name'));
        if ($req->has('short_name')) $product->short_name = $req->input('short_name');
        if ($req->has('description')) $product->description = $req->input('description');
        $product->parent_id = $req->input('parent_id');
        if ($req->has('category_id')) $product->category_id = $req->input('category_id');
        if ($req->has('preparation_location_id')) $product->preparation_location_id = $req->input('preparation_location_id');
        if ($req->has('collection_preparation_location_id')) $product->collection_preparation_location_id = $req->input('collection_preparation_location_id');
        if ($req->has('delivery_preparation_location_id')) $product->delivery_preparation_location_id = $req->input('delivery_preparation_location_id');
        if ($req->has('banquet_preparation_location_id')) $product->banquet_preparation_location_id = $req->input('banquet_preparation_location_id');
        if ($req->has('top_color')) $product->top_color = $req->input('top_color');
        if ($req->has('font_color')) $product->font_color = $req->input('font_color');
        if ($req->has('barcode')) $product->barcode = $req->input('barcode');
        $product->price = $req->input('price');
        $product->takeaway_price = $req->input('takeaway_price');
        $product->web_price = $req->input('web_price');
        $product->delivery_price = $req->input('delivery_price');
        $product->waiting_price = $req->input('waiting_price');
        $product->is_dinein = $req->input('is_dinein');
        $product->is_delivery = $req->input('is_delivery');
        $product->is_collection = $req->input('is_collection');
        $product->is_banquet = $req->input('is_banquet');
        $product->favourite = $req->input('favourite');
        $product->bar_favourite = $req->input('bar_favourite');
        $product->tax_included = $req->input('tax_included');
        $product->taxable = $req->input('taxable');
        $product->auto_modify = $req->input('auto_modify');
        $product->auto_addon = $req->input('auto_addon');
        $product->stockable = $req->input('stockable');
        $product->misc = $req->input('misc');
        $product->sequence = $req->input('sequence');
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_EPOS_PRODUCT);
            $product->image = $fileName;
        }
        $product->save();

        return $product;
    }


    /**
     * Delete product by id.
     * DELETE /epos/products/{id}
     */
    public function delete(Request $req, $id)
    {
        $product = EposProduct::find($id);
        if (empty($product)) Utils::error('EposProduct not found!', 404);
        $product->delete();
        return $product;
    }

    /**
     * Enable product.
     * POST /epos/products/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $product = EposProduct::find($id);
        if (empty($product)) Utils::error('EposProduct not found!', 404);
        $product->disabled = 0;
        $product->save();
        return $product;
    }

    /**
     * Disable product.
     * POST /epos/products/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $product = EposProduct::find($id);
        if (empty($product)) Utils::error('EposProduct not found!', 404);
        $product->disabled = 1;
        $product->save();
        return $product;
    }


    public function product_bulk(Request $req)
    {
        if ($req->has('products') && !is_array($req->input('products'))) {
            $req->merge(['products' => json_decode($req->input('products'))]);
        }
        $this->validate($req, [
            'products' => 'required|array'
        ],);
        $products = ($req->input('products'));
        $count = 1;
        foreach ($products as $product) {
            $productObj = EposProduct::find($product->id);
            $productObj->sequence = $count;
            $productObj->save();
            $count++;
        }
        return $products;
    }
}
