<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\ImageHelper;
use App\ModelsEpos\EposProductIngredient;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class EposProductIngredientController extends Controller
{
    /**
     * List productIngredients.
     * GET /epos/product-ingredients/
     */
    public function index(Request $req)
    {
        $productIngredients = EposProductIngredient::select('product_ingredients.*', 'ingredients.name as ingredient_name', 'ingredients.price', 'ingredients.price_without', 'ingredients.name as ingredient_name')->leftJoin('ingredients', 'ingredients.id', '=', 'product_ingredients.ingredient_id')->orderByRaw('product_ingredients.sequence ASC');

        if ($query = $req->query('product_id', false)) {
            $productIngredients->where('product_id', $query);
        }
        if ($req->query('with')) {
            $productIngredients->where('with', '1');
        }
        if ($req->query('without')) {
            $productIngredients->where('without', '1');
        }
        if ($req->query('disabled') == '0' || $req->query('disabled') == '1') {
            $productIngredients->where('disabled', $req->query('disabled'));
        }
        if ($req->query('nopaginate', false) == '1') {
            $productIngredients = $productIngredients->get();
        } else {
            $productIngredients = $productIngredients->paginate($req->query('per_page', 20));
        }

        if ($productIngredients->isEmpty()) Utils::error('No productIngredients found!', 404);
        return $productIngredients;
    }

    public function create(Request $req)
    {
        $this->validate($req, [
            'with' => 'boolean',
            'without' => 'boolean',
            'product_id' =>  ['required', 'exists:epos_connection.products,id', Rule::unique('epos_connection.product_ingredients')->where(function ($query) use ($req) {
                return $query->where('product_id', $req->input('product_id'))->where('ingredient_id', $req->input('ingredient_id'))->where('with', $req->input('with'));
            })],
            'ingredient_id' =>  ['required', 'exists:epos_connection.products,id', Rule::unique('epos_connection.product_ingredients')->where(function ($query) use ($req) {
                return $query->where('product_id', $req->input('product_id'))->where('ingredient_id', $req->input('ingredient_id'))->where('with', $req->input('with'));
            })],
            'sequence' => 'numeric',
        ], [
            'product_id.unique' => "Ingredient already assigned with the product",
            'ingredient_id.unique' => "Product already assigned with the ingredient",
        ]);

        // create productIngredient
        $productIngredient = new EposProductIngredient;
        $productIngredient->fill($req->all());
        $productIngredient->save();

        // Sequence add
        $productIngredient->sequence = $productIngredient->id;
        $productIngredient->save();

        return $productIngredient;
    }


    /**
     * Add productIngredient multiple.
     * POST /epos/product-ingredients/multiple
     */
    public function create_multiple(Request $req)
    {
        if ($req->has('with_ids') && !is_array($req->input('with_ids')))
            $req->merge(['with_ids' => json_decode($req->input('with_ids'))]);
        if ($req->has('without_ids') && !is_array($req->input('without_ids')))
            $req->merge(['without_ids' => json_decode($req->input('without_ids'))]);

        $this->validate($req, [
            'with_ids' => 'array|required_without:without_ids',
            'without_ids' => 'array|required_without:with_ids',
            'product_id' => 'required|exists:epos_connection.products,id'
        ]);
        $product_id = $req->input('product_id');

        if (!empty($req->input('with_ids')) && sizeof($req->input('with_ids')) > 0) {
            foreach ($req->input('with_ids') as $with_id) {
                $productIngredient = EposProductIngredient::where('ingredient_id', $with_id)->where('product_id', $product_id)->first();
                if (empty($productIngredient)) {
                    $productIngredient = new EposProductIngredient();
                }
                $productIngredient->ingredient_id = $with_id;
                $productIngredient->product_id = $product_id;
                $productIngredient->with = 1;
                $productIngredient->save();
                $productIngredient->sequence = $productIngredient->id;
                $productIngredient->save();
            }
        }
        if (!empty($req->input('without_ids')) && sizeof($req->input('without_ids')) > 0) {
            foreach ($req->input('without_ids') as $without_id) {
                $productIngredient = EposProductIngredient::where('ingredient_id', $without_id)->where('product_id', $product_id)->first();
                if (empty($productIngredient)) {
                    $productIngredient = new EposProductIngredient();
                }
                $productIngredient->ingredient_id = $without_id;
                $productIngredient->product_id = $product_id;
                $productIngredient->without = 1;
                $productIngredient->save();
                $productIngredient->sequence = $productIngredient->id;
                $productIngredient->save();
            }
        }
        return ["message" => 'Ingredients saved'];
    }

    /**
     * Show productIngredient profile.
     * GET /epos/product-ingredients/{id}
     */
    public function show(Request $req, $id)
    {
        $productIngredient = EposProductIngredient::select('product_ingredients.*', 'ingredients.price', 'ingredients.web_price', 'ingredients.takeaway_price', 'ingredients.delivery_price', 'ingredients.waiting_price', 'ingredients.name as ingredient_name')->leftJoin('ingredients', 'ingredients.id', '=', 'product_ingredients.ingredient_id')->find($id);
        if (empty($productIngredient)) Utils::error('EposProductIngredient not found!', 404);
        return $productIngredient;
    }

    /**
     * Update productIngredient.
     * POST /epos/product-ingredients/{id}
     */
    public function update(Request $req, $id)
    {
        $productIngredient = EposProductIngredient::find($id);

        if (empty($productIngredient)) Utils::error('EposProductIngredient not found!', 404);

        $this->validate($req, [
            'with' => 'boolean',
            'without' => 'boolean',
            'product_id' =>  ['required', 'exists:epos_connection.products,id', Rule::unique('epos_connection.product_ingredients')->where(function ($query) use ($req, $id) {
                return $query->where('product_id', $req->input('product_id'))->where('ingredient_id', $req->input('ingredient_id'))->where('with', $req->input('with'))->whereNotIn('id', [$id]);
            })],
            'ingredient_id' =>  ['required', 'exists:epos_connection.products,id', Rule::unique('epos_connection.product_ingredients')->where(function ($query) use ($req, $id) {
                return $query->where('product_id', $req->input('product_id'))->where('ingredient_id', $req->input('ingredient_id'))->where('with', $req->input('with'))->whereNotIn('id', [$id]);
            })],
            'sequence' => 'numeric',
        ], [
            'product_id.unique' => "Ingredient already assigned with the product",
            'ingredient_id.unique' => "Product already assigned with the ingredient",
        ]);

        // update productIngredient
        if ($req->has('product_id')) $productIngredient->product_id = $req->input('product_id');
        if ($req->has('ingredient_id')) $productIngredient->ingredient_id = $req->input('ingredient_id');
        $productIngredient->with = $req->input('with');
        $productIngredient->without = $req->input('without');
        $productIngredient->sequence = $req->input('sequence');

        $productIngredient->save();

        return $productIngredient;
    }

    /**
     * Enable productIngredient.
     * POST /epos/product-ingredients/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $productIngredient = EposProductIngredient::find($id);
        if (empty($productIngredient)) Utils::error('EposProductIngredient not found!', 404);
        $productIngredient->disabled = 0;
        $productIngredient->save();
        return $productIngredient;
    }

    /**
     * Disable productIngredient.
     * POST /epos/product-ingredients/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $productIngredient = EposProductIngredient::find($id);
        if (empty($productIngredient)) Utils::error('EposProductIngredient not found!', 404);
        $productIngredient->disabled = 1;
        $productIngredient->save();
        return $productIngredient;
    }

    /**
     * Enable Contains Status productIngredient.
     * POST /epos/product-ingredients/{id}/enable
     */
    public function containEnable(Request $req, $id)
    {
        $productIngredient = EposProductIngredient::find($id);
        if (empty($productIngredient)) Utils::error('EposProductIngredient not found!', 404);
        $productIngredient->contain = 0;
        $productIngredient->save();
        return $productIngredient;
    }

    /**
     * Disable productIngredient.
     * POST /epos/product-ingredients/{id}/disable
     */
    public function containDisable(Request $req, $id)
    {
        $productIngredient = EposProductIngredient::find($id);
        if (empty($productIngredient)) Utils::error('EposProductIngredient not found!', 404);
        $productIngredient->contain = 1;
        $productIngredient->save();
        return $productIngredient;
    }

    /**
     * Delete productIngredient by id.
     * DELETE /epos/product-ingredients/{id}
     */
    public function delete(Request $req, $id)
    {
        $productIngredient = EposProductIngredient::find($id);
        if (empty($productIngredient)) Utils::error('EposProductIngredient not found!', 404);
        $productIngredient->delete();
        if ($productIngredient->image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_USER, $productIngredient->image);
        return $productIngredient;
    }

    /**
     * Delete productIngredient by id.
     * POST /epos/product-ingredients/delete-bulk
     */
    public function deleteBulk(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids')))
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        $this->validate($req, [
            'ids' => 'required|array'
        ]);

        if (!empty($req->input('ids')) || sizeof($req->input('ids')) > 0) {
            foreach ($req->input('ids') as $id) {
                $productIngredient = EposProductIngredient::find($id);
                if (!empty($productIngredient)) $productIngredient->delete();
            }
        }
        return $productIngredient;
    }
}
