<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\Auth;
use App\Helpers\PushNotificationHelper;
use App\ModelsEpos\EposReservation;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\ModelsEpos\EposCustomer;
use App\ModelsEpos\EposReservationItem;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class EposReservationController extends Controller
{
    /**
     * List reservations.
     * GET /epos/reservations
     */
    public function index(Request $req)
    {
        $reservations = EposReservation::selectRaw('reservations.*,customers.name as customer_name,tables.number as table_number,reservation_statuses.status as reservation_status,deposit_types.type as deposit_type')->leftJoin('deposit_types', 'deposit_types.id', '=', 'deposit_type_id')->leftJoin('customers', 'customers.id', '=', 'customer_id')->leftJoin('tables', 'tables.id', '=', 'table_id')->leftJoin('reservation_statuses', 'reservation_statuses.id', '=', 'reservation_status_id')->orderByRaw('reservations.created_at DESC');

        $this->validate($req, [
            'from_date' => 'date_format:Y-m-d',
            'to_date' => 'required_with:from_date|date_format:Y-m-d',
        ]);

        if ($req->has('from_date')) {
            $from_date = $req->input('from_date') . ' ' . "00:00:00";
            $to_date = $req->input('to_date') . ' ' . "23:59:59";
            $from_date = Carbon::parse($from_date);
            $to_date = Carbon::parse($to_date);

            $reservations = $reservations->where('reservation_date_time', '>', "$from_date");
            $reservations = $reservations->where('reservation_date_time', '<', "$to_date");
        }

        if ($query = $req->query('query', false)) {
            $reservations->whereRaw('(reservations.deposit_amount ?  )', ["%$query%"]);
        }
        if ($query = $req->query('table_id', false)) {
            $reservations->where('table_id', $query);
        }
        if ($query = $req->query('reservation_status_id', false)) {
            $reservations->where('reservation_status_id', $query);
        }
        if ($query = $req->query('reservation_confirmation_id', false)) {
            $reservations->where('reservation_confirmation_id', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $reservations = $reservations->get();
        } else {
            $reservations = $reservations->paginate($req->query('per_page', 20));
        }
        if ($reservations->isEmpty()) Utils::error('No reservations found!', 404);

        $reservations->append('customer');
        $reservations->append('table');
        return ($reservations);
    }

    /**
     * Create table.
     * POST /epos/reservations
     */
    public function create(Request $req)
    {
        if (!is_array($req->customer))
            $req->merge(['customer' => (json_decode($req->customer, true))]);

        $this->validate($req, [
            'customer_id' => 'exists:epos_connection.customers,id',
            'table_id' => 'exists:epos_connection.tables,id',
            'reservation_status_id' => 'exists:epos_connection.reservation_statuses,id',
            'reservation_confirmation_id' => 'exists:epos_connection.reservation_confirmations,id',
            'deposit_type_id' => 'exists:epos_connection.deposit_types,id',
            'deposit_amount' => 'numeric',
            'children' => 'numeric',
            'infants' => 'numeric',
            'diners' => 'numeric',
            'telephone' => 'string',
            'reservation_date_time' => 'date',
            'special_instruction' => 'string',
            'unique_id' => 'string',
        ]);

        // create reservation
        $reservation = new EposReservation;
        $reservation->fill($req->all());
        if (empty($req->input('customer_id'))) {
            $customerDetail = $req->input('customer');
            $customer = new EposCustomer((array)$customerDetail);
            $customer->membership_no = rand(100000000, 999999999);
            $customer->save();

            $reservation->customer_id = $customer->id;
        } else {
            $customerDetail = ($req->input('customer'));
            $customer = EposCustomer::find($customerDetail['id']);
            if (isset($customerDetail['name'])) $customer->name = $customerDetail['name'];
            if (isset($customerDetail['mobile'])) $customer->mobile = $customerDetail['mobile'];
            if (isset($customerDetail['email'])) $customer->email = $customerDetail['email'];
            if (isset($customerDetail['landline'])) $customer->landline = $customerDetail['landline'];
            if (isset($customerDetail['birth_date'])) $customer->birth_date = $customerDetail['birth_date'];
            if (isset($customerDetail['anniversary_date'])) $customer->anniversary_date = $customerDetail['anniversary_date'];
            if (isset($customerDetail['house_no'])) $customer->house_no = $customerDetail['house_no'];
            if (isset($customerDetail['street'])) $customer->street = $customerDetail['street'];
            if (isset($customerDetail['postcode'])) $customer->postcode = $customerDetail['postcode'];
            if (isset($customerDetail['distance'])) $customer->distance = $customerDetail['distance'];
            if (isset($customerDetail['city'])) $customer->city = $customerDetail['city'];
            if (isset($customerDetail['country'])) $customer->country = $customerDetail['country'];
            if (isset($customerDetail['state'])) $customer->state = $customerDetail['state'];
            $customer->save();

            $reservation->customer_id = $customer->id;
        }
        $reservation->save();
        $reservation->append('customer');
        $reservation->append('table');

        return $reservation;
    }

    /**
     * Show table.
     * GET /epos/reservations/{id}
     */
    public function show(Request $req, $id)
    {
        $reservations = EposReservation::selectRaw('reservations.*,customers.name as customer_name,tables.number as table_number,reservation_statuses.status as reservation_status,deposit_types.type as deposit_type')->leftJoin('customers', 'customers.id', '=', 'customer_id')->leftJoin('tables', 'tables.id', '=', 'table_id')->leftJoin('reservation_statuses', 'reservation_statuses.id', '=', 'reservation_status_id')->leftJoin('reservation_confirmations', 'reservation_confirmations.id', '=', 'reservation_confirmation_id')->leftJoin('deposit_types', 'deposit_types.id', '=', 'deposit_type_id')->find($id);
        if (empty($reservations)) Utils::error('EposReservation not found!', 404);

        $reservations->append('customer');
        $reservations->append('table');
        return $reservations;
    }

    /**
     * Update reservation.
     * POST /epos/reservations/{id}
     */
    public function update(Request $req, $id)
    {
        if (!is_array($req->customer))
            $req->merge(['customer' => (json_decode($req->customer, true))]);

        $reservation = EposReservation::find($id);
        if (empty($reservation)) Utils::error('EposReservation not found!', 404);

        $this->validate($req, [
            'customer_id' => 'exists:epos_connection.customers,id',
            'table_id' => 'exists:epos_connection.tables,id',
            'reservation_status_id' => 'exists:epos_connection.reservation_statuses,id',
            'reservation_confirmation_id' => 'exists:epos_connection.reservation_confirmations,id',
            'deposit_type_id' => 'exists:epos_connection.deposit_types,id',
            'deposit_amount' => 'numeric',
            'children' => 'numeric',
            'infants' => 'numeric',
            'diners' => 'numeric',
            'telephone' => 'string',
            'reservation_date_time' => 'date',
            'special_instruction' => 'string',
            'unique_id' => 'string',
        ]);

        // update reservation
        if ($req->has('customer_id')) $reservation->customer_id = $req->input('customer_id');
        if ($req->has('table_id')) $reservation->table_id = $req->input('table_id');
        if ($req->has('reservation_status_id')) $reservation->reservation_status_id = $req->input('reservation_status_id');
        if ($req->has('reservation_confirmation_id')) $reservation->reservation_confirmation_id = $req->input('reservation_confirmation_id');
        if ($req->has('deposit_type_id')) $reservation->deposit_type_id = $req->input('deposit_type_id');
        if ($req->has('deposit_amount')) $reservation->deposit_amount = $req->input('deposit_amount');
        if ($req->has('children')) $reservation->children = $req->input('children');
        if ($req->has('infants')) $reservation->infants = $req->input('infants');
        if ($req->has('diners')) $reservation->diners = $req->input('diners');
        if ($req->has('telephone')) $reservation->telephone = $req->input('telephone');
        if ($req->has('reservation_date_time')) $reservation->reservation_date_time = $req->input('reservation_date_time');
        if ($req->has('special_instruction')) $reservation->special_instruction = $req->input('special_instruction');
        if ($req->has('unique_id')) $reservation->unique_id = $req->input('unique_id');
        if (empty($req->input('customer_id'))) {
            $customerDetail = $req->input('customer');
            $customer = new EposCustomer((array)$customerDetail);
            $customer->membership_no = rand(100000000, 999999999);
            $customer->save();

            $reservation->customer_id = $customer->id;
        } else {
            $customerDetail = $req->input('customer');
            $customer = EposCustomer::find($customerDetail['id']);
            if (isset($customerDetail['name'])) $customer->name = $customerDetail['name'];
            if (isset($customerDetail['mobile'])) $customer->mobile = $customerDetail['mobile'];
            if (isset($customerDetail['email'])) $customer->email = $customerDetail['email'];
            if (isset($customerDetail['landline'])) $customer->landline = $customerDetail['landline'];
            if (isset($customerDetail['birth_date'])) $customer->birth_date = $customerDetail['birth_date'];
            if (isset($customerDetail['anniversary_date'])) $customer->anniversary_date = $customerDetail['anniversary_date'];
            if (isset($customerDetail['house_no'])) $customer->house_no = $customerDetail['house_no'];
            if (isset($customerDetail['street'])) $customer->street = $customerDetail['street'];
            if (isset($customerDetail['postcode'])) $customer->postcode = $customerDetail['postcode'];
            if (isset($customerDetail['distance'])) $customer->distance = $customerDetail['distance'];
            if (isset($customerDetail['city'])) $customer->city = $customerDetail['city'];
            if (isset($customerDetail['country'])) $customer->country = $customerDetail['country'];
            if (isset($customerDetail['state'])) $customer->state = $customerDetail['state'];
            $customer->save();

            $reservation->customer_id = $customer->id;
        }

        $reservation->save();
        $reservation->append('customer');
        $reservation->append('table');

        return $reservation;
    }


    /**
     * Delete reservation by id.
     * DELETE /epos/reservations/{id}
     */
    public function delete(Request $req, $id)
    {
        $reservation = EposReservation::find($id);
        if (empty($reservation)) Utils::error('EposReservation not found!', 404);
        $reservation->delete();

        return $reservation;
    }

    /**
     * Delete All reservation.
     * GET All/epos/reservations/delete-all
     */
    public function delete_all(Request $req)
    {
        (new EposReservation)->newQueryWithoutScopes()->forceDelete();
        DB::connection('epos_connection')->statement("ALTER TABLE reservations AUTO_INCREMENT =  0");

        // Factory reset notification
        $message = 'reservation reset done!';
        $notification['title'] = "reservation reset done!";
        $notification['body']  = $message;
        $notificationdata['data']['title'] = "reservation reset done!";
        $notificationdata['data']['message'] = $message;
        $notificationdata['data']['business_id'] = Auth::$business_id;
        $notificationdata['data']['device_id'] = $req->input('device_id');
        $notificationdata['data']['is_background']  = false;
        $notificationdata['data']['notification_for']  = 'epos';
        $notificationdata['data']['notification_section']  = 'reservation_factory_reset';
        $notificationdata['data']['timestamp'] = date('Y-m-d G:i:s');
        $notificationdata['data']['business_id'] = Auth::$business_id;

        $topic = '/topics/business_' . Auth::$business_id;
        $fcm_api_key = (new SiteSettingController())->findByKey('fcm_api_key');
        if (!empty($fcm_api_key)) {
            $api_key = $fcm_api_key->value;
            (new PushNotificationHelper())->sendNotification($notificationdata, $topic, $api_key, $notification);
        }

        return ['message' => 'reservations reset done!'];
    }
}
