<?php

namespace App\Http\Controllers\Epos;

use App\DeviceRegistration;
use App\Helpers\Auth;
use App\Helpers\PushIosNotificationHelper;
use App\Helpers\PushNotificationHelper;
use App\ModelsEpos\EposTable;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\SiteSettingController;
use App\ModelsEpos\EposDevice;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class EposTableController extends Controller
{
    /**
     * List tables.
     * GET /epos/tables
     */
    public function index(Request $req)
    {
        $tables = EposTable::selectRaw('tables.*,floors.name as floor_name,table_statuses.status as status')->leftJoin('table_statuses', 'table_statuses.id', '=', 'table_status_id')->leftJoin('floors', 'floors.id', '=', 'floor_id')->orderByRaw('tables.sequence ASC, tables.created_at DESC');

        if ($query = $req->query('query', false)) {
            $tables->whereRaw('(tables.number LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('floor_id', false)) {
            $tables->where('floor_id', $query);
        }
        if (Auth::isUser()) {
            $tables->where('disabled', 0);
        }
        if ($req->query('disabled') == '0' || $req->query('disabled') == '1') {
            $tables->where('disabled', $req->query('disabled'));
        }
        if ($req->query('nopaginate', false) == '1') {
            $tables = $tables->get();
        } else {
            $tables = $tables->paginate($req->query('per_page', 20));
        }

        if ($tables->isEmpty()) Utils::error('No tables found!', 404);


        return ($tables);
    }

    /**
     * Create table.
     * POST /epos/tables
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'number' => 'required|string|max:255',
            'floor_id' => 'required|exists:epos_connection.floors,id',
            'table_status_id' => 'required|exists:epos_connection.table_statuses,id',
            'sequence' => 'numeric|gt:0',
            'merge_table_id' => 'string',
        ]);

        // create table
        $table = new EposTable;
        $table->fill($req->all());
        $table->save();

        // Sequence add
        $table->sequence = $table->id;
        $table->save();

        return $table;
    }

    /**
     * Show table.
     * GET /epos/tables/{id}
     */
    public function show(Request $req, $id)
    {
        $table = EposTable::selectRaw('tables.*,floors.name as floor_name,table_statuses.status as status')->leftJoin('table_statuses', 'table_statuses.id', '=', 'table_status_id')->leftJoin('floors', 'floors.id', '=', 'floor_id')->find($id);
        if (empty($table)) Utils::error('EposTable not found!', 404);
        return $table;
    }

    /**
     * Update table.
     * POST /epos/tables/{id}
     */
    public function update(Request $req, $id)
    {
        $table = EposTable::find($id);
        if (empty($table)) Utils::error('EposTable not found!', 404);

        $this->validate($req, [
            'number' => 'string|max:255',
            'floor_id' => 'exists:epos_connection.floors,id',
            'table_status_id' => 'exists:epos_connection.table_statuses,id',
            'sequence' => 'numeric|gt:0',
            'locked' => 'boolean',
            'hold' => 'boolean',
            'merge_table_id' => 'string',
        ]);

        // update table
        if ($req->has('number')) $table->number = $req->input('number');
        if ($req->has('floor_id')) $table->floor_id = $req->input('floor_id');
        if ($req->has('table_status_id')) $table->table_status_id = $req->input('table_status_id');
        if ($req->has('sequence')) $table->sequence = $req->input('sequence');
        if ($req->has('locked')) $table->locked = $req->input('locked');
        if ($req->has('hold')) $table->hold = $req->input('hold');
        if ($req->has('merge_table_id')) $table->merge_table_id = $req->input('merge_table_id');
        if ($req->has('updater_id')) $table->updater_id = $req->input('updater_id');

        $table->save();

        $message      = 'You have a table update ' . $table->id;

        $notification['title'] = "Table Update";
        $notification['body'] = $message;

        $notificationdata['data']['title']          = "Table Update";
        $notificationdata['data']['message']        = $message;
        $notificationdata['data']['is_background']  = false;
        $notificationdata['data']['table_id']  = $table->id;
        $notificationdata['data']['updater_id']  = $table->updater_id;
        $notificationdata['data']['table_status']  = $table->table_status_id;
        $notificationdata['data']['notification_for']  = 'epos';
        $notificationdata['data']['notification_section']  = 'tables';
        $notificationdata['data']['timestamp']      = date('Y-m-d G:i:s');

        $topic = '/topics/business_' . Auth::$business_id;
        $fcm_api_key = (new SiteSettingController())->findByKey('fcm_api_key');
        if (!empty($fcm_api_key)) {
            $api_key = $fcm_api_key->value;
            (new PushNotificationHelper())->sendNotification($notificationdata, $topic, $api_key, $notification);
        }

        // $devices = EposDevice::orderByRaw('created_at DESC')->whereNotNull('device_token')->where('type', 'iOS')->where('disabled', 0)->get();
        $devices = DeviceRegistration::orderByRaw('created_at DESC')->whereNotNull('device_token')->where('device_type', 'iOS')->where('status', 'Accepted')->get();
        if (!empty($devices)) {
            foreach ($devices as $device) {
                $message = 'You have a table update ' . $table->id;
                (new PushIosNotificationHelper())->sendNotification($message, $device->device_token, $table->id, $table->table_status_id, 'tables');
            }
        }

        return $table;
    }


    /**
     * Delete table by id.
     * DELETE /epos/tables/{id}
     */
    public function delete(Request $req, $id)
    {
        $table = EposTable::find($id);
        if (empty($table)) Utils::error('EposTable not found!', 404);
        $table->delete();
        return $table;
    }

    /**
     * Enable table.
     * POST /epos/tables/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $table = EposTable::find($id);
        if (empty($table)) Utils::error('EposTable not found!', 404);
        $table->disabled = 0;
        $table->save();
        return $table;
    }

    /**
     * Disable table.
     * POST /epos/tables/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $table = EposTable::find($id);
        if (empty($table)) Utils::error('EposTable not found!', 404);
        $table->disabled = 1;
        $table->save();
        return $table;
    }

    public function table_bulk(Request $req)
    {
        if ($req->has('tables') && !is_array($req->input('tables'))) {
            $req->merge(['tables' => json_decode($req->input('tables'))]);
        }
        $this->validate($req, [
            'tables' => 'required|array'
        ],);
        $tables = ($req->input('tables'));
        $count = 1;
        foreach ($tables as $table) {
            $tableObj = EposTable::find($table->id);
            $tableObj->sequence = $count;
            $tableObj->save();
            $count++;
        }
        return $tables;
    }

    public function status_bulk(Request $req)
    {
        if ($req->has('tables') && !is_array($req->input('tables'))) {
            $req->merge(['tables' => json_decode($req->input('tables'))]);
        }
        $this->validate($req, [
            'tables' => 'required|array'
        ],);
        $tables = ($req->input('tables'));
        foreach ($tables as $table) {
            $tableObj = EposTable::find($table->id);
            $tableObj = EposTable::find($table->id);
            if (empty($tableObj)) {
                Utils::error("Table not found", 404);
            }

            if (isset($table->table_status_id)) $tableObj->table_status_id = $table->table_status_id;
            if (isset($table->updater_id)) $tableObj->updater_id = $table->updater_id;
            if (isset($table->locked)) $tableObj->locked = $table->locked;
            if (isset($table->hold)) $tableObj->hold = $table->hold;
            if (isset($table->merge_table_id)) $tableObj->merge_table_id = $table->merge_table_id;

            $tableObj->save();
        }
        return $tables;
    }
}
