<?php

namespace App\Http\Controllers\Epos;

use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use Lcobucci\JWT\Token;
use Illuminate\Support\Facades\Crypt;
use App\ModelsEpos\EposUser;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use DateTimeImmutable;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class EposUserController extends Controller
{
    /**
     * List users.
     * GET /epos/users/
     */
    public function index(Request $req)
    {
        $users = EposUser::latest();

        if ($query = $req->query('query', false)) {
            $users->whereRaw('(username LIKE ? OR mobile LIKE ? OR email LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($req->query('nopaginate', false) == '1') {
            $users = $users->get();
        } else {
            $users = $users->paginate($req->query('per_page', 20));
        }

        if ($users->isEmpty()) Utils::error('No users found!', 404);
        return $users;
    }
    /**
     * List mini users.
     * GET /epos/users/mini
     */
    public function mini(Request $req)
    {
        $users = EposUser::select('id', 'username', 'mobile', 'email', 'password')->orderByRaw('username ASC');
        $users->where('disabled', 0);
        if ($query = $req->query('query', false)) {
            $users->whereRaw('(username LIKE ? OR mobile LIKE ? OR email LIKE ?)', ["%$query%", "%$query%", "%$query%"]);
        }
        $users = $users->get();
        if ($users->isEmpty()) Utils::error('No users found!', 404);

        $users = $users->each(function ($user) {
            $user->makeVisible(['password']);
        });
        return ($users);
    }
    public function create(Request $req)
    {
        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }

        $this->validate($req, [
            'user_type_id' => 'exists:epos_connection.user_types,id',
            'user_group_id' => 'exists:epos_connection.user_groups,id',
            'username' => 'required|unique:epos_connection.users,username',
            'image' => 'image',
            'password' => 'required',
            'mobile' => 'nullable',
            'email' => 'nullable',
            'permissions' => 'required',
        ]);

        // create user
        $user = new EposUser;
        $user->fill($req->all());
        $user->password = Hash::make($user->password);
        $user->business_id = Auth::user()->selected_business->id;
        if ($req->has('permissions')) $user->permissions = $req->input('permissions');
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_EPOS_USER);
            $user->image = $fileName;
        }
        $user->save();

        // Add Preffix
        $this->addPriffix();

        return $user;
    }

    /**
     * Show user profile.
     * GET /epos/users/{id}
     */
    public function show(Request $req, $id = 'me')
    {
        if ($id == 'me') $user = Auth::user();
        else $user = EposUser::find($id);
        if (empty($user)) Utils::error('EposUser not found!', 404);
        $user->makeVisible(['password']);
        return $user;
    }

    /**
     * Update user.
     * POST /epos/users/{id}
     */
    public function update(Request $req, $id)
    {
        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }

        $user = EposUser::find($id);
        if (empty($user)) Utils::error('EposUser not found!', 404);

        $this->validate($req, [
            'user_type_id' => 'exists:epos_connection.user_types,id',
            'user_group_id' => 'exists:epos_connection.user_groups,id',
            'username' => ['required', Rule::unique('epos_connection.users')->ignore($user->id)],
            'image' => 'image',
            'mobile' => 'nullable',
            'email' => 'nullable',
            'permissions' => 'required',
        ]);

        // update user
        if ($req->has('mobile')) $user->mobile = $req->input('mobile');
        if ($req->has('email')) $user->email = $req->input('email');
        if ($req->has('user_type_id')) $user->user_type_id = $req->input('user_type_id');
        if ($req->has('user_group_id')) $user->user_group_id = $req->input('user_group_id');
        if ($req->has('username')) $user->username = $req->input('username');
        if ($req->has('permissions')) $user->permissions = $req->input('permissions');
        $user->is_loggin = $req->input('is_loggin');
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_EPOS_USER);
            $user->image = $fileName;
        }
        $user->save();

        // Add Preffix
        $this->addPriffix();

        return $user;
    }


    /**
     * Update Password user.
     * POST /epos/users/{id}/change-password
     */
    public function chage_password(Request $req, $id)
    {
        $user = EposUser::find($id);

        if (empty($user)) Utils::error('EposUser not found!', 404);

        $this->validate($req, [
            'password' => 'required',
        ]);

        // update user
        if ($req->has('password')) $user->password = $req->input('password');
        $user->save();

        // Add Preffix
        $this->addPriffix();

        return $user;
    }

    /**
     * Enable user.
     * POST /epos/users/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $user = EposUser::find($id);
        if (empty($user)) Utils::error('EposUser not found!', 404);
        $user->disabled = 0;
        $user->save();

        // Add Preffix
        $this->addPriffix();

        return $user;
    }

    /**
     * Disable user.
     * POST /epos/users/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $user = EposUser::find($id);
        if (empty($user)) Utils::error('EposUser not found!', 404);
        $user->disabled = 1;
        $user->save();

        // Add Preffix
        $this->addPriffix();

        return $user;
    }

    /**
     * Delete user by id.
     * DELETE /epos/users/{id}
     */
    public function delete(Request $req, $id)
    {
        $user = EposUser::find($id);
        if (empty($user)) Utils::error('EposUser not found!', 404);
        $user->delete();
        if ($user->image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_USER, $user->image);

        // Add Preffix
        $this->addPriffix();

        return $user;
    }

    /**
     * Login to Epos.
     * POST epos/users/login
     */
    public function login(Request $req)
    {
        $this->validate($req, [
            'username' => 'required',
            'password' => 'required',
            'mode' => 'in:auto,manual',
        ]);
        $user = EposUser::where('username', $req->input('username'))->first();
        if (empty($user)) Utils::error("Invalid login details", 400);

        // if (!Hash::check($req->input('password'), $user->password)) {
        if ($req->input('password') != $user->password) {
            return Utils::error('Invalid password.', 400);
        } else {
            if ($user->is_loggin && $req->input('mode') == 'auto') Utils::error("User already logged in other device,Please try later", 400);
            $user->makeHidden(['business_counts', 'balance']);
            $tokens = $this->getNewTokens($user->id);

            if ($req->input('mode') == 'auto') {
                $user->is_loggin = 1;
                $user->save();
            }

            return response()->json($user, 200, [
                'x-token' => (string) $tokens['aToken'],
                'x-refresh-token' => (string) $tokens['rToken'],
            ]);
        }
    }

    /**
     * Logout to Epos.
     * GET epos/users/{id}/logout
     */
    public function logout($id)
    {
        $user = EposUser::where('id', $id)->first();
        if (empty($user)) Utils::error("Invalid user details", 400);

        $user->is_loggin = 0;
        $user->save();
        return $user;
    }


    /**
     * Update user password.
     * POST /epos/users/me/password
     */
    public function password(Request $req)
    {
        $user = Auth::user();
        if (empty($user)) Utils::error('EposUser not found!', 404);

        $this->validate($req, [
            'password' => 'required|string',
            'new_password' => 'required|string',
        ]);

        // if (!Hash::check($req->input('password'), $user->password)) {
        if ($req->input('password') != $user->password) {
            return Utils::error('Invalid password.', 400);
        } else {
            $user->password = Hash::make($req->input('new_password'));
            $user->save();

            $tokens = $this->getNewTokens($user->id);
            return response()->json($user, 200, [
                'x-token' => (string) $tokens['aToken'],
                'x-refresh-token' => (string) $tokens['rToken'],
            ]);
        }
    }

    /**
     * Token helper to get new tokens.
     */
    public static function getNewTokens($userId)
    {
        $user = EposUser::find($userId);
        if (empty($user)) return null;
        // if ($user->disabled == 1) Utils::error('Your account is disabled', 401, 401);

        $now   = new DateTimeImmutable();

        $aTokenExp = $now->modify('+1 minute');
        $aTokenConfig = Auth::getConfig(env('X_TOKEN_KEY'));
        $aBuilder = $aTokenConfig->builder()
            ->issuedAt($now)
            ->expiresAt($aTokenExp)
            ->withClaim('id', $user->id)
            ->withClaim('role', 'epos_user');
        $aToken = $aBuilder->getToken($aTokenConfig->signer(), $aTokenConfig->signingKey())->toString();

        $rTokenExp = (new DateTimeImmutable())->modify('+168 hours');
        $rTokenConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . $user->password);
        $rBuilder = $rTokenConfig->builder()
            ->issuedAt($now)
            ->expiresAt($rTokenExp)
            ->withClaim('id', $user->id)
            ->withClaim('role', 'epos_user');
        $rToken = $rBuilder->getToken($rTokenConfig->signer(), $rTokenConfig->signingKey())->toString();
        return array('aToken' => Crypt::encrypt((string) $aToken), 'rToken' => Crypt::encrypt((string) $rToken));
    }

    /**
     * Token helper to get verify tokens.
     */
    public static function verifyRefreshToken(Token $rToken)
    {
        $admin = EposUser::find($rToken->claims()->get('id'));
        if (empty($admin)) return false;
        $rTokenConfig = Auth::getConfig(env('X_REFRESH_TOKEN_KEY') . $admin->password);
        $constraints = $rTokenConfig->validationConstraints();
        $rTokenConfig->validator()->assert($rToken, ...$constraints);
        if (!$rTokenConfig->validator()->validate($rToken, ...$constraints)) {
            return false;
        } else {
            return true;
        }
    }

    public function addPriffix()
    {
        try {
            // User Preffix add
            $users = EposUser::whereNotNull('id')->get();
            foreach ($users as $key => $user) {
                $priffix = $this->getColName($key + 1);
                $user->preffix = $priffix;
                $user->save();
            }
        } catch (Exception $e) {
            return null;
        }
    }

    public function getColName($index)
    {
        $alpha = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'];

        $index--;
        $nAlphabets = 26;
        $f = floor($index / pow($nAlphabets, 0)) % $nAlphabets;
        $s = (floor($index / pow($nAlphabets, 1)) % $nAlphabets) - 1;
        $t = (floor($index / pow($nAlphabets, 2)) % $nAlphabets) - 1;

        $f = $f < 0 ? '' : $alpha[$f];
        $s = $s < 0 ? '' : $alpha[$s];
        $t = $t < 0 ? '' : $alpha[$t];

        return trim("{$t}{$s}{$f}");
    }
}
