<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\HardwareProduct;
use App\HardwareProductImage;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class HardwareProductController extends Controller
{
    /**
     * List hardware-products.
     * GET /api/hardware-products
     */
    public function index(Request $req)
    {
        $products = HardwareProduct::orderByRaw('id ASC');

        if ($query = $req->query('query', false)) {
            $products->whereRaw('(name LIKE ? OR  amount LIKE ?)', ["%$query%", "%$query%"]);
        }

        if ($disabled = $req->query('disabled', false)) {
            $products->where('disabled', $disabled);
        }

        if ($req->query('nopaginate', false) == '1') {
            $products = $products->get();
        } else {
            $products = $products->paginate($req->query('per_page', 20));
        }

        if ($products->isEmpty()) Utils::error('No products found!', 404);

        return $products;
    }

    /**
     * Create hardware-product.
     * POST /api/hardware-products
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string',
            'stock_quantity' => 'required|numeric',
            'amount' => 'required|numeric',
            'description' => 'string',
            'tax_amount' => 'numeric',
        ]);

        // create hardware-product
        $product = new HardwareProduct;
        $product->fill($req->all());

        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_HARDWARE_PRODUCT);
            $product->image = $fileName;
        }
        $product->save();

        if ($req->file('images')) {
            // $order_items = HardwareProductImage::where('product_id', $product->id)->delete();
            $file_names  = $req->file('images');
            for ($i = 0; $i < count($file_names); $i++) {
                $file_name = $file_names[$i];
                $fileName = ImageHelper::saveImageToServer($file_name, ImageHelper::$TYPE_HARDWARE_PRODUCT);
                $product_image = new HardwareProductImage();
                $product_image->product_id = $product->id;
                $product_image->priority = $i;
                $product_image->image = $fileName;
                $product_image->save();
            }
        }

        return $product;
    }

    /**
     * Show hardware-product.
     * GET /api/hardware-products/{id}
     */
    public function show(Request $req, $id)
    {
        $product = HardwareProduct::find($id);
        if (empty($product)) Utils::error('Product not found!', 404);
        return $product;
    }

    /**
     * Update hardware-product.
     * POST /api/hardware-products/{id}
     */
    public function update(Request $req, $id)
    {
        $product = HardwareProduct::find($id);
        if (empty($product)) Utils::error('Product not found!', 404);

        $this->validate($req, [
            'name' => 'required|string',
            'stock_quantity' => 'required|numeric',
            'amount' => 'required|numeric',
            'description' => 'string',
            'tax_amount' => 'numeric',
            'is_exclude' => 'boolean',
        ]);

        // update hardware-product
        if ($req->has('name')) $product->name = $req->input('name');
        if ($req->has('stock_quantity')) $product->stock_quantity = $req->input('stock_quantity');
        if ($req->has('amount')) $product->amount = $req->input('amount');
        if ($req->has('description')) $product->description = $req->input('description');
        if ($req->has('tax_amount')) $product->tax_amount = $req->input('tax_amount');
        $product->is_exclude = $req->input('is_exclude');

        $product->save();

        if ($req->file('images')) {
            // $order_items = HardwareProductImage::where('product_id', $product->id)->delete();
            $file_names  = $req->file('images');
            for ($i = 0; $i < count($file_names); $i++) {
                $file_name = $file_names[$i];
                $fileName = ImageHelper::saveImageToServer($file_name, ImageHelper::$TYPE_HARDWARE_PRODUCT);
                $product_image = new HardwareProductImage();
                $product_image->product_id = $product->id;
                $product_image->priority = $i;
                $product_image->image = $fileName;
                $product_image->save();
            }
        }

        if ($req->has('deleted_images')) {
            $deleted_images = explode(",", $req->input('deleted_images'));
            HardwareProductImage::whereIn('id', $deleted_images)->delete();
        }

        return $product;
    }

    /**
     * Delete hardware-product by id.
     * DELETE /api/hardware-products/{id}
     */
    public function delete(Request $req, $id)
    {
        $product = HardwareProduct::find($id);
        if (empty($product)) Utils::error('Product not found!', 404);
        $product->delete();
        return $product;
    }

    /**
     * Enable hardware-product.
     * POST /epos/hardware-products/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $product = HardwareProduct::find($id);
        if (empty($product)) Utils::error('Product not found!', 404);
        $product->disabled = 0;
        $product->save();
        return $product;
    }

    /**
     * Disable hardware-product.
     * POST /epos/hardware-products/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $product = HardwareProduct::find($id);
        if (empty($product)) Utils::error('Product not found!', 404);
        $product->disabled = 1;
        $product->save();
        return $product;
    }
}
