<?php

namespace App\Http\Controllers;

use App\HardwareOrder;
use App\HardwareVoucher;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Carbon\Carbon;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class HardwareVoucherController extends Controller
{
    /**
     * List vouchers.
     * GET /api/vouchers
     */
    public function index(Request $req)
    {
        $vouchers = HardwareVoucher::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $vouchers->whereRaw('(voucher_code LIKE ?  )', ["%$query%"]);
        }

        if ($product_id = $req->query('product_id', false)) {
            $vouchers->whereRaw('(product_id LIKE ? OR product_id is null)', ["%$product_id%"]);
        }

        if ($from_date = $req->query('from_date', false)) {
            $vouchers->where('start_date', '<=', Carbon::parse($from_date)->startOfDay());
            $vouchers->whereRaw('(end_date <= ? OR end_date is null)', [Carbon::parse($req->input('to_date'))->endOfDay()]);
        }

        if ($disabled = $req->query('disabled', false)) {
            $vouchers->where('disabled', $disabled);
        }
        if ($req->query('nopaginate', false) == '1') {
            $vouchers = $vouchers->get();
        } else {
            $vouchers = $vouchers->paginate($req->query('per_page', 20));
        }

        if ($vouchers->isEmpty()) Utils::error('No vouchers found!', 404);

        return $vouchers;
    }

    /**
     * Create voucher.
     * POST /epos/vouchers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'product_id' => 'required|string',
            'voucher_code' => 'required|string',
            'voucher_amount' => 'required|numeric|gt:0',
            'minimum_order_value' => 'numeric',
            'start_date' => 'required|date',
            'end_date' => 'date',
        ]);

        // create voucher
        $voucher = new HardwareVoucher;
        $voucher->fill($req->all());
        $charset = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
        $voucher->voucher_code = substr(str_shuffle($charset), 0, 6);
        $voucher->save();

        return $voucher;
    }

    /**
     * Show voucher.
     * GET /epos/vouchers/{id}
     */
    public function show(Request $req, $id)
    {
        $voucher = HardwareVoucher::find($id);
        if (empty($voucher)) Utils::error('Voucher not found!', 404);
        return $voucher;
    }


    /**
     * 
     * Search Voucher Code.
     * GET voucher/find/{code}
     * 
     */
    public function find(Request $req, $code)
    {

        $this->validate($req, [
            // 'order_id' => 'required',
        ]);

        $day_of_week = date('N', strtotime(date('l')));

        $from_date = Carbon::today();
        $to_date = Carbon::today();

        $voucher = HardwareVoucher::whereRaw('(voucher_code LIKE ? OR voucher_code LIKE ? ) ', [$code, str_replace(" ", "", $code)])->where('disabled', '0')->first();
        if (empty($voucher)) Utils::error('Invalid voucher!', 404);

        if ($voucher->end_date != null) {
            if (Carbon::parse($voucher->start_date)->timestamp > Carbon::today()->timestamp && Carbon::parse($voucher->end_date)->timestamp < Carbon::today()->timestamp) {
                Utils::error("Voucher expired", 400);
            }
        } else {
            if (Carbon::parse($voucher->start_date)->timestamp > Carbon::today()->endOfDay()->timestamp) {
                Utils::error("Voucher is not valid before date", 400);
            }
        }

        if ($voucher->grand_total > $req->input('grand_total'))
            Utils::error("Voucher applicable minimum " . $voucher->grand_total . " order.", 400);

        if (str_contains($voucher->product_id, $req->input('productIds')))
            Utils::error("Voucher not applicable for this product", 400);

        // if (!empty($voucher->week_day) && str_contains($voucher->week_day, $day_of_week))
        //     Utils::error("Voucher not applicable for today", 400);

        // $appliedVoucherPayment = HardwareOrder::where('voucher_id', $voucher->id)->where('payment_method_id', '5')->get();
        // $voucher->applied = !$appliedVoucherPayment->isEmpty();

        return $voucher;
    }

    /**
     * Update voucher.
     * POST /epos/vouchers/{id}
     */
    public function update(Request $req, $id)
    {
        $voucher = HardwareVoucher::find($id);
        if (empty($voucher)) Utils::error('Voucher not found!', 404);

        $this->validate($req, [
            'product_id' => 'required|string',
            'voucher_code' => 'required|string',
            'voucher_amount' => 'required|numeric|gt:0',
            'minimum_order_value' => 'numeric',
            'start_date' => 'required|date',
            'end_date' => 'date',
        ]);

        // update voucher
        if ($req->has('product_id')) $voucher->product_id = $req->input('product_id');
        if ($req->has('voucher_code')) $voucher->voucher_code = $req->input('voucher_code');
        if ($req->has('voucher_amount')) $voucher->voucher_amount = $req->input('voucher_amount');
        if ($req->has('minimum_order_value')) $voucher->minimum_order_value = $req->input('minimum_order_value');
        if ($req->has('start_date')) $voucher->start_date = $req->input('start_date');
        if ($req->has('end_date')) $voucher->end_date = $req->input('end_date');

        $voucher->save();

        return $voucher;
    }

    /**
     * Delete voucher by id.
     * DELETE /epos/vouchers/{id}
     */
    public function delete(Request $req, $id)
    {
        $voucher = HardwareVoucher::find($id);
        if (empty($voucher)) Utils::error('Voucher not found!', 404);
        $voucher->delete();
        return $voucher;
    }

    /**
     * Enable voucher.
     * POST /epos/vouchers/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $voucher = HardwareVoucher::find($id);
        if (empty($voucher)) Utils::error('Voucher not found!', 404);
        $voucher->disabled = 0;
        $voucher->save();
        return $voucher;
    }

    /**
     * Disable voucher.
     * POST /epos/vouchers/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $voucher = HardwareVoucher::find($id);
        if (empty($voucher)) Utils::error('Voucher not found!', 404);
        $voucher->disabled = 1;
        $voucher->save();
        return $voucher;
    }
}
