<?php

namespace App\Http\Controllers;

use App\IdProof;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class IdProofController extends Controller
{
    /**
     * List idProofs.
     * GET /api/id-proofs
     */
    public function index(Request $req)
    {
        $idProofs = IdProof::orderByRaw('type DESC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $idProofs->whereRaw('(type LIKE ? OR id_number LIKE ? )', ["%$query%", "%$query%"]);
        }

        if (Auth::isUser()) {
            $idProofs->where('user_id', Auth::id());
        } else {

            if ($query = $req->query('user_id', false)) {
                $idProofs->where('user_id', $query);
            }
        }
        if ($req->query('nopaginate', false) == '1') {
            $idProofs = $idProofs->get();
        } else {
            $idProofs = $idProofs->paginate($req->query('per_page', 20));
        }

        if ($idProofs->isEmpty()) Utils::error('No idProofs found!', 404);


        return ($idProofs);
    }

    /**
     * Create idProof.
     * POST /api/id-proofs
     */
    public function create(Request $req)
    {
        if (Auth::isUser()) {
            $req->merge(['user_id' => Auth::id()]);
        }
        $this->validate($req, [
            'type' => 'required|string|max:255',
            'id_number' => 'required|string',
            'issuing_country' => 'string',
            'issue_place' => 'string',
            'issue_date' => 'date',
            'expiry_date' => 'date',
            'user_id' => 'required|exists:users,id',
            'file_type ' => 'in:image,pdf',
            // 'front_image' => 'mimes:jpeg,bmp,png,gif,svg,pdf',
            // 'back_image' => 'mimes:jpeg,bmp,png,gif,svg,pdf',
            'type_proof ' => 'in:id_proof,address_proof',
        ]);

        // create idProof
        $idProof = new IdProof;
        $idProof->fill($req->all());
        if (Auth::isAdmin()) {
            $idProof->admin_id = Auth::id();
        }
        if ($req->has('front_image')) {
            $frontImage = ImageHelper::saveImageToServer($req->file('front_image'), ImageHelper::$TYPE_PROOF);
            $idProof->front_image = $frontImage;
        }
        if ($req->has('back_image')) {
            $backImage = ImageHelper::saveImageToServer($req->file('back_image'), ImageHelper::$TYPE_PROOF);
            $idProof->back_image = $backImage;
        }
        $idProof->save();

        return $idProof;
    }

    /**
     * Show idProof.
     * GET /api/id-proofs/{id}
     */
    public function show(Request $req, $id)
    {
        $idProof = IdProof::find($id);
        if (empty($idProof)) Utils::error('IdProof not found!', 404);
        return $idProof;
    }

    /**
     * Update idProof.
     * POST /api/id-proofs/{id}
     */
    public function update(Request $req, $id)
    {
        $idProof = IdProof::find($id);
        if (empty($idProof)) Utils::error('IdProof not found!', 404);
        if (Auth::isUser()) {
            $req->merge(['user_id' => Auth::id()]);
        }
        $this->validate($req, [
            'type' => 'string|max:255',
            'id_number' => 'string',
            'issuing_country' => 'string',
            'issue_place' => 'string',
            'issue_date' => 'date',
            'expiry_date' => 'date',
            'user_id' => 'exists:users,id',
            'file_type ' => 'in:image,pdf',
            // 'front_image' => 'mimes:jpeg,bmp,png,gif,svg,pdf',
            // 'back_image' => 'mimes:jpeg,bmp,png,gif,svg,pdf',
            'type_proof ' => 'in:id_proof,address_proof',
        ]);

        // update idProof
        if ($req->has('type')) $idProof->type = $req->input('type');
        if ($req->has('id_number')) $idProof->id_number = $req->input('id_number');
        if ($req->has('issuing_country')) $idProof->issuing_country = $req->input('issuing_country');
        if ($req->has('issue_place')) $idProof->issue_place = $req->input('issue_place');
        if ($req->has('issue_date')) $idProof->issue_date = $req->input('issue_date');
        if ($req->has('expiry_date')) $idProof->expiry_date = $req->input('expiry_date');
        if ($req->has('user_id')) $idProof->user_id = $req->input('user_id');
        if ($req->has('type_proof')) $idProof->type_proof = $req->input('type_proof');
        if ($req->has('file_type')) $idProof->file_type = $req->input('file_type');
        if ($req->has('front_image')) {
            $frontImage = ImageHelper::saveImageToServer($req->file('front_image'), ImageHelper::$TYPE_PROOF);
            if ($idProof->front_image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_PROOF, $idProof->front_image);
            $idProof->front_image = $frontImage;
        }
        if ($req->has('back_image')) {
            $backImage = ImageHelper::saveImageToServer($req->file('back_image'), ImageHelper::$TYPE_PROOF);
            if ($idProof->back_image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_PROOF, $idProof->back_image);
            $idProof->back_image = $backImage;
        }
        $idProof->save();

        return $idProof;
    }


    /**
     * Delete idProof by id.
     * DELETE /api/id-proofs/{id}
     */
    public function delete(Request $req, $id)
    {
        $idProof = IdProof::find($id);
        if (empty($idProof)) Utils::error('IdProof not found!', 404);
        $idProof->delete();
        if ($idProof->back_image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_PROOF, $idProof->back_image);
        if ($idProof->front_image != null) ImageHelper::deleteServerImage(ImageHelper::$TYPE_PROOF, $idProof->front_image);
        return $idProof;
    }
}
