<?php

namespace App\Http\Controllers;

use App\Business;
use App\Exports\InvoiceExport;
use App\Helpers\Auth;
use App\Helpers\ImageHelper;
use App\Helpers\MailHelper;
use App\Helpers\OtpHelper;
use App\Invoice;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\InvoiceItem;
use App\InvoicePayLink;
use App\InvoicePayment;
use App\ModelsTiffintom\TiffintomShortlnUrl;
use App\ModelsTiffintom\TiffintomSiteSetting;
use App\Passbook;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Maatwebsite\Excel\Facades\Excel;
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use Mpdf\Mpdf;

class InvoiceController extends Controller
{
    /**
     * List invoices.
     * GET /api/invoices
     */
    public function index(Request $req)
    {
        $invoices = Invoice::selectRaw("invoices.*,businesses.name as business_name,businesses.online_service as online_service,businesses.merchant_service as merchant_service,businesses.epos_service as epos_service,businesses.connect_service as connect_service")->leftJoin('businesses', 'business_id', '=', 'businesses.id')->orderByRaw('invoices.id DESC, invoices.created_at DESC');

        if ($query = $req->query('query', false)) {
            $invoices->whereRaw('(invoices.invoice_type LIKE ? OR businesses.name LIKE ? OR invoices.status LIKE ? )', ["%$query%", "%$query%", "%$query%"]);
        }
        if ($query = $req->query('business_id', false)) {
            $invoices->where('invoices.business_id', $query);
        }
        if ($query = $req->query('recurring', false)) {
            $invoices->where('invoices.recurring', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $invoices = $invoices->get();
        } else {
            $invoices = $invoices->paginate($req->query('per_page', 20));
        }

        if ($invoices->isEmpty()) Utils::error('No invoices found!', 404);

        $invoices->append('invoice_items');
        $invoices->append('invoice_payments');

        return ($invoices);
    }

    /**
     * Create invoice.
     * POST /api/invoices
     */
    public function create(Request $req)
    {
        if ($req->has('items') && !is_array($req->input('items'))) {
            $req->merge(['items' => json_decode($req->input('items'))]);
        }
        $this->validate($req, [
            'image' => 'required_if:invoice_type,Customer|image',
            'invoice_type' => 'required|in:Internal,External,Customer',
            'business_id' => 'exists:businesses,id',
            'currency_symbol' => 'required|string',
            'header' => 'required|string',
            'bill_to' => 'required|string',
            'ship_to' => 'string',
            'date' => 'required|date',
            'sub_total' => 'required|numeric',
            'grand_total' => 'required|numeric',
            'tax_mode' => 'in:Flat,Percentage',
            'tax' => 'numeric',
            'discount_mode' => 'in:Flat,Percentage',
            'discount' => 'numeric',
            'notes' => 'string',
            'terms' => 'string',
            'terms' => 'string',
            'status' => 'in:Unpaid,Paid,Cancel,Dispute,Partial',
            'paid_amount' => 'numeric',
            'mobile_number' => 'string',
            'remark' => 'string',
            'archived' => 'boolean',
            'recurring' => 'boolean',
            'recurring_days' => 'required_if:recurring,1|string',
            'recurring_date' => 'string',
            'items' => 'required|array'
        ]);

        // create invoice
        $invoice = new Invoice;
        $invoice->fill($req->all());
        if ($req->has('recurring_days')) {
            $invoice->recurring_date = Carbon::today()->addDays($req->input('recurring_days'))->format('Y-m-d');
        }
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_INVOICE_LOGO);
            $invoice->image = $fileName;
        }
        $invoice->save();

        $items = ($req->input('items'));
        foreach ($items as $item) {
            $invoiceItem = new InvoiceItem((array)$item);
            $invoiceItem->invoice_id = $invoice->id;
            $invoiceItem->save();
        }
        $invoice->append('invoice_items');
        $invoice->append('invoice_payments');
        return $invoice;
    }

    /**
     * Show invoice.
     * GET /api/invoices/{id}
     */
    public function show(Request $req, $id)
    {
        $invoice = Invoice::find($id);
        if (empty($invoice)) Utils::error('Invoice not found!', 404);
        $invoice->append('invoice_items');
        $invoice->append('invoice_payments');
        return $invoice;
    }

    /**
     * Update invoice.
     * POST /api/invoices/{id}
     */
    public function update(Request $req, $id)
    {
        $invoice = Invoice::find($id);
        if (empty($invoice)) Utils::error('Invoice not found!', 404);
        if ($req->has('items') && !is_array($req->input('items'))) {
            $req->merge(['items' => json_decode($req->input('items'))]);
        }

        $this->validate($req, [
            'image' => 'image',
            'invoice_type' => 'required|in:Internal,External,Customer',
            'business_id' => 'exists:businesses,id',
            'currency_symbol' => 'required|string',
            'header' => 'string',
            'bill_to' => 'string',
            'ship_to' => 'string',
            'date' => 'required|date',
            'sub_total' => 'required|numeric',
            'grand_total' => 'required|numeric',
            'tax_mode' => 'in:Flat,Percentage',
            'tax' => 'numeric',
            'discount_mode' => 'in:Flat,Percentage',
            'discount' => 'numeric',
            'notes' => 'string',
            'terms' => 'string',
            'status' => 'in:Unpaid,Paid,Cancel,Dispute,Partial',
            'paid_amount' => 'numeric',
            'mobile_number' => 'string',
            'remark' => 'string',
            'archived' => 'boolean',
            'recurring' => 'boolean',
            'recurring_days' => 'required_if:recurring,1|string',
            'recurring_date' => 'string',
            'items' => 'required|array'
        ]);

        // update invoice
        if ($req->has('invoice_type')) $invoice->invoice_type = $req->input('invoice_type');
        if ($req->has('business_id')) $invoice->business_id = $req->input('business_id');
        if ($req->has('currency_symbol')) $invoice->currency_symbol = $req->input('currency_symbol');
        if ($req->has('header')) $invoice->header = $req->input('header');
        if ($req->has('bill_to')) $invoice->bill_to = $req->input('bill_to');
        if ($req->has('ship_to')) $invoice->ship_to = $req->input('ship_to');
        if ($req->has('date')) $invoice->date = $req->input('date');
        if ($req->has('sub_total')) $invoice->sub_total = $req->input('sub_total');
        if ($req->has('grand_total')) $invoice->grand_total = $req->input('grand_total');
        if ($req->has('tax_mode')) $invoice->tax_mode = $req->input('tax_mode');
        if ($req->has('tax')) $invoice->tax = $req->input('tax');
        if ($req->has('discount_mode')) $invoice->discount_mode = $req->input('discount_mode');
        if ($req->has('discount')) $invoice->discount = $req->input('discount');
        if ($req->has('notes')) $invoice->notes = $req->input('notes');
        if ($req->has('terms')) $invoice->terms = $req->input('terms');
        if ($req->has('status')) $invoice->status = $req->input('status');
        if ($req->has('paid_amount')) $invoice->paid_amount = $req->input('paid_amount');
        if ($req->has('mobile_number')) $invoice->mobile_number = $req->input('mobile_number');
        if ($req->has('remark')) $invoice->remark = $req->input('remark');
        $invoice->archived = $req->input('archived');
        $invoice->recurring = $req->input('recurring');
        if ($req->has('recurring_days')) $invoice->recurring_days = $req->input('recurring_days');
        if ($req->has('recurring_days')) {
            $invoice->recurring_date = Carbon::today()->addDays($req->input('recurring_days'))->format('Y-m-d');
        }
        if ($req->has('image')) {
            $fileName = ImageHelper::saveImageToServer($req->file('image'), ImageHelper::$TYPE_INVOICE_LOGO);
            $invoice->image = $fileName;
        }
        $invoice->save();

        $oldItems = InvoiceItem::where('invoice_id', $invoice->id)->delete();
        $items = ($req->input('items'));
        foreach ($items as $item) {
            $invoiceItem = new InvoiceItem((array)$item);
            $invoiceItem->invoice_id = $invoice->id;
            $invoiceItem->save();
        }
        $invoice->append('invoice_items');
        $invoice->append('invoice_payments');
        return $invoice;
    }

    /**
     * Update Status invoice.
     * POST /api/invoices/{id}/status-change
     */
    public function update_status(Request $req, $id)
    {
        $invoice = Invoice::find($id);
        if (empty($invoice)) Utils::error('Invoice not found!', 404);

        $this->validate($req, [
            'invoice_type' => 'required|in:Internal,External,Customer',
            'business_id' => 'exists:businesses,id',
            'currency_symbol' => 'required|string',
            'header' => 'string',
            'bill_to' => 'string',
            'ship_to' => 'string',
            'date' => 'required|date',
            'sub_total' => 'required|numeric',
            'grand_total' => 'required|numeric',
            'tax_mode' => 'in:Flat,Percentage',
            'tax' => 'numeric',
            'discount_mode' => 'in:Flat,Percentage',
            'discount' => 'numeric',
            'notes' => 'string',
            'terms' => 'string',
            'status' => 'in:Unpaid,Paid,Cancel,Dispute,Partial',
            'paid_amount' => 'numeric',
            'mobile_number' => 'string',
            'remark' => 'string',
            'archived' => 'boolean',
            'recurring' => 'boolean',
            'recurring_days' => 'required_if:recurring,1|string',
            'recurring_date' => 'string',
        ]);

        if ($req->input('payment_mode') == 'Link') {
            // create invoice_pay_link
            $invoice_pay_link = new InvoicePayLink();
            $invoice_pay_link->fill($req->all());
            if ($req->has('amount')) {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                if ($mode == 'test') {
                    $columns = 'connect_stripe_public_key_test';
                    $columns1 = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $columns = 'connect_stripe_public_key_live';
                    $columns1 = 'connect_stripe_private_key_live';
                }
                $publishkey = SiteSettingController::findByKey($columns)->value;
                $secretkey = SiteSettingController::findByKey($columns1)->value;
                $invoice_pay_link->publishkey = $publishkey;
                $invoice_pay_link->secretkey = $secretkey;
                $invoice_pay_link->invoice_id = $invoice->id;
                $invoice_pay_link->status = 'Pending';
            }

            // Rendom Shuffle Code Generate
            $arr = str_split('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789');
            shuffle($arr);
            $arr = array_slice($arr, 0, 9);
            $random = implode('', $arr);
            $invoice_pay_link->shortn = $random;
            // $invoice_pay_link->name = ;
            // $invoice_pay_link->email = $bussiness->email;
            $invoice_pay_link->mobile_number  = $req->input('mobile_number');
            $invoice_pay_link->description = 'Ubsidi Invoice INV000' . $invoice->id;
            $invoice_pay_link->descriptor = 'Ubsidi Invoice INV000' . $invoice->id;
            $invoice_pay_link->amount =  $req->input('amount');
            $invoice_pay_link->save();

            $send_url = env('APP_URL') . 'invoice-payment?code=' . $random . '&'; // http://192.168.1.23:8300/invoice-payment?code=dZCMEwjIO&

            $message = 'This is payment request sent, please click link below to complete your payment.' . $send_url;
            (new OtpHelper())->sendMessage($message, $req->input('mobile_number'), 'payment_sms_link');
        }

        // update invoice
        if ($req->has('invoice_type')) $invoice->invoice_type = $req->input('invoice_type');
        if ($req->has('business_id')) $invoice->business_id = $req->input('business_id');
        if ($req->has('currency_symbol')) $invoice->currency_symbol = $req->input('currency_symbol');
        if ($req->has('header')) $invoice->header = $req->input('header');
        if ($req->has('bill_to')) $invoice->bill_to = $req->input('bill_to');
        if ($req->has('ship_to')) $invoice->ship_to = $req->input('ship_to');
        if ($req->has('date')) $invoice->date = $req->input('date');
        if ($req->has('sub_total')) $invoice->sub_total = $req->input('sub_total');
        if ($req->has('grand_total')) $invoice->grand_total = $req->input('grand_total');
        if ($req->has('tax_mode')) $invoice->tax_mode = $req->input('tax_mode');
        if ($req->has('tax')) $invoice->tax = $req->input('tax');
        if ($req->has('discount_mode')) $invoice->discount_mode = $req->input('discount_mode');
        if ($req->has('discount')) $invoice->discount = $req->input('discount');
        if ($req->has('notes')) $invoice->notes = $req->input('notes');
        if ($req->has('terms')) $invoice->terms = $req->input('terms');
        if ($req->has('status') && $req->input('status') != 'Paid' && $req->input('status') != 'Partial') $invoice->status = $req->input('status');
        if ($req->has('paid_amount')) $invoice->paid_amount = $req->input('paid_amount');
        if ($req->has('mobile_number')) $invoice->mobile_number = $req->input('mobile_number');
        if ($req->has('remark')) $invoice->remark = $req->input('remark');
        $invoice->archived = $req->input('archived');
        $invoice->recurring = $req->input('recurring');
        if ($req->has('recurring_days')) $invoice->recurring_days = $req->input('recurring_days');
        if ($req->has('recurring_days')) {
            $invoice->recurring_date = Carbon::today()->addDays($req->input('recurring_days'))->format('Y-m-d');
        }
        $invoice->save();

        if ($req->has('amount')) {
            $invoicePayment = new InvoicePayment();
            if ($req->has('amount')) $invoicePayment->amount = $req->input('amount');
            if ($req->has('payment_mode')) $invoicePayment->payment_mode = $req->input('payment_mode');
            $invoicePayment->invoice_id = $invoice->id;
            $invoicePayment->updater_id = Auth::id();
            $invoicePayment->archived = $req->input('archived');
            if ($req->input('payment_mode') == 'Link') {
                $invoicePayment->reference = $send_url;
            } else {
                if ($req->has('reference')) $invoicePayment->reference = $req->input('reference');
            }
            $invoicePayment->save();

            // Invoice Payment Status Change 
            if ($req->input('payment_mode') != 'Link') {
                $invoice->paid_amount = $invoice->paid_amount + $invoicePayment->amount;
            }
            if ($invoice->paid_amount >=  $invoice->grand_total) {
                $invoice->status = 'Paid';
            } else {
                $invoice->status = 'Partial';
            }
            $invoice->save();
        }

        $invoice->append('invoice_items');
        $invoice->append('invoice_payments');
        return $invoice;
    }

    /**
     * mail send invoice by id.
     * POST /api/invoices/{id}/send-message
     */
    public function send_message(Request $req, $id)
    {
        $this->validate($req, [
            'mobile_number' => 'required|string',
        ]);

        // create shortln_url
        $shortln_url = new TiffintomShortlnUrl();
        if ($req->input('business_id') != '') {
            $bussiness = Business::where('id', $req->input('business_id'))->first();
            if (!empty($bussiness)) {
                if ($bussiness->connect_service == 1) {
                    $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                    if ($mode == 'test') {
                        $columns = 'connect_stripe_public_key_test';
                        $columns1 = 'connect_stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $columns = 'connect_stripe_public_key_live';
                        $columns1 = 'connect_stripe_private_key_live';
                    }
                    $publishkey = SiteSettingController::findByKey($columns)->value;
                    $secretkey = SiteSettingController::findByKey($columns1)->value;
                    $shortln_url->publishkey = $publishkey;
                    $shortln_url->secretkey = $secretkey;
                } else {
                    $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                    if ($mode == 'test') {
                        $columns = 'stripe_public_key_test';
                        $columns1 = 'stripe_private_key_test';
                    }
                    if ($mode == 'live') {
                        $columns = 'stripe_public_key_live';
                        $columns1 = 'stripe_private_key_live';
                    }
                    $publishkey = SiteSettingController::findByKey($columns)->value;
                    $secretkey = SiteSettingController::findByKey($columns1)->value;
                    $shortln_url->publishkey = $publishkey;
                    $shortln_url->secretkey = $secretkey;
                }
            }
        }

        // Rendom Shuffle Code Generate
        $arr = str_split('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789');
        shuffle($arr);
        $arr = array_slice($arr, 0, 9);
        $random = implode('', $arr);
        $shortln_url->shortn = $random;
        $shortln_url->name = $bussiness->name;
        $shortln_url->email = $bussiness->email;
        $shortln_url->phone  = $req->input('mobile_number');
        $shortln_url->description = $bussiness->statement_description;
        $shortln_url->descriptor = $bussiness->statement_descriptor;
        $shortln_url->amount =  $req->input('grand_total');
        $shortln_url->save();

        $send_url = env('APP_URL') . 'invoice-payment?code=' . $random . '&';
        $message = 'This is payment request sent from ' . $bussiness->name . ', please click link below to complete your payment.' . $send_url;
        (new OtpHelper())->sendMessage($message, $req->input('mobile_number'), 'payment_sms_link');

        return ['message' => 'message send successfully.'];
    }

    /**
     * mail send invoice by id.
     * POST /api/invoices/{id}/send-mail
     */
    public function send_mail(Request $req, $id)
    {
        $this->validate($req, [
            'email' => 'required|string',
        ]);

        // $this->makePdf($req->input('id'));

        $this->sendMail($req->input('email'), $req->input('id'), $req->input('message'));

        return ['message' => 'mail send successfully.'];
    }

    public function print(Request $req, $id)
    {
        $invoice = Invoice::find($id);
        if (empty($invoice)) Utils::error('Invoice not found!', 404);
        $invoicePaymentLinks = InvoicePayment::where('invoice_id', $invoice->id)->where('payment_mode', 'Link')->get();
        $invoice->append('invoice_items');
        $invoice->append('invoice_payments');
        $invoice->append('business');
        $fileName = 'INV000' . $invoice->id . '-' . $invoice->date;

        $html = '<html>';
        $html = $html . '<head>';
        $html = $html . '<style>
        * {
            margin: 0;
            padding: 0;
            font-family: "Verdana";
            font-size: 9pt;
            color: #000;
        }
        body {
            width: 100%;
            font-family: "Verdana";
            font-size: 9pt;
            margin: 0;
            padding: 0;
        }
        table {
            border-spacing: 0;
            border-collapse: collapse;
            font-size:15px;
        }
        .info-table  tr td{
            padding:1px;
        }
        .info-table tr td:nth-child(even){
            text-align:right;
        }
        .items-table{
            margin-top:20px;
            width:100%;
        }
        .items-table tr th{
            padding: 4px;
            font-size: 16px; 
            border: 1px solid #fff;
            background-color:#393939;
            font-weight:600;
            color:#fff;
            text-align:center;
        }
        .items-table tr td{
            font-size: 15px; 
            padding: 4px;
            border: 1px solid #fff;
        }
        img{
            -ms-interpolation-mode:bicubic;
        }';

        $html = $html . '</style>';

        $html = $html . '</head>';
        $html = $html . '<body>';
        $html = $html . '';

        $html = $html . '<table style="width:100%">';
        $html = $html . '<tr>';
        if ($invoice->invoice_type == 'Customer' && $invoice->image_url != '') {
            $html = $html . '<td style="text-align:left;"> <img align="center" alt="" src="' . $invoice->image_url . '" width="150.12" style="max-width:1200px; padding-bottom: 0; display: inline !important; vertical-align: bottom;" class="mcnImage"></td>';
        } else {
            $html = $html . '<td style="text-align:left;"> <img align="center" alt="" src="https://www.ubsidi.com/V2/public/images/ubsidi-logo-color.png" width="150.12" style="max-width:1200px; padding-bottom: 0; display: inline !important; vertical-align: bottom;" class="mcnImage"></td>';
        }
        $html = $html . '<td style="text-align:right;"><p style="text-align:right;font-weight:bold;font-size: 0.5cm;">Invoice</p># ' . $invoice->id . '-UBSIDI</td>';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<br>';

        $html = $html . '<table style="width:100%">';
        $html = $html . '<tr>';
        $html = $html . '<td style="text-align:left;">' . nl2br($invoice->header) . '</td>';
        $html = $html . '<td style="text-align:right;">' . Carbon::parse($invoice->date)->format('F d, Y') . '</td>';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<br>';

        $html = $html . '<table style="width:100%">';
        $html = $html . '<tr>';
        $html = $html . '<td> <b>Bill To:</b><br>' . nl2br($invoice->bill_to) . '</td>';
        if (isset($invoice->ship_to)) {
            $html = $html . '<td> <b>Ship To:</b><br>' . nl2br($invoice->ship_to) . '</td>';
        } else {
            $html = $html . '<td></td>';
        }
        $html = $html . '<td></td>';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<br>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Item</th>';
        $html = $html . '<th>Quantity</th>';
        $html = $html . '<th>Rate</th>';
        $html = $html . '<th>Total</th>';
        $html = $html . '</tr>';
        $srNo = 1;
        foreach ($invoice->invoice_items as $item) {
            $html = $html . '<tr>';
            $html = $html . '<td>' . $item->description . '</td>';
            $html = $html . '<td>' . $item->quantity . '</td>';
            $html = $html . '<td>' . $invoice->currency_symbol . '' . $item->rate . '</td>';
            $html = $html . '<td>' . $invoice->currency_symbol . '' . $item->total . '</td>';
            $html = $html . '</tr>';
            $srNo++;
        }

        $html = $html . '<tr>';
        $html = $html . '<td colspan="3" style="text-align:right;">Subtotal :</td>';
        $html = $html . '<td>' . $invoice->currency_symbol . ' ' . $invoice->sub_total . '</td>';
        $html = $html . '</tr>';
        if ($invoice->tax_mode == 'Percentage') {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;">Tax :</td>';
            $html = $html . '<td>' . $invoice->tax . '%</td>';
            $html = $html . '</tr>';
        } else {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;">Tax :</td>';
            $html = $html . '<td>' . $invoice->currency_symbol . '' . $invoice->tax . '</td>';
            $html = $html . '</tr>';
        }

        if ($invoice->discount_mode == 'Percentage') {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;">Discount :</td>';
            $html = $html . '<td>' . $invoice->discount . '%</td>';
            $html = $html . '</tr>';
        } else {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;">Discount :</td>';
            $html = $html . '<td>' . $invoice->currency_symbol . '' . $invoice->discount . '</td>';
            $html = $html . '</tr>';
        }

        $html = $html . '<tr>';
        $html = $html . '<td colspan="3" style="text-align:right;">Total :</td>';
        $html = $html . '<td>' . $invoice->currency_symbol . '' . $invoice->grand_total . '</td>';
        $html = $html . '</tr>';

        if ($invoice->status == "Paid") {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3"><b>Status</b></td>';
            $html = $html . '<td style="color:green;"><b>PAID</b></td>';
            $html = $html . '</tr>';
        } else if ($invoice->status == "Unpaid") {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;"><b>Status</b> :</td>';
            $html = $html . '<td style="color:red;"><b>UNPAID</b></td>';
            $html = $html . '</tr>';
        } else if ($invoice->status == "Cancel") {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;"><b>Status</b> :</td>';
            $html = $html . '<td style="color:red;"><b>CANCEL</b></td>';
            $html = $html . '</tr>';
        } else if ($invoice->status == "Partial") {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;"><b>Status</b> :</td>';
            $html = $html . '<td style="color:blue;"><b>PARTIAL</b></td>';
            $html = $html . '</tr>';
        } else {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;"><b>Status</b> :</td>';
            $html = $html . '<td style="color:blue;"><b>DISPUTE</b></td>';
            $html = $html . '</tr>';
        }
        $html = $html . '</table>';


        if (isset($invoice->notes)) {
            $html = $html . '<br>';
            $html = $html . '<table style="width:100%">';
            $html = $html . '<tr>';
            $html = $html . '<td> <b>Notes:</b><br>' . nl2br($invoice->notes) . '</td>';
            $html = $html . '</tr>';
            $html = $html . '</table>';
        }

        if (isset($invoice->terms)) {
            $html = $html . '<br>';
            $html = $html . '<table style="width:100%">';
            $html = $html . '<tr>';
            $html = $html . '<td> <b>Terms:</b><br>' . nl2br($invoice->terms) . '</td>';
            $html = $html . '</tr>';
            $html = $html . '</table>';
        }

        if (sizeof($invoicePaymentLinks) > 0) {
            $html = $html . '<br>';
            $html = $html . '<table style="width:100%">';
            $html = $html . '<tr>';
            $html = $html . '<td> <b>Payment Link:</b></td>';
            $html = $html . '</tr>';

            foreach ($invoicePaymentLinks as $invoicePaymentLink) {
                $html = $html . '<tr>';
                $html = $html . '<td> ' . str_replace(
                    '<?xml version="1.0" encoding="UTF-8"?>',
                    '',
                    $this->generateQRCodeImage($invoicePaymentLink->reference)
                ) . '</td>';
                $html = $html . '</tr>';
            }
            $html = $html . '</table>';
        }


        $html = $html . '</body>';
        $html = $html . '

</html>';

        $mpdf = new Mpdf();
        $mpdf->WriteHTML($html);
        return $mpdf->Output("$fileName.pdf", "D");
    }

    public function generateQRCodeImage($link)
    {
        $qrCode = QrCode::size(100)->generate($link);
        // $qrCode = $qrCode->header('Content-Type', 'image/png');
        return $qrCode; // ->header('Content-Type', 'image/png')
    }

    /**
     * Delete invoice by id.
     * DELETE /api/invoices/{id}
     */
    public function delete(Request $req, $id)
    {
        $invoice = Invoice::find($id);
        if (empty($invoice)) Utils::error('Invoice not found!', 404);
        $invoice->delete();
        return $invoice;
    }

    public function makePdf($id)
    {
        $invoice = Invoice::find($id);
        if (empty($invoice)) Utils::error('Invoice not found!', 404);
        $invoice->append('invoice_items');
        $invoice->append('invoice_payments');
        $invoice->append('business');
        $fileName = $invoice['business']['name'];

        $html = '<html>';
        $html = $html . '

<head>';
        $html = $html . '<style>
    * {
        margin: 0;
        padding: 0;
        font-family: "Verdana";
        font-size: 9pt;
        color: #000;
    }

    body {
        width: 100%;
        font-family: "Verdana";
        font-size: 9pt;
        margin: 0;
        padding: 0;
    }

    table {
        border-spacing: 0;
        border-collapse: collapse;
        font-size: 15px;

    }

    .info-table tr td {
        padding: 2px;
    }

    .info-table tr td:nth-child(even) {
        text-align: right;
    }

    .items-table {
        margin-top: 20px;
        width: 100%;
        text-align: center;
    }

    .items-table tr th {
        padding: 8px;
        font-size: 16px;
        border: 1px solid #73C6B6;
        font-weight: 600;
        background-color: #393939;
        color: #fff;
    }

    .items-table tr td {
        font-size: 15px;
        padding: 8px;
        border: 1px solid #73C6B6;
    }

    .items-table tr:nth-child(even) {
        background-color: #E8F6F3
    }

    .items-table tr:nth-child(odd) {
        background-color: #ffffff
    }

    ';

        $html = $html . '
    </style>';
        $html = $html . '</head>';
        $html = $html . '

<body>';
        $html = $html . '<h2 style="text-align:right">Invoice</h2>';
        $html = $html . '<table style="width:100%" class="info-table">';

        $html = $html . '<tr>';
        $html = $html . '<td style="font-weight: 900;">Simple Merchant Payment Services</td>';
        $html = $html . '<td>' . Carbon::parse($invoice->date)->format('F d, Y') . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td></td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>66 Little Caldmore</td>';
        $html = $html . '<td>
                <h4>' . $invoice->business->name . '</h4>
            </td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Walsall</td>';
        $html = $html . '<td>' . $invoice->business->address . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>WS1 3RA</td>';
        $html = $html . '<td>' . $invoice->business->country->name . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Email: admin@smartmps.co.uk</td>';
        $html = $html . '<td>Email: ' . $invoice->business->email . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Tel: 08712 884 044</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Sr No.</th>';
        $html = $html . '<th>Description</th>';
        $html = $html . '<th>Rate</th>';
        $html = $html . '<th>Quantity</th>';
        $html = $html . '<th>Total</th>';
        $html = $html . '</tr>';
        $srNo = 1;
        foreach ($invoice->invoice_items as $item) {
            $html = $html . '<tr>';
            $html = $html . '<td>' . $srNo . '</td>';
            $html = $html . '<td>' . $item->description . '</td>';
            $html = $html . '<td>' . $item->rate . '</td>';
            $html = $html . '<td>' . $item->quantity . '</td>';
            $html = $html . '<td>' . $item->total . '</td>';
            $html = $html . '</tr>';
            $srNo++;
        }
        $html = $html . '<tr>';
        $html = $html . '<th colspan="4" style="text-align:right;">Grand Total:</th>';
        $html = $html . '<th>' . $invoice->grand_total . '</th>';
        $html = $html . '</tr>';
        if ($invoice->status != "Paid") {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="4" style="text-align:right;">Paid Amount:</td>';
            $html = $html . '<td>' . $invoice->total_paid_amount . '</td>';
            $html = $html . '</tr>';
        } else {
            $html = $html . '<tr>';
            $html = $html . '<th colspan="5">Invoice Paid</th>';
            $html = $html . '</tr>';
        }


        $html = $html . '</table>';
        $html = $html . '</body>';
        $html = $html . '

</html>';


        $mpdf = new Mpdf();
        $mpdf->SetHeader('Invoice');
        $mpdf->SetFooter('{PAGENO}');
        $mpdf->defaultheaderfontsize = 10;
        $mpdf->defaultheaderfontstyle = 'B';
        $mpdf->defaultheaderline = 0;
        $mpdf->defaultfooterfontsize = 10;
        $mpdf->defaultfooterfontstyle = 'BI';
        $mpdf->defaultfooterline = 0;
        $mpdf->showWatermarkImage = true;
        $mpdf->WriteHTML($html);
        $fileName = str_replace(' ', '', $fileName);
        $path = $fileName . '.pdf';
        $mpdf->Output('images/' . $path, "S");
        $file = storage_path() . '/app/public/' . $fileName . '.pdf';
        $file_local = Storage::disk('public')->url("images/$fileName.pdf");
        Storage::disk('ubsidi')->put(ImageHelper::$TYPE_MISC_INVOICE . '/' . $path, $file_local);
    }

    public function sendMail($sender_email, $id, $message)
    {
        $invoice = Invoice::find($id);
        if (empty($invoice)) Utils::error('Invoice not found!', 404);
        $invoice->append('invoice_items');
        $invoice->append('invoice_payments');
        $invoice->append('business');
        $fileName = $invoice['business']['name'];

        $siteSettings = TiffintomSiteSetting::find('1');
        if (empty($siteSettings)) Utils::error("No site-settings found", 404);

        // Customize Invoice mail
        $html = '<html>';
        $html = $html . '

<head>';
        $html = $html . '<style>
    * {
        margin: 0;
        padding: 0;
        font-family: "Verdana";
        font-size: 9pt;
        color: #000;
    }

    body {
        width: 100%;
        font-family: "Verdana";
        font-size: 9pt;
        margin: 0;
        padding: 0;
    }

    table {
        border-spacing: 0;
        border-collapse: collapse;
        font-size: 15px;
    }

    .info-table tr td {
        padding: 1px;
    }

    .info-table tr td:nth-child(even) {
        text-align: right;
    }

    .items-table {
        margin-top: 20px;
        width: 100%;
        text-align: center;
    }

    .items-table tr th {
        padding: 8px;
        font-size: 16px;
        border: 1px solid #fff;
        background-color: #393939;
        font-weight: 600;
        color: #fff;
    }

    .items-table tr td {
        font-size: 15px;
        padding: 4px;
        border: 1px solid #fff;
    }

    img {
        -ms-interpolation-mode: bicubic;
    }

    ';

        $html = $html . '
    </style>';

        $html = $html . '</head>';
        $html = $html . '

<body>';
        $html = $html . '';

        $html = $html . '<table style="width:100%">';
        $html = $html . '<tr>';
        $html = $html . '<td style="text-align:left;"> <img align="center" alt=""
                    src="https://www.ubsidi.com/V2/public/images/ubsidi-logo-color.png" width="150.12"
                    style="max-width:1200px; padding-bottom: 0; display: inline !important; vertical-align: bottom;"
                    class="mcnImage"></td>';
        $html = $html . '<td style="text-align:right;">
                <p style="text-align:right;font-weight:bold;font-size: 0.5cm;">Invoice</p># ' . $invoice->id . '-UBSIDI
            </td>';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<br>';

        $html = $html . '<table style="width:100%">';
        $html = $html . '<tr>';
        $html = $html . '<td style="text-align:left;">' . nl2br($invoice->header) . '</td>';
        $html = $html . '<td style="text-align:right;">' . Carbon::parse($invoice->date)->format('F d, Y') . '</td>
            ';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<br>';

        $html = $html . '<table style="width:100%">';
        $html = $html . '<tr>';
        $html = $html . '<td> Bill To:<br>' . nl2br($invoice->bill_to) . '</td>';
        if (isset($invoice->ship_to)) {
            $html = $html . '<td> Ship To:<br>' . nl2br($invoice->ship_to) . '</td>';
        } else {
            $html = $html . '<td></td>';
        }
        $html = $html . '<td></td>';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<br>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Item</th>';
        $html = $html . '<th>Quantity</th>';
        $html = $html . '<th>Rate</th>';
        $html = $html . '<th>Total</th>';
        $html = $html . '</tr>';
        $srNo = 1;
        foreach ($invoice->invoice_items as $item) {
            $html = $html . '<tr>';
            $html = $html . '<td>' . $item->description . '</td>';
            $html = $html . '<td>' . $item->quantity . '</td>';
            $html = $html . '<td>' . $invoice->currency_symbol . '' . $item->rate . '</td>';
            $html = $html . '<td>' . $invoice->currency_symbol . '' . $item->total . '</td>';
            $html = $html . '</tr>';
            $srNo++;
        }

        $html = $html . '<tr>';
        $html = $html . '<td colspan="3" style="text-align:right;">Subtotal :</td>';
        $html = $html . '<td>' . $invoice->currency_symbol . ' ' . $invoice->sub_total . '</td>';
        $html = $html . '</tr>';
        if ($invoice->tax_mode == 'Percentage') {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;">Tax :</td>';
            $html = $html . '<td>' . $invoice->tax . '%</td>';
            $html = $html . '</tr>';
        } else {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;">Tax :</td>';
            $html = $html . '<td>' . $invoice->currency_symbol . '' . $invoice->tax . '</td>';
            $html = $html . '</tr>';
        }

        if ($invoice->discount_mode == 'Percentage') {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;">Discount :</td>';
            $html = $html . '<td>' . $invoice->discount . '%</td>';
            $html = $html . '</tr>';
        } else {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;">Discount :</td>';
            $html = $html . '<td>' . $invoice->currency_symbol . '' . $invoice->discount . '</td>';
            $html = $html . '</tr>';
        }

        $html = $html . '<tr>';
        $html = $html . '<td colspan="3" style="text-align:right;">Total :</td>';
        $html = $html . '<td>' . $invoice->currency_symbol . '' . $invoice->grand_total . '</td>';
        $html = $html . '</tr>';

        if ($invoice->status != "Paid") {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3" style="text-align:right;">Status :</td>';
            $html = $html . '<td>' . $invoice->status . '</td>';
            $html = $html . '</tr>';
        } else {
            $html = $html . '<tr>';
            $html = $html . '<td colspan="3"></td>';
            $html = $html . '<td>Invoice Paid</td>';
            $html = $html . '</tr>';
        }
        $html = $html . '</table>';


        if (isset($invoice->notes)) {
            $html = $html . '<br>';
            $html = $html . '<table style="width:100%">';
            $html = $html . '<tr>';
            $html = $html . '<td> Notes:<br>' . nl2br($invoice->notes) . '</td>';
            $html = $html . '</tr>';
            $html = $html . '</table>';
        }

        if (isset($invoice->terms)) {
            $html = $html . '<br>';
            $html = $html . '<table style="width:100%">';
            $html = $html . '<tr>';
            $html = $html . '<td> Terms:<br>' . nl2br($invoice->terms) . '</td>';
            $html = $html . '</tr>';
            $html = $html . '</table>';
        }

        $html = $html . '</body>';
        $html = $html . '

</html>';

        $mpdf = new Mpdf();
        $mpdf->SetHeader('Invoice');
        $mpdf->SetFooter('{PAGENO}');
        $mpdf->defaultheaderfontsize = 10;
        $mpdf->defaultheaderfontstyle = 'B';
        $mpdf->defaultheaderline = 0;
        $mpdf->defaultfooterfontsize = 10;
        $mpdf->defaultfooterfontstyle = 'BI';
        $mpdf->defaultfooterline = 0;
        $mpdf->showWatermarkImage = true;
        $mpdf->WriteHTML($html);
        $fileName = str_replace(' ', '', $fileName);
        $pdf = $mpdf->Output('', 'S');
        $attachment = chunk_split(base64_encode($pdf));

        $emailinfo = '<table>
    <tr>
        <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
            <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="max-width:767px;">
                <tr>
                    <td align="center" valign="top" style="font-size:0; padding: 15px;" bgcolor="#09c4a9">
                        <div style="display:inline-block; vertical-align:middle; text-align:center">
                            <table align="left" border="0" cellpadding="0" cellspacing="0" width="100%"
                                style="max-width:300px;">
                                <tr>
                                    <td align="left" valign="top"
                                        style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 34px; font-weight: 800;">
                                        <img width="170" src="' . env('APP_URL') . 'images/ubsidi-logo-color.png">
                                    </td>
                                </tr>
                            </table>
                        </div>
                    </td>
                </tr>';

        $emailinfo .= ' <tr>
                    <td align="center" style="background-color: #eeeeee;" bgcolor="#eeeeee">
                        <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%"
                            style="max-width:767px;">
                            <tr>
                                <td align="center" style="padding: 20px; background-color: #ffffff;" bgcolor="#ffffff">
                                    <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%"
                                        style="max-width:670px;">
                                        <tr>
                                            <td align="left"
                                                style="font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px; padding-top: 10px;">
                                                ' . $message . '
                                            </td>
                                        </tr>
                                    </table>
                                </td>
                            </tr>
                        </table>
                    </td>
                </tr>';
        $emailinfo .= ' <tr>
                    <td bgcolor="#fff">
                        <table role="presentation" cellspacing="0" cellpadding="0" border="0" width="100%">
                            <tr>
                                <td style="padding: 30px 30px; text-align: right;">
                                    <table align="left" style="text-align: left;">
                                        <tr>
                                            <td>
                                                <p style="color:#000;"> Get The App : </p>
                                            </td>
                                            <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://play.google.com/store/apps/details?id=com.tiffintom"
                                                    target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/playstore.png"
                                                        style="margin:0; padding:0; border:none; display:block;"
                                                        border="0" alt=""></a>
                                            </td>
                                            <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://itunes.apple.com/in/app/tiffintom/id1437199436?mt=8"
                                                    target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/appstore.png"
                                                        style="margin:0; padding:0; border:none; display:block;"
                                                        border="0" alt=""></a>
                                            </td>
                                            <td width="10">&nbsp;</td>
                                        </tr>

                                    </table>
                                    <table align="right" style="text-align: right;">
                                        <tr>
                                            <td>
                                                <p style="color:#000;">Follow Us : </p>
                                            </td>
                                            <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://www.facebook.com/tiffintom/" target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/facebook.png"
                                                        style="margin:0; padding:0; border:none; display:block;"
                                                        border="0" alt=""></a>
                                            </td>
                                            <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://twitter.com/TiffinTomUK/" target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/twitter.png"
                                                        style="margin:0; padding:0; border:none; display:block;"
                                                        border="0" alt=""></a>
                                            </td>
                                            <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://in.linkedin.com/company/tiffin-tom-limited/"
                                                    target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/linkedin.png"
                                                        style="margin:0; padding:0; border:none; display:block;"
                                                        border="0" alt=""></a>
                                            </td>
                                            <td width="10">&nbsp;</td>
                                            <td>
                                                <a href="https://www.instagram.com/tiffintomuk/" target="_blank">
                                                    <img src="' . env('APP_URL') . 'images/Instragram.png"
                                                        style="margin:0; padding:0; border:none; display:block;"
                                                        border="0" alt=""></a>
                                            </td>
                                            <td width="10">&nbsp;</td>
                                        </tr>
                                    </table>
                                </td>
                            </tr>
                        </table>
                    </td>
                </tr>
                <tr>
                    <td align="center" valign="top" style="font-size:0; padding: 10px;" bgcolor="#09c4a9">
                        <div
                            style="color:#000;font-family: Open Sans, Helvetica, Arial, sans-serif; font-size: 16px; font-weight: 400; line-height: 24px;">
                            Copyrights @ ' . date('Y') . '</div>
                    </td>
                </tr>
            </table>
        </td>
    </tr>
</table>';

        $separator = md5(time());
        $eol = PHP_EOL;
        $filename = $fileName . ".pdf";

        $headers = '';
        $headers .= "MIME-Version: 1.0" . $eol;
        $headers .= "Content-Type: multipart/mixed; boundary=\"" . $separator . "\"" . $eol . $eol;

        $body = "Content-Transfer-Encoding: 7bit" . $eol;
        $body .= "This is a MIME encoded message." . $eol;
        $body .= "--" . $separator . $eol;
        $body .= "Content-Type: text/html; charset=\"UTF-8\"" . $eol;
        $body .= "Content-Transfer-Encoding: 8bit" . $eol . $eol;
        $body .= $emailinfo . $eol;
        $body .= "--" . $separator . $eol;
        $body .= "Content-Type: application/octet-stream; name=\"" . $filename . "\"" . $eol;
        $body .= "Content-Transfer-Encoding: base64" . $eol;
        $body .= "Content-Disposition: attachment" . $eol . $eol;
        $body .= $attachment . $eol;
        $emailinfo .= "--" . $separator . "--";

        $email = array(
            'body' => $body,
            'subject' => 'Invoice',
            'mail_type' => $siteSettings->mail_option,
            'to' => $sender_email,
            'from' => $siteSettings->admin_email,
            'header_pdf' => $headers,
        );

        (new MailHelper())->sendMail($email);

        return 'Mail Send Successfully';
    }
}
