<?php

namespace App\Http\Controllers;

use App\Business;
use App\CardHolder;
use App\IssuingCard;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;
use Stripe\StripeClient;

class IssuingCardController extends Controller
{
    /**
     * List issuingCards.
     * GET /api/issuing-cards
     */
    public function index(Request $req)
    {
        $issuingCards = IssuingCard::orderByRaw('id ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $issuingCards->whereRaw('(last4 LIKE ? OR brand LIKE ? OR service LIKE ? )', ["%$query%", "%$query%", "%$query%"]);
        }

        if ($business_id = $req->query('business_id', false)) {
            $issuingCards->where('business_id', $business_id);
        }

        if ($req->query('nopaginate', false) == '1') {
            $issuingCards = $issuingCards->get();
        } else {
            $issuingCards = $issuingCards->paginate($req->query('per_page', 20));
        }

        if ($issuingCards->isEmpty()) Utils::error('No issuing cards found!', 404);


        return ($issuingCards);
    }

    /**
     * Create issuingCard.
     * POST /api/issuing-cards
     */
    public function create(Request $req)
    {
        if (!is_array($req->card_holder))
            $req->merge(['card_holder' => (json_decode($req->card_holder, true))]);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'cardholder_id' => 'string|exists:card_holders,id',
            'card_id' => 'string|max:255',
            'brand' => 'string|max:255',
            'cancellation_reason' => 'string|max:255',
            'currency' => 'string|max:255',
            'exp_month' => 'string|max:255',
            'exp_year' => 'string|max:255',
            'last4' => 'string|max:255',
            'livemode' => 'boolean',
            'replaced_by' => 'string|max:255',
            'replacement_for' => 'string|max:255',
            'replacement_reason' => 'string|max:255',
            'type' => 'string|in:physical,virtual',
            'status' => 'string|in:active,inactive',
            'service' => 'required_if:type,physical|string|in:standard,express,priority',
            'shipping_type' => 'required_if:type,physical|string|in:bulk,individual',
            'city' => 'required_if:type,physical|string|max:255',
            'country' => 'required_if:type,physical|string|max:255',
            'line1' => 'required_if:type,physical|string|max:255',
            'postal_code' => 'required_if:type,physical|string|max:255',
            'line2' => 'string|max:255',
            'state' => 'string|max:255',
        ]);

        // create issuingCard
        $issuingCard = new IssuingCard;
        $issuingCard->fill($req->all());

        try {

            $business = Business::where('id', $req->input('business_id'))->first();
            if ($business->connect_service == 1 && $business->s_account_id != '') {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            } else {
                $mode = SiteSettingController::findByKey('stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            }
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

            $card_holder = $req->input('card_holder');

            $cardHolder = $stripe->issuing->cardholders->create([
                'type' => $card_holder['type'],
                'name' => $card_holder['name'],
                'email' => $card_holder['email'],
                'phone_number' => $card_holder['phone_number'],
                'status' => 'active',
                'billing' => [
                    'address' => [
                        'line1' => $card_holder['line1'],
                        'city' => $card_holder['city'],
                        'state' => $card_holder['state'],
                        'country' => 'GB',
                        'postal_code' => $card_holder['postal_code'],
                    ],
                ],
            ]);


            if ($req->input('type') == 'physical') {
                $card = $stripe->issuing->cards->create([
                    'cardholder' => $cardHolder->id,
                    'currency' => 'gbp',
                    'type' => $req->input('type'),
                    'status' => 'active',
                    'shipping' => [
                        'address' => [
                            'line1' => $req->input('line1'),
                            'city' => $req->input('city'),
                            'state' => $req->input('state'),
                            'country' => 'GB',
                            'postal_code' => $req->input('postal_code'),
                        ],
                        'name' => $card_holder['name'],
                        'service' => $req->input('service'),
                        'type' => $req->input('shipping_type'),
                    ]
                ]);
            } else {
                $card = $stripe->issuing->cards->create([
                    'cardholder' => $cardHolder->id,
                    'currency' => 'gbp',
                    'type' => $req->input('type'),
                    'status' => 'active'
                ]);
            }
        } catch (Exception $e) {
            Utils::error($e, 404);
        }

        $cardHolders = new CardHolder();
        $cardHolders->business_id = $req->input('business_id');
        $cardHolders->holder_id = $cardHolder->id;
        $cardHolders->name = $cardHolder->name;
        $cardHolders->email = $cardHolder->email;
        $cardHolders->phone_number = $cardHolder->phone_number;
        $cardHolders->city = $cardHolder->billing->address->city;
        $cardHolders->country = $cardHolder->billing->address->country;
        $cardHolders->line1 = $cardHolder->billing->address->line1;
        $cardHolders->postal_code = $cardHolder->billing->address->postal_code;
        $cardHolders->line2 = $cardHolder->billing->address->line2;
        $cardHolders->state = $cardHolder->billing->address->state;
        $cardHolders->company = $cardHolder->company;
        $cardHolders->individual = $cardHolder->individual;
        $cardHolders->livemode = $cardHolder->livemode;
        $cardHolders->type = $cardHolder->type;
        $cardHolders->status = $cardHolder->status;
        $cardHolders->save();

        $issuingCard->cardholder_id = $cardHolders->id;
        $issuingCard->card_id = $card->id;
        $issuingCard->brand = $card->brand;
        $issuingCard->cancellation_reason = $card->cancellation_reason;
        $issuingCard->currency = $card->currency;
        $issuingCard->exp_month = $card->exp_month;
        $issuingCard->exp_year = $card->exp_year;
        $issuingCard->last4 = $card->last4;
        $issuingCard->livemode = ($card->livemode == true) ? '1' : '0';
        $issuingCard->replaced_by = $card->replaced_by;
        $issuingCard->replacement_for = $card->replacement_for;
        $issuingCard->replacement_reason = $card->replacement_reason;
        $issuingCard->type = $card->type;
        $issuingCard->status = $card->status;
        if ($card->type == 'physical') {
            $issuingCard->service = $card->shipping->service;
            $issuingCard->shipping_type = $card->shipping->type;
            $issuingCard->city = $card->shipping->address->city;
            $issuingCard->country = $card->shipping->address->country;
            $issuingCard->line1 = $card->shipping->address->line1;
            $issuingCard->postal_code = $card->shipping->address->postal_code;
            $issuingCard->line2 = $card->shipping->address->line2;
            $issuingCard->state = $card->shipping->address->state;
        }
        $issuingCard->save();

        return $issuingCard;
    }

    /**
     * Show issuingCard.
     * GET /api/issuing-cards/{id}
     */
    public function show(Request $req, $id)
    {
        $issuingCard = IssuingCard::find($id);
        if (empty($issuingCard)) Utils::error('IssuingCard not found!', 404);
        return $issuingCard;
    }

    /**
     * Update issuingCard.
     * POST /api/issuing-cards/{id}
     */
    public function update(Request $req, $id)
    {
        $issuingCard = IssuingCard::find($id);
        if (empty($issuingCard)) Utils::error('IssuingCard not found!', 404);

        $this->validate($req, [
            'business_id' => 'required|exists:businesses,id',
            'cardholder_id' => 'string|exists:card_holders,id',
            'card_id' => 'string|max:255',
            'brand' => 'string|max:255',
            'cancellation_reason' => 'string|max:255',
            'currency' => 'string|max:255',
            'exp_month' => 'string|max:255',
            'exp_year' => 'string|max:255',
            'last4' => 'string|max:255',
            'livemode' => 'boolean',
            'replaced_by' => 'string|max:255',
            'replacement_for' => 'string|max:255',
            'replacement_reason' => 'string|max:255',
            'type' => 'string|in:physical,virtual',
            'status' => 'required_if:type,physical|string|in:active,inactive',
            'service' => 'required_if:type,physical|string|in:standard,express,priority',
            'shipping_type' => 'required_if:type,physical|string|in:bulk,individual',
            'city' => 'required_if:type,physical|string|max:255',
            'country' => 'required_if:type,physical|string|max:255',
            'line1' => 'required_if:type,physical|string|max:255',
            'postal_code' => 'required_if:type,physical|string|max:255',
            'line2' => 'string|max:255',
            'state' => 'string|max:255',
        ]);

        // update issuingCard
        if ($req->has('business_id')) $issuingCard->business_id = $req->input('business_id');
        if ($req->has('cardholder_id')) $issuingCard->cardholder_id = $req->input('cardholder_id');
        if ($req->has('card_id')) $issuingCard->card_id = $req->input('card_id');
        if ($req->has('brand')) $issuingCard->brand = $req->input('brand');
        if ($req->has('cancellation_reason')) $issuingCard->cancellation_reason = $req->input('cancellation_reason');
        if ($req->has('currency')) $issuingCard->currency = $req->input('currency');
        if ($req->has('exp_month')) $issuingCard->exp_month = $req->input('exp_month');
        if ($req->has('exp_year')) $issuingCard->exp_year = $req->input('exp_year');
        if ($req->has('last4')) $issuingCard->last4 = $req->input('last4');
        $issuingCard->livemode = $req->input('livemode');
        if ($req->has('replaced_by')) $issuingCard->replaced_by = $req->input('replaced_by');
        if ($req->has('replacement_for')) $issuingCard->replacement_for = $req->input('replacement_for');
        if ($req->has('replacement_reason')) $issuingCard->replacement_reason = $req->input('replacement_reason');
        if ($req->has('type')) $issuingCard->type = $req->input('type');
        if ($req->has('status')) $issuingCard->status = $req->input('status');
        if ($req->has('service')) $issuingCard->service = $req->input('service');
        if ($req->has('shipping_type')) $issuingCard->shipping_type = $req->input('shipping_type');
        if ($req->has('city')) $issuingCard->city = $req->input('city');
        if ($req->has('country')) $issuingCard->country = $req->input('country');
        if ($req->has('line1')) $issuingCard->line1 = $req->input('line1');
        if ($req->has('postal_code')) $issuingCard->postal_code = $req->input('postal_code');
        if ($req->has('line2')) $issuingCard->line2 = $req->input('line2');
        if ($req->has('state')) $issuingCard->state = $req->input('state');

        $issuingCard->save();

        return $issuingCard;
    }

    /**
     * Update issuingCard.
     * POST /api/issuing-cards/{id}
     */
    public function change_pin(Request $req, $id)
    {
        $issuingCard = IssuingCard::find($id);
        if (empty($issuingCard)) Utils::error('IssuingCard not found!', 404);

        $this->validate($req, [
            'pin_encryption' => 'required|string',
        ]);

        try {

            $business = Business::where('id', $issuingCard->business_id)->first();
            if ($business->connect_service == 1 && $business->s_account_id != '') {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            } else {
                $mode = SiteSettingController::findByKey('stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            }
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);

            $pin = $stripe->issuing->cards->update(
                $issuingCard->card_id,
                [
                    'pin' => [
                        'encrypted_number' => $req->input('pin_encryption')
                    ]
                ]
            );
        } catch (Exception $e) {
            Utils::error($e->getMessage(), 404);
        }

        return $issuingCard;
    }


    /**
     * Reveal issuingCard.
     * POST /api/issuing-cards/reveal/{id}
     */
    public function reveal(Request $req, $card_id)
    {
        $issuingCard = IssuingCard::where('card_id', $card_id)->where('type', 'virtual')->first();
        if (empty($issuingCard)) Utils::error('IssuingCard not virtual please try virtual card!', 404);

        try {
            $business = Business::where('id', $req->input('business_id'))->first();
            if ($business->connect_service == 1 && $business->s_account_id != '') {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            } else {
                $mode = SiteSettingController::findByKey('stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            }
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);


            $issuingCard = $stripe->issuing->cards->retrieve(
                $card_id,
                ['expand' => ['number', 'cvc']]
            );
        } catch (Exception $e) {
            Utils::error($e, 404);
        }

        return $issuingCard;
    }

    /**
     * Enable issuingCard.
     * POST /api/issuing-cards/enable/{id}
     */
    public function enable(Request $req, $id)
    {
        $issuingCard = IssuingCard::find($id);
        if (empty($issuingCard)) Utils::error('IssuingCard not found!', 404);

        try {
            $business = Business::where('id', $issuingCard->business_id)->first();
            if ($business->connect_service == 1 && $business->s_account_id != '') {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            } else {
                $mode = SiteSettingController::findByKey('stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            }
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);


            $stripe->issuing->cards->update(
                $issuingCard->card_id,
                ['status' => 'active']
            );
        } catch (Exception $e) {
            Utils::error($e, 404);
        }
        $issuingCard->status = 'active';
        $issuingCard->save();

        return $issuingCard;
    }

    /**
     * Enable issuingCard.
     * POST /api/issuing-cards/disable/{id}
     */
    public function disable(Request $req, $id)
    {
        $issuingCard = IssuingCard::find($id);
        if (empty($issuingCard)) Utils::error('IssuingCard not found!', 404);

        try {
            $business = Business::where('id', $issuingCard->business_id)->first();
            if ($business->connect_service == 1 && $business->s_account_id != '') {
                $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'connect_stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            } else {
                $mode = SiteSettingController::findByKey('stripe_mode')->value;
                if ($mode == 'test') {
                    $stripeKeyColoumns = 'stripe_private_key_test';
                }
                if ($mode == 'live') {
                    $stripeKeyColoumns = 'stripe_private_key_live';
                }
                $key =  SiteSettingController::findByKey($stripeKeyColoumns)->value;
            }
            $stripe = new StripeClient(["api_key" => $key, "stripe_version" => "2020-08-27"]);


            $stripe->issuing->cards->update(
                $issuingCard->card_id,
                ['status' => 'inactive']
            );
        } catch (Exception $e) {
            Utils::error($e, 404);
        }
        $issuingCard->status = 'inactive';
        $issuingCard->save();

        return $issuingCard;
    }

    /**
     * Delete issuingCard by id.
     * DELETE /api/issuing-cards/{id}
     */
    public function delete(Request $req, $id)
    {
        $issuingCard = IssuingCard::find($id);
        if (empty($issuingCard)) Utils::error('IssuingCard not found!', 404);
        $issuingCard->delete();
        return $issuingCard;
    }
}
