<?php

namespace App\Http\Controllers;

use App\Business;
use App\Helpers\ImageHelper;
use App\Passbook;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\User;
use Carbon\Carbon;
use Faker\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Mpdf\Mpdf;

class PassbookController extends Controller
{
    /**
     * List passbooks.
     * GET /api/passbook
     */
    public function index(Request $req)
    {
        $passbooks = Passbook::select("passbook.*", "businesses.name as business_name", "users.first_name", "users.last_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->leftJoin('users', 'users.id', '=', 'user_id')->orderByRaw('date DESC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $passbooks->whereRaw('(businesses.name LIKE ? OR first_name LIKE ? OR last_name LIKE ? OR amount LIKE ? OR description LIKE ?)', ["%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }

        if ($query = $req->query('passbook_status', false)) {
            if ($query != 'both') {
                $passbooks->whereRaw('(description LIKE ?)', ["%$query%"]);
            }
        }

        if ($query = $req->query('business_id', false)) {
            $passbooks->where('business_id', $query);
        }
        if ($query = $req->query('user_id', false)) {
            $passbooks->where('user_id', $query);
        }
        if ($query = $req->query('from_date', false)) {
            $passbooks->where('from_date', '>=', $query);
        }
        if ($query = $req->query('to_date', false)) {
            $passbooks->where('to_date', '<=', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $passbooks = $passbooks->get();
        } else {
            $passbooks = $passbooks->paginate($req->query('per_page', 20));
        }

        if ($passbooks->isEmpty()) Utils::error('No passbooks found!', 404);


        return ($passbooks);
    }

    /**
     * Create passbook.
     * POST /api/passbook
     */
    public function create(Request $req)
    {

        $this->validate($req, [
            'amount' => 'required|numeric:gt,0',
            'description' => 'string',
            'date' => 'date',
            'from_date' => 'date',
            'to_date' => 'date',
            'entity' => 'in:payout_request,payout_invoice,reseller_payout_request,reseller_payout_invoice',
            'business_id' => 'required_without:user_id|exists:businesses,id',
            'user_id' => 'required_without:business_id|exists:users,id',
        ]);

        // create passbook
        $passbook = new Passbook;
        $passbook->fill($req->all());
        $passbook->from_date = Carbon::today();
        $passbook->to_date = Carbon::today();
        $passbook->save();

        return $passbook;
    }

    /**
     * Show passbook.
     * GET /api/passbook/{id}
     */
    public function show(Request $req, $id)
    {
        $passbook = Passbook::find($id);
        if (empty($passbook)) Utils::error('Passbook not found!', 404);
        return $passbook;
    }

    /**
     * Update passbook.
     * POST /api/passbook/{id}
     */
    public function update(Request $req, $id)
    {
        $passbook = Passbook::find($id);
        if (empty($passbook)) Utils::error('Passbook not found!', 404);

        $this->validate($req, [
            'amount' => 'required|numeric',
            'entity' => 'in:payout_request,payout_invoice,reseller_payout_request,reseller_payout_invoice',
            'description' => 'string',
            'date' => 'date',
            'from_date' => 'date',
            'to_date' => 'date',
            'business_id' => 'required_without:user_id|exists:businesses,id',
            'user_id' => 'required_without:business_id|exists:users,id',
        ]);

        // update passbook
        if ($req->has('amount')) $passbook->amount = $req->input('amount');
        if ($req->has('description')) $passbook->description = $req->input('description');
        if ($req->has('entity')) $passbook->entity = $req->input('entity');
        if ($req->has('date')) $passbook->date = $req->input('date');
        if ($req->has('from_date')) $passbook->from_date = $req->input('from_date');
        if ($req->has('to_date')) $passbook->to_date = $req->input('to_date');
        if ($req->has('entity_id')) $passbook->entity_id = $req->input('entity_id');
        if ($req->has('business_id')) $passbook->business_id = $req->input('business_id');
        if ($req->has('user_id')) $passbook->user_id = $req->input('user_id');

        $passbook->save();

        return $passbook;
    }


    /**
     * Delete passbook by id.
     * DELETE /api/passbook/{id}
     */
    public function delete(Request $req, $id)
    {
        $passbook = Passbook::find($id);
        if (empty($passbook)) Utils::error('Passbook not found!', 404);
        $passbook->delete();
        return $passbook;
    }

    public function statement(Request $req)
    {
        $this->validate($req, [
            'from_date' => 'date',
            'to_date' => 'after:from_date|date',
        ]);
        if (!$req->has('from_date')) {
            $req->merge(['from_date' => Carbon::today()->subDays(15)->toDateString()]);
        }
        if (!$req->has('to_date')) {
            $req->merge(['to_date' => Carbon::today()]);
        }
        $passbooks = Passbook::select("passbook.*", "businesses.name as business_name", "users.first_name", "users.last_name")->leftJoin('businesses', 'businesses.id', '=', 'business_id')->leftJoin('users', 'users.id', '=', 'user_id')->orderByRaw('date DESC');

        $business = null;
        $user = null;
        if ($query = $req->query('business_id', false)) {
            $business = Business::find($query);
            $passbooks->where('business_id', $query);
        }
        if ($query = $req->query('user_id', false)) {
            $user = User::find($query);
            $passbooks->where('user_id', $query);
        }
        if ($query = $req->query('from_date', false)) {
            $passbooks->where('from_date', '>=', $query);
        }
        if ($query = $req->query('to_date', false)) {
            $passbooks->where('to_date', '<=', $query);
        }
        if ($query = $req->query('passbook_status', false)) {
            if ($query != 'both') {
                $passbooks->whereRaw('(description LIKE ?)', ["%$query%"]);
            }
        }
        $passbooks = $passbooks->get();
        if ($passbooks->isEmpty()) Utils::error('No passbooks found!', 404);
        if ($query = $req->query('export', false)) {
            if ($query == 'pdf') {
                return $this->printStatement($req, $business, $user, $passbooks);
            }
        }
        return $passbooks;
    }
    public function printStatement($req, $business, $user, $passbooks)
    {
        $from_date = Carbon::parse($req->input('from_date'))->format('d-M-Y');
        $to_date = Carbon::parse($req->input('to_date'))->format('d-M-Y');

        $html = '<html>';
        $html = $html . '<head>';
        $html = $html . '<style>
        * {
            margin: 0;
            padding: 0;
            font-family: "Verdana";
            font-size: 9pt;
            color: #000;
        }

        body {
            width: 100%;
            font-family: "Verdana";
            font-size: 9pt;
            margin: 0;
            padding: 0;
        }
        table {
            /*border-left: 1px solid #000000;
                border-top: 1px solid #000000;*/
            border-spacing: 0;
            border-collapse: collapse;
            font-size:15px;

        }
        .info-table  tr td{
            padding:2px;
        }
        .info-table tr td:nth-child(even){
            text-align:right;
        }
        .items-table{
            margin-top:20px;
            width:100%;
            text-align:center;
        }
        .items-table tr th{
            padding: 8px;
            font-size: 16px; 
            border: 1px solid #73C6B6;
            font-weight:600;
            background-color:#138D75;
            color:#fff;
        }
        .items-table tr td{
            font-size: 15px; 
            padding: 8px;
            border: 1px solid #73C6B6;
        }
        .items-table tr:nth-child(even){
            background-color:#E8F6F3   
        }
        .items-table tr:nth-child(odd){
            background-color:#ffffff   
        }';

        $html = $html . '</style>';
        $html = $html . '</head>';
        $html = $html . '<body>';
        $html = $html . '<h2 style = "text-align:right">Statement from ' . $from_date . ' to ' . $to_date . '</h2>';
        $html = $html . '<table style="width:100%" class="info-table">';

        $html = $html . '<tr>';
        $html = $html . '<td style="font-weight: 900;">UBSIDI Limited</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td></td>';
        if (!empty($business)) {
            $html = $html . '<td>' . $business->name . '</td>';
        } else if (!empty($user)) {
            $html = $html . '<td>' . $user->first_name . '</td>';
        } else {
            $html = $html . '<td></td>';
        }

        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>66 Little Caldmore</td>';
        $html = $html . '<td>' . Carbon::today()->format('F d, Y') . '</td>';
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Walsall</td>';
        if (!empty($business)) {
            $html = $html . '<td>' . $business->address . '</td>';
        } else if (!empty($user)) {
            $html = $html . '<td>' . $user->residential_address . '</td>';
        } else {
            $html = $html . '<td></td>';
        }
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>WS1 3RA</td>';
        if (!empty($business)) {
            $html = $html . '<td>' . $business->country->name  . '</td>';
        } else if (!empty($user)) {
            $html = $html . '<td>' . $user->country->name  . '</td>';
        } else {
            $html = $html . '<td></td>';
        }
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Email: admin@smartmps.co.uk</td>';
        if (!empty($business)) {
            $html = $html . '<td>' . $business->email  . '</td>';
        } else if (!empty($user)) {
            $html = $html . '<td>' . $user->login_email  . '</td>';
        } else {
            $html = $html . '<td></td>';
        }
        $html = $html . '</tr>';

        $html = $html . '<tr>';
        $html = $html . '<td>Tel: 08712 884 044</td>';
        $html = $html . '<td></td>';
        $html = $html . '</tr>';
        $html = $html . '</table>';

        $html = $html . '<table class="items-table">';

        $html = $html . '<tr>';
        $html = $html . '<th>Sr No.</th>';
        $html = $html . '<th>Date</th>';
        $html = $html . '<th>Description</th>';
        $html = $html . '<th>Status</th>';
        $html = $html . '<th>Purpose</th>';
        $html = $html . '<th>Gross Amount</th>';
        $html = $html . '<th>Gross Fee</th>';
        $html = $html . '<th>Total Amount</th>';
        $html = $html . '</tr>';
        $srNo = 1;
        $grand_total = 0;
        $archive_total = 0;
        $normal_total = 0;
        foreach ($passbooks as $item) {
            if ($req->input('archive_status') == 'archived' && $item->payout_invoice->transaction_type == 'Archived') {
                $grand_total = $grand_total + $item->amount;

                if ($item->payout_invoice->transaction_type == 'Archived') {
                    $archive_total = $archive_total + $item->amount;
                }

                $html = $html . '<tr>';
                $html = $html . '<td>' . $srNo . '</td>';
                $html = $html . '<td>' . Carbon::parse($item->date)->format('d-m-Y') . '</td>';
                $html = $html . '<td style="text-align:left;">' . $item->description . '</td>';
                $html = $html . '<td>' . $item->payout_invoice->transaction_type . '</td>';
                $html = $html . '<td>' . $item->entity . '</td>';
                if (!empty($business)) {
                    $html = $html . '<td>' . $business->country->currency_symbol . '' . $item->payout_invoice->gross_total . '</td>';
                } else if (!empty($user) && !empty($user->country)) {
                    $html = $html . '<td>' . $user->country->currency_symbol . '' . $item->payout_invoice->gross_total . '</td>';
                } else {
                    $html = $html . '<td>' . $item->payout_invoice->gross_total . '</td>';
                }

                if (!empty($business)) {
                    $html = $html . '<td>' . $business->country->currency_symbol . '' . $item->payout_invoice->gross_fee . '</td>';
                } else if (!empty($user) && !empty($user->country)) {
                    $html = $html . '<td>' . $user->country->currency_symbol . '' . $item->payout_invoice->gross_fee . '</td>';
                } else {
                    $html = $html . '<td>' . $item->payout_invoice->gross_fee . '</td>';
                }

                if (!empty($business)) {
                    $html = $html . '<td>' . $business->country->currency_symbol . '' . $item->amount . '</td>';
                } else if (!empty($user) && !empty($user->country)) {
                    $html = $html . '<td>' . $user->country->currency_symbol . '' . $item->amount . '</td>';
                } else {
                    $html = $html . '<td>' . $item->amount . '</td>';
                }

                $html = $html . '</tr>';
                $srNo++;
            } else if ($req->input('archive_status') == 'normal' && $item->payout_invoice->transaction_type == 'Normal') {
                $grand_total = $grand_total + $item->amount;

                if ($item->payout_invoice->transaction_type == 'Normal') {
                    $normal_total = $normal_total + $item->amount;
                }

                $html = $html . '<tr>';
                $html = $html . '<td>' . $srNo . '</td>';
                $html = $html . '<td>' . Carbon::parse($item->date)->format('d-m-Y') . '</td>';
                $html = $html . '<td style="text-align:left;">' . $item->description . '</td>';
                $html = $html . '<td>' . $item->payout_invoice->transaction_type . '</td>';
                $html = $html . '<td>' . $item->entity . '</td>';
                if (!empty($business)) {
                    $html = $html . '<td>' . $business->country->currency_symbol . '' . $item->payout_invoice->gross_total . '</td>';
                } else if (!empty($user) && !empty($user->country)) {
                    $html = $html . '<td>' . $user->country->currency_symbol . '' . $item->payout_invoice->gross_total . '</td>';
                } else {
                    $html = $html . '<td>' . $item->payout_invoice->gross_total . '</td>';
                }

                if (!empty($business)) {
                    $html = $html . '<td>' . $business->country->currency_symbol . '' . $item->payout_invoice->gross_fee . '</td>';
                } else if (!empty($user) && !empty($user->country)) {
                    $html = $html . '<td>' . $user->country->currency_symbol . '' . $item->payout_invoice->gross_fee . '</td>';
                } else {
                    $html = $html . '<td>' . $item->payout_invoice->gross_fee . '</td>';
                }
                if (!empty($business)) {
                    $html = $html . '<td>' . $business->country->currency_symbol . '' . $item->amount . '</td>';
                } else if (!empty($user) && !empty($user->country)) {
                    $html = $html . '<td>' . $user->country->currency_symbol . '' . $item->amount . '</td>';
                } else {
                    $html = $html . '<td>' . $item->amount . '</td>';
                }
                $html = $html . '</tr>';
                $srNo++;
            } else if ($req->input('archive_status') == 'both') {
                $grand_total = $grand_total + $item->amount;

                if ($item->payout_invoice->transaction_type == 'Archived') {
                    $archive_total = $archive_total + $item->amount;
                }
                if ($item->payout_invoice->transaction_type == 'Normal') {
                    $normal_total = $normal_total + $item->amount;
                }

                $html = $html . '<tr>';
                $html = $html . '<td>' . $srNo . '</td>';
                $html = $html . '<td>' . Carbon::parse($item->date)->format('d-m-Y') . '</td>';
                $html = $html . '<td style="text-align:left;">' . $item->description . '</td>';
                $html = $html . '<td>' . $item->payout_invoice->transaction_type . '</td>';
                $html = $html . '<td>' . $item->entity . '</td>';
                if (!empty($business)) {
                    $html = $html . '<td>' . $business->country->currency_symbol . '' . $item->payout_invoice->gross_total . '</td>';
                } else if (!empty($user) && !empty($user->country)) {
                    $html = $html . '<td>' . $user->country->currency_symbol . '' . $item->payout_invoice->gross_total . '</td>';
                } else {
                    $html = $html . '<td>' . $item->payout_invoice->gross_total . '</td>';
                }

                if (!empty($business)) {
                    $html = $html . '<td>' . $business->country->currency_symbol . '' . $item->payout_invoice->gross_fee . '</td>';
                } else if (!empty($user) && !empty($user->country)) {
                    $html = $html . '<td>' . $user->country->currency_symbol . '' . $item->payout_invoice->gross_fee . '</td>';
                } else {
                    $html = $html . '<td>' . $item->payout_invoice->gross_fee . '</td>';
                }
                if (!empty($business)) {
                    $html = $html . '<td>' . $business->country->currency_symbol . '' . $item->amount . '</td>';
                } else if (!empty($user) && !empty($user->country)) {
                    $html = $html . '<td>' . $user->country->currency_symbol . '' . $item->amount . '</td>';
                } else {
                    $html = $html . '<td>' . $item->amount . '</td>';
                }
                $html = $html . '</tr>';
                $srNo++;
            } else {
            }
        }

        if ($req->input('archive_status') == 'normal') {
            $html = $html . '<tr>';
            $html = $html . '<th colspan="7" style="text-align:right;">Normal Total:</th>';
            if (!empty($business)) {
                $html = $html . '<th><b>' . $business->country->currency_symbol . '' . $normal_total . '</b></th>';
            } else if (!empty($user) && !empty($user->country)) {
                $html = $html . '<th><b>' . $user->country->currency_symbol . '' . $normal_total . '</b></th>';
            } else {
                $html = $html . '<th><b>' . $normal_total . '</b></th>';
            }
            $html = $html . '</tr>';
        }

        if ($req->input('archive_status') == 'archived') {
            $html = $html . '<tr>';
            $html = $html . '<th colspan="7" style="text-align:right;">Archive Total:</th>';
            if (!empty($business)) {
                $html = $html . '<th><b>' . $business->country->currency_symbol . '' . $archive_total . '</b></th>';
            } else if (!empty($user) && !empty($user->country)) {
                $html = $html . '<th><b>' . $user->country->currency_symbol . '' . $archive_total . '</b></th>';
            } else {
                $html = $html . '<th><b>' . $archive_total . '</b></th>';
            }
            $html = $html . '</tr>';
        }

        $html = $html . '<tr>';
        $html = $html . '<th colspan="7" style="text-align:right;">Grand Total:</th>';
        if (!empty($business)) {
            $html = $html . '<th><b>' . $business->country->currency_symbol . '' . $grand_total . '</b></th>';
        } else if (!empty($user) && !empty($user->country)) {
            $html = $html . '<th><b>' . $user->country->currency_symbol . '' . $grand_total . '</b></th>';
        } else {
            $html = $html . '<th><b>' . $grand_total . '</b></th>';
        }
        $html = $html . '</tr>';


        $html = $html . '</table>';
        $html = $html . '</body>';
        $html = $html . '</html>';

        $mpdf = new Mpdf();
        $mpdf->WriteHTML($html);
        $fileName = "Statement";
        if (!empty($business)) {
            $fileName = $business->name . '-Statement-' . $from_date . ' to ' . $to_date;
        } else if (!empty($user)) {
            $fileName = $user->first_name . '-Statement-' . $from_date . ' to ' . $to_date;
        }
        return $mpdf->Output("$fileName.pdf", "D");
    }
}
