<?php

namespace App\Http\Controllers;

use App\Plan;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class PlanController extends Controller
{
    /**
     * List plans.
     * GET /api/plans
     */
    public function index(Request $req)
    {
        $plans = Plan::orderByRaw('name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $plans->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('service', false)) {
            $plans->where('service', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $plans = $plans->get();
        } else {
            $plans = $plans->paginate($req->query('per_page', 20));
        }

        if ($plans->isEmpty()) Utils::error('No plans found!', 404);


        return ($plans);
    }

    /**
     * Create plan.
     * POST /api/plans
     */
    public function create(Request $req)
    {
        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }

        $this->validate($req, [
            'name' => 'required|string|max:255|unique:plans,name',
            'service' => 'required|in:Epos,Online,Merchant,Card,Connect',
            'permissions' => 'required',
            'description' => 'string',
        ]);

        // create plan
        $plan = new Plan;
        $plan->fill($req->all());
        if ($req->has('permissions')) $plan->permissions = $req->input('permissions');
        $plan->save();

        return $plan;
    }

    /**
     * Show plan.
     * GET /api/plans/{id}
     */
    public function show(Request $req, $id)
    {
        $plan = Plan::find($id);
        if (empty($plan)) Utils::error('Plan not found!', 404);
        return $plan;
    }

    /**
     * Update plan.
     * POST /api/plans/{id}
     */
    public function update(Request $req, $id)
    {
        $plan = Plan::find($id);
        if (empty($plan)) Utils::error('Plan not found!', 404);

        if (!is_array($req->permissions)) {
            $req->merge(['permissions' => (json_decode(stripslashes($req->permissions), true))]);
        }

        $this->validate($req, [
            'name' => ['string', 'max:255', Rule::unique('plans')->ignore($plan->id)],
            'service' => 'required|in:Epos,Online,Merchant,Card,Connect',
            'permissions' => 'required',
            'description' => 'string',
        ]);

        // update plan
        if ($req->has('name')) $plan->name = $req->input('name');
        if ($req->has('service')) $plan->service = $req->input('service');
        if ($req->has('permissions')) $plan->permissions = $req->input('permissions');
        if ($req->has('description')) $plan->description = $req->input('description');

        $plan->save();

        return $plan;
    }


    /**
     * Delete plan by id.
     * DELETE /api/plans/{id}
     */
    public function delete(Request $req, $id)
    {
        $plan = Plan::find($id);
        if (empty($plan)) Utils::error('Plan not found!', 404);
        $plan->delete();
        return $plan;
    }
}
