<?php

namespace App\Http\Controllers;

use App\PlanProduct;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class PlanProductController extends Controller
{
    /**
     * List plan-products.
     * GET /api/plan-products
     */
    public function index(Request $req)
    {
        $plan_product_products = PlanProduct::orderByRaw('name ASC, created_at DESC');

        if ($query = $req->query('query', false)) {
            $plan_product_products->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('service_type', false)) {
            $plan_product_products->where('service_type', $query);
        }
        if ($query = $req->query('recurring_type', false)) {
            $plan_product_products->where('recurring_type', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $plan_product_products = $plan_product_products->get();
        } else {
            $plan_product_products = $plan_product_products->paginate($req->query('per_page', 20));
        }

        if ($plan_product_products->isEmpty()) Utils::error('No plans found!', 404);


        return ($plan_product_products);
    }

    /**
     * Create plan-product.
     * POST /api/plan-products
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255|unique:plan_products,name',
            'service_type' => 'required|in:Epos,Online,Merchant,Card,Connect',
            'price'  => 'required_if:price_available,1|numeric',
            'recurring_type' => 'in:Daily,Weekly,Monthly,Quarterly,Half Yearly,Yearly',
            'recurring_price'  => 'required_if:recurring_available,1|numeric',
            'quantity'  => 'numeric',
            'discount '  => 'numeric',
        ], [
            'price.required_if' => "The price field is required.",
            'recurring_price.required_if' => "The recurring price field is required.",
        ]);

        // create plan-product
        $plan_product = new PlanProduct;
        $plan_product->fill($req->all());
        $plan_product->save();

        return $plan_product;
    }

    /**
     * Show plan-product.
     * GET /api/plan-products/{id}
     */
    public function show(Request $req, $id)
    {
        $plan_product = PlanProduct::find($id);
        if (empty($plan_product)) Utils::error('PlanProduct not found!', 404);
        return $plan_product;
    }

    /**
     * Update plan-product.
     * POST /api/plan-products/{id}
     */
    public function update(Request $req, $id)
    {
        $plan_product = PlanProduct::find($id);
        if (empty($plan_product)) Utils::error('PlanProduct not found!', 404);

        $this->validate($req, [
            'name' => ['string', 'max:255', Rule::unique('plan_products')->ignore($plan_product->id)],
            'service_type' => 'required|in:Epos,Online,Merchant,Card,Connect',
            'price'  => 'required_if:price_available,1|numeric',
            'recurring_type' => 'in:Daily,Weekly,Monthly,Quarterly,Half Yearly,Yearly',
            'recurring_price'  => 'required_if:recurring_available,1|numeric',
            'quantity'  => 'numeric',
            'discount '  => 'numeric',
        ], [
            'price.required_if' => "The price field is required.",
            'recurring_price.required_if' => "The recurring price field is required.",
        ]);

        // update plan
        if ($req->has('name')) $plan_product->name = $req->input('name');
        if ($req->has('service_type')) $plan_product->service_type = $req->input('service_type');
        $plan_product->price_available = $req->input('price_available');
        if ($req->has('price')) $plan_product->price = $req->input('price');
        if ($req->has('recurring_type')) $plan_product->recurring_type = $req->input('recurring_type');
        $plan_product->recurring_available = $req->input('recurring_available');
        if ($req->has('recurring_price')) $plan_product->recurring_price = $req->input('recurring_price');
        if ($req->has('quantity')) $plan_product->quantity = $req->input('quantity');
        if ($req->has('discount')) $plan_product->discount = $req->input('discount');
        $plan_product->save();

        return $plan_product;
    }


    /**
     * Delete plan-product by id.
     * DELETE /api/plan-products/{id}
     */
    public function delete(Request $req, $id)
    {
        $plan_product = PlanProduct::find($id);
        if (empty($plan_product)) Utils::error('PlanProduct not found!', 404);
        $plan_product->delete();
        return $plan_product;
    }

    /**
     * Enable plan-product
     * POST /api/plan-products/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $plan_product = PlanProduct::find($id);
        if (empty($plan_product)) Utils::error('Product not found!', 404);
        $plan_product->disabled = 0;
        $plan_product->save();
        return $plan_product;
    }

    /**
     * Disable plan-product.
     * POST /api/plan-products/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $plan_product = PlanProduct::find($id);
        if (empty($plan_product)) Utils::error('Product not found!', 404);
        $plan_product->disabled = 1;
        $plan_product->save();
        return $plan_product;
    }
}
