<?php

namespace App\Http\Controllers;

use App\Helpers\Utils;
use App\Jobs\EmailSender;
use App\Postcode;
use App\SiteSetting;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Queue;

class PostcodeController extends Controller
{
    public function index(Request $req)
    {
        $postcodes = Postcode::orderByRaw('id DESC');

        if ($query = $req->query('status')) {
            $postcodes->where('status', $query);
        }

        if ($query = $req->query('query', false)) {
            $postcodes->whereRaw('post_code LIKE ? OR post_code LIKE ? OR post_town LIKE ? OR street LIKE ? OR latitude LIKE ? OR longitude LIKE ? OR dependent_locality LIKE ? OR double_dependent_locality LIKE ?', ["%$query%", str_replace(" ", "", $query), "%$query%", "%$query%", "%$query%", "%$query%", "%$query%", "%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $postcodes = $postcodes->get();
        } else {
            $postcodes = $postcodes->paginate($req->query('per_page', 20));
        }

        if ($postcodes->isEmpty()) Utils::error('No postcode found!', 404);


        return ($postcodes);
    }
    /**
     * Create postcode.
     * POST /postcodes
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'thorough_fare' => 'string',
            'street' => 'string',
            'double_dependent_locality' => 'string',
            'dependent_locality' => 'string',
            'post_town' => 'string',
            'post_code' => 'required|string|unique:postcode.postcode,post_code',
            'post_code_type' => 'string',
            'latitude' => 'numeric',
            'longitude' => 'numeric',
            'status' => 'in:Pending,Accepted',
        ]);

        // create postcode
        $postcode = new Postcode;
        $postcode->fill($req->all());
        $postcode->post_code = str_replace(" ", "", $req->input('post_code'));
        $postcode->save();

        return $postcode;
    }

    /**
     * Update postcode.
     * POST /postcodes
     */
    public function update(Request $req, $id)
    {
        $postcode = Postcode::find($id);
        if (empty($postcode)) Utils::error('Postcode not found!', 404);

        $this->validate($req, [
            'thorough_fare' => 'string',
            'street' => 'string',
            'double_dependent_locality' => 'string',
            'dependent_locality' => 'string',
            'post_town' => 'string',
            'post_code' => 'required|string',
            'post_code_type' => 'string',
            'latitude' => 'numeric',
            'longitude' => 'numeric',
            'status' => 'in:Pending,Accepted',

        ]);

        // update postcode
        $postcode->thorough_fare = $req->input('thorough_fare');
        $postcode->street = $req->input('street');
        $postcode->double_dependent_locality = $req->input('double_dependent_locality');
        $postcode->dependent_locality = $req->input('dependent_locality');
        $postcode->post_town = $req->input('post_town');
        $postcode->post_code = str_replace(" ", "", $req->input('post_code'));
        $postcode->post_code_type = $req->input('post_code_type');
        $postcode->latitude = $req->input('latitude');
        $postcode->longitude = $req->input('longitude');
        $postcode->status = $req->input('status');
        $postcode->save();

        return $postcode;
    }

    /**
     * 
     * Search Post code.
     * GET postcodes/find/{code}
     * 
     */
    public function find(Request $req, $code)
    {
        $postcode = Postcode::whereRaw('(post_code LIKE ? OR post_code LIKE ? )', [$code, str_replace("%20", "", $code)])->first();
        if (empty($postcode)) {
            $newPostcode = $this->findFromGoogleAndReturn($code);
            if (empty($newPostcode)) {
                $this->notifyAdmin($code, $newPostcode);
                Utils::error("Invalid postcode data", 400);
            }
            if ($newPostcode->post_code == null && $newPostcode->latitude == 0 && $newPostcode->longitude == 0) {
                Utils::error("Invalid postcode", 400);
            }
            $newPostcode->status = "Pending";
            $newPostcode->save();
            $newPostcode->refresh();
            $this->notifyAdmin($code, $newPostcode);
            return $newPostcode;
        } else {
            if ($postcode->latitude == 0 && $postcode->longitude == 0) {
                Utils::error("Invalid postcode", 400);
            }
        }
        return $postcode;
    }

    /**
     * Accept status postcode by id.
     * POST postcodes/{id}/accept
     */
    public function accept(Request $req, $id)
    {
        $postcode = Postcode::find($id);
        if (empty($postcode)) Utils::error('Postcode not found!', 404);
        $postcode->status = 'Accepted';
        $postcode->save();
        return $postcode;
    }

    /**
     * Accept bulk status postcode by id.
     * POST postcodes/bulk-accept
     */
    public function bulk_accept(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids'))) {
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        }
        $this->validate($req, [
            'ids' => 'required|array'
        ]);
        $postcodes = Postcode::whereIn('id', $req->input('ids'))->update(['status' => 'Accepted']);
        return $postcodes;
    }

    /**
     * Delete bulk status postcode by id.
     * POST postcodes/bulk-delete
     */
    public function bulk_delete(Request $req)
    {
        if ($req->has('ids') && !is_array($req->input('ids'))) {
            $req->merge(['ids' => json_decode($req->input('ids'))]);
        }
        $this->validate($req, [
            'ids' => 'required|array'
        ]);
        $postcodes = Postcode::whereIn('id', $req->input('ids'))->delete();
        return $postcodes;
    }

    /**
     * Pending status postcode by id.
     * POST postcodes/{id}/pending
     */
    public function pending(Request $req, $id)
    {
        $postcode = Postcode::find($id);
        if (empty($postcode)) Utils::error('Postcode not found!', 404);
        $postcode->status = 'Pending';
        $postcode->save();
        return $postcode;
    }

    /**
     * Delete postcode by id.
     * DELETE postcodes/{id}
     */
    public function delete(Request $req, $id)
    {
        $postcode = Postcode::find($id);
        if (empty($postcode)) Utils::error('Postcode not found!', 404);
        $postcode->delete();
        return $postcode;
    }

    public function findFromGoogleAndReturn($code)
    {
        $siteSetting = SiteSettingController::findByKey('google_map_api_key');
        $key = "AIzaSyBX3HSrqBRxfsvfuLBV0ZdK_BfHSymdS-U";
        if (!empty($siteSetting)) {
            $key = $siteSetting->value;
        }
        $response = (new Client())->request('GET', "https://maps.google.com/maps/api/geocode/json", [
            'query' => ['address' => $code . '.UK', 'key' => $key, 'region' => 'UK']
        ]);
        if ($response->getStatusCode() == 200) {
            $jsonBody = json_decode($response->getBody());
            if ($jsonBody->status != 'ZERO_RESULTS') {

                $results = $jsonBody->results;
                if (!empty($results) && sizeof($results) > 0) {
                    $singleResult = $results[0];
                    $addressComponnet = $singleResult->address_components;

                    $newPostcode = new Postcode;
                    $newPostcode->latitude = $singleResult->geometry->location->lat;
                    $newPostcode->longitude = $singleResult->geometry->location->lng;

                    if (!empty($addressComponnet) && sizeof($addressComponnet) > 0) {
                        foreach ($addressComponnet as $key => $value) {
                            if ($value->types[0] == 'street_number') {
                                $newPostcode->street = $value->long_name;
                            }
                            if ($value->types[0] == 'premise') {
                                $newPostcode->thorough_fare = $value->long_name;
                            }
                            if ($value->types[0] == 'postal_town') {
                                $newPostcode->post_town = $value->long_name;
                            }
                            if ($value->types[0] == 'postal_code') {
                                $newPostcode->post_code = str_replace(" ", "", $value->long_name);
                            }
                        }
                    }
                    return $newPostcode;
                }
            }
            return null;
        }
    }
    public function notifyAdmin($code, $newPostcode)
    {
        $message = "";
        if (empty($newPoscode)) {
            $message = "New postcode searched by user and not found in google. Code: $code";
        } else {
            $message = "New postcode searched by user and saved to database. Code: $newPostcode->post_code";
        }
        $email = [
            'to' => '8dots.in@gmail.com',
            'to_name' => '8 Dots',
            'body' => $message,
            'subject' => 'New postcode',
        ];
        Queue::push(new EmailSender($email));
    }
}
