<?php

namespace App\Http\Controllers;

use App\Business;
use App\Helpers\ImageHelper;
use App\Printer;
use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Epos\EposPrinterModelController;
use App\Jobs\PrinterCreatorUpdaterJob;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Queue;

class PrinterController extends Controller
{
    /**
     * List printers.
     * GET /api/printers
     */
    public function index(Request $req)
    {
        $printers = Printer::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $printers->whereRaw('(name LIKE ?  )', ["%$query%"]);
        }
        if ($query = $req->query('ip', false)) {
            $printers->where('ip', $query);
        }
        if ($req->query('nopaginate', false) == '1') {
            $printers = $printers->get();
        } else {
            $printers = $printers->paginate($req->query('per_page', 20));
        }

        if ($printers->isEmpty()) Utils::error('No printers found!', 404);


        return ($printers);
    }

    /**
     * Create printer.
     * POST /api/printers
     */
    public function create(Request $req)
    {

        $this->validate($req, [
            'name' => 'required|string',
        ]);

        // create printer
        $printer = new Printer;
        $printer->fill($req->all());

        $printer->save();

        $businesses = Business::orderByRaw('name ASC');
        $businesses->where('disabled', 0);
        $businesses->where('epos_service', 1);
        $businesses = $businesses->get();
        if (!$businesses->isEmpty()) {
            foreach ($businesses as $business) {
                $business = Business::find($business->id);
                if (!empty($business)) {
                    Queue::later(Carbon::now(), new PrinterCreatorUpdaterJob($business, $printer, true));
                }
            }
        }

        return $printer;
    }

    /**
     * Show printer.
     * GET /api/printers/{id}
     */
    public function show(Request $req, $id)
    {
        $printer = Printer::find($id);
        if (empty($printer)) Utils::error('Printer not found!', 404);
        return $printer;
    }

    /**
     * Update printer.
     * POST /api/printers/{id}
     */
    public function update(Request $req, $id)
    {
        $printer = Printer::find($id);
        if (empty($printer)) Utils::error('Printer not found!', 404);

        $this->validate($req, [
            'name' => 'required|string',
        ]);

        // update printer
        if ($req->has('name')) $printer->name = $req->input('name');

        $printer->save();

        $businesses = Business::orderByRaw('name ASC');
        $businesses->where('disabled', 0);
        $businesses->where('epos_service', 1);
        $businesses = $businesses->get();
        if (!$businesses->isEmpty()) {
            foreach ($businesses as $business) {
                $business = Business::find($business->id);
                if (!empty($business)) {
                    Queue::later(Carbon::now(), new PrinterCreatorUpdaterJob($business, $printer, false));
                }
            }
        }

        return $printer;
    }

    /**
     * Enable printer.
     * POST /api/printers/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $printer = Printer::find($id);
        if (empty($printer)) Utils::error('Printer not found!', 404);
        $printer->disabled = 0;
        $printer->save();
        return $printer;
    }

    /**
     * Disable printer.
     * POST /api/printers/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $printer = Printer::find($id);
        if (empty($printer)) Utils::error('Printer not found!', 404);
        $printer->disabled = 1;
        $printer->save();
        return $printer;
    }
    /**
     * Delete printer by id.
     * DELETE /api/printers/{id}
     */
    public function delete(Request $req, $id)
    {
        $printer = Printer::find($id);
        if (empty($printer)) Utils::error('Printer not found!', 404);
        $printer->delete();
        return $printer;
    }


    public function load_printers(Request $req)
    {
        $printers[] = [
            "name" => "Yoke4 POS80",
            "disabled" => "0",
        ];
        $printers[] = [
            "name" => "ODP300 Dot",
            "disabled" => "0",
        ];
        $printers[] = [
            "name" => "POS-80 11.3.0.1",
            "disabled" => "0",
        ];
        $printers[] = [
            "name" => "AB-88 H",
            "disabled" => "0",
        ];
        $printers[] = [
            "name" => "KP-80",
            "disabled" => "0",
        ];

        foreach ($printers as $printer) {

            $printer_model = $this->findByName($printer['name']);
            if (empty($printer_model)) {
                $printer_model = new Printer($printer);
                $printer_model->save();
            } else {
                error_log("Printer found for " . $printer['name']);
            }
        }
        return Printer::get();
    }

    public static function findByName($name)
    {
        $printers = Printer::where('name', $name)->first();
        return $printers;
    }
}
