<?php

namespace App\Http\Controllers;

use App\ResellerPayoutRequest;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Helpers\Utils;
use App\Passbook;
use App\User;
use Exception;
use App\Exports\ResellerPayoutRequestExport;
use Illuminate\Support\Facades\Validator;
use Maatwebsite\Excel\Facades\Excel;

class ResellerPayoutRequestController extends Controller
{
    public function index(Request $req)
    {
        $resellerPayoutRequest = ResellerPayoutRequest::with(['bankDetails:id,bank_name,account_name,account_number,sort_code','user:id,first_name,last_name,country_id'])->select('id', 'user_id', 'amount', 'status', 'note', 'rejection_note', 'request_type', 'bank_details_id', 'created_at', 'accepted_at', 'rejected_at', 'updated_at')->latest();

        if ($user_id = $req->query('user_id', false)) {
            $resellerPayoutRequest->where('user_id', $user_id);
        }

        if ($query = $req->query('query', false)) {
            $resellerPayoutRequest->where('amount', 'like', $req->query);
            $resellerPayoutRequest->orWhere('status', 'like', $req->query);
            $resellerPayoutRequest->orWhere('note', 'like', $req->query);
            $resellerPayoutRequest->orWhere('rejection_note', 'like', $req->query);
            $resellerPayoutRequest->orWhere('request_type', 'like', $req->query);
        }

        if ($query = $req->query('from_date', false)) {
            $this->validate($req, [
                'from_date' => 'date_format:Y-m-d',
                'to_date' => 'required_with:from_date|date_format:Y-m-d|after_or_equal:from_date',
            ]);

            $from_date = Carbon::parse($req->input('from_date'))->startOfDay();
            $to_date = Carbon::parse($req->input('to_date'))->endOfDay();

            $resellerPayoutRequest = $resellerPayoutRequest->where('created_at', '>=', $from_date->getTimestamp());
            $resellerPayoutRequest = $resellerPayoutRequest->where('created_at', '<=', $to_date->getTimestamp());
        }
        if ($req->query('nopaginate', false) == '1') {
            $resellerPayoutRequest = $resellerPayoutRequest->get();
        } else {
            $resellerPayoutRequest = $resellerPayoutRequest->paginate($req->query('per_page', 20));
        }

        if ($resellerPayoutRequest->isEmpty()) Utils::error('No reseller payout requests found!', 404);

        return ($resellerPayoutRequest);
    }

    public function fetch(Request $request)
    {
        $users = User::with(['bankDetails:id,user_id,bank_name,account_name,account_number,account_type,sort_code'])->where('reseller', '1')->select('id','first_name', 'last_name', 'country_id')->get();

        return $users;
    }

    public function create(Request $req)
    {
        $this->validate($req, [
            'user_id' => 'required|exists:users,id',
            'amount' => 'required|numeric:gt,0',
            'note' => 'string',
            'status' => 'required|in:Pending,Accepted,Rejected',
            'request_type' => 'required|digits_between:1,4',
            'bank_details_id' => 'required_if:request_type,2|exists:bank_details,id',
            'rejection_note' => 'nullable|string'
        ], [
            'bank_details_id.required_if' => 'Bank details are required if your request type is bank transfer.',
        ]);

        try {
            $resellerPayoutRequest = new ResellerPayoutRequest;
            $resellerPayoutRequest->fill($req->all());

            if ($req->has('user_id')) {
                $user = User::find($req->input('user_id'));
                if (!empty($user) && $user->balance < $resellerPayoutRequest->amount) {
                    Utils::error("Unsufficeint balance", 400);
                }
            }

            $resellerPayoutRequest->save();

            if ($resellerPayoutRequest->status == 'Accepted') {

                $resellerPayoutRequest->accepted_at = Carbon::now();
                $resellerPayoutRequest->save();

                $passbook = new Passbook;
                $passbook->entity = "reseller_payout_request";
                $passbook->entity_id = $resellerPayoutRequest->id;
                $passbook->user_id = $resellerPayoutRequest->user_id;
                $passbook->date = Carbon::today()->toDateString();
                $passbook->from_date = $resellerPayoutRequest->created_at;
                $passbook->to_date = $resellerPayoutRequest->created_at;
                $passbook->amount = -1 * $resellerPayoutRequest->amount;
                $passbook->description = "Withdrawal request accepted dated on " . Carbon::parse($resellerPayoutRequest->created_at)->toDateString();
                $passbook->save();
            }

            if ($resellerPayoutRequest->status == 'Rejected') {
                $resellerPayoutRequest->rejected_at = Carbon::now();
                $resellerPayoutRequest->save();
            }
        } catch (Exception $e) {
            Utils::error($e->getMessage(), 400);
        }

        return $resellerPayoutRequest;
    }

    public function bulk_create(Request $request)
    {
        if ($request->has('payouts') && !is_array($request->input('payouts')))
            $request->merge(['payouts' => json_decode($request->input('payouts'))]);

        $this->validate($request, [
            'payouts' => 'array',
        ]);

        if ($request->has('payouts') && !empty($request->input('payouts')) || sizeof($request->input('payouts')) > 0) {
            
            // create payoutRequest
            $payouts = ($request->input('payouts'));
            foreach ($payouts as $payout) {
                $inputData = (array)$payout;
                $validator = Validator::make($inputData, [
                    'user_id' => 'required|exists:users,id',
                    'amount' => 'required|numeric:gt,0',
                    'note' => 'string',
                    'status' => 'required|in:Pending,Accepted,Rejected',
                    'request_type' => 'required|digits_between:1,4',
                    'bank_details_id' => 'required_if:request_type,2|exists:bank_details,id',
                    'rejection_note' => 'nullable|string'
                ], [
                    'bank_details_id.required_if' => 'Bank details are required if your request type is bank transfer.',
                ]);

                if ($validator->fails()) {
                    return $validator->errors();
                    Utils::error($validator->errors(), 400);
                }

                if ($payout->amount > 0) {
                    try {
                        $resellerPayoutRequest = new ResellerPayoutRequest;
                        $resellerPayoutRequest->fill($inputData);
            
                        if ($payout->user_id) {
                            $user = User::find($payout->user_id);
                            if (!empty($user) && $user->balance < $resellerPayoutRequest->amount) {
                                Utils::error("Unsufficeint balance", 400);
                            }
                        }
            
                        $resellerPayoutRequest->save();
            
                        if ($resellerPayoutRequest->status == 'Accepted') {
            
                            $resellerPayoutRequest->accepted_at = Carbon::now();
                            $resellerPayoutRequest->save();
            
                            $passbook = new Passbook;
                            $passbook->entity = "reseller_payout_request";
                            $passbook->entity_id = $resellerPayoutRequest->id;
                            $passbook->user_id = $resellerPayoutRequest->user_id;
                            $passbook->date = Carbon::today()->toDateString();
                            $passbook->from_date = $resellerPayoutRequest->created_at;
                            $passbook->to_date = $resellerPayoutRequest->created_at;
                            $passbook->amount = -1 * $resellerPayoutRequest->amount;
                            $passbook->description = "Withdrawal request accepted dated on " . Carbon::parse($resellerPayoutRequest->created_at)->toDateString();
                            $passbook->save();
                        }
            
                        if ($resellerPayoutRequest->status == 'Rejected') {
                            $resellerPayoutRequest->rejected_at = Carbon::now();
                            $resellerPayoutRequest->save();
                        }
                    } catch (Exception $e) {
                        Utils::error($e->getMessage(), 400);
                    }
                }
            }
        }
        return $payouts;
    }

    public function show(Request $req, $id)
    {
        $resellerPayoutRequest = ResellerPayoutRequest::find($id);
        if (empty($resellerPayoutRequest)) Utils::error('Reseller payout request not found!', 404);
        return $resellerPayoutRequest;
    }

    public function update(Request $req, $id)
    {
        $resellerPayoutRequest = ResellerPayoutRequest::find($id);
        if (empty($resellerPayoutRequest)) Utils::error('Reseller payout request not found!', 404);

        $this->validate($req, [
            'user_id' => 'required|exists:users,id',
            'amount' => 'required|numeric:gt,0',
            'note' => 'string',
            'status' => 'required|in:Pending,Accepted,Rejected',
            'request_type' => 'required|digits_between:1,4',
            'bank_details_id' => 'required_if:request_type,2|exists:bank_details,id',
            'rejection_note' => 'nullable|string'
        ], [
            'bank_details_id.required_if' => 'Bank details are required if your request type is bank transfer.',
        ]);

        // Update Seller Payout Request
        if ($req->has('amount')) $resellerPayoutRequest->amount = $req->input('amount');
        if ($req->has('note')) $resellerPayoutRequest->note = $req->input('note');
        if ($req->has('status')) $resellerPayoutRequest->status = $req->input('status');
        if ($req->has('request_type')) $resellerPayoutRequest->request_type = $req->input('request_type');
        if ($req->has('bank_details_id')) $resellerPayoutRequest->bank_details_id = $req->input('bank_details_id');
        if ($req->has('user_id')) {
            $user = User::find($req->input('user_id'));
            if (!empty($user) && $user->balance < $resellerPayoutRequest->amount) {
                Utils::error("Unsufficeint balance", 400);
            }
        }
        if ($req->input('status') == 'Accepted') {
            $resellerPayoutRequest->accepted_at = Carbon::now();
            $resellerPayoutRequest->rejected_at = null;
        }
        if ($req->input('status') == 'Rejected') {
            $resellerPayoutRequest->rejected_at = Carbon::now();
            $resellerPayoutRequest->accepted_at = null;
        }

        $passbook = Passbook::where('entity', 'payout_request')->where('entity_id', $resellerPayoutRequest->id);
        if ($req->has('user_id')) {
            $passbook->where('user_id', $resellerPayoutRequest->user_id);
        }
        if ($req->has('business_id')) {
            $passbook->where('business_id', $resellerPayoutRequest->business_id);
        }
        $passbook = $passbook->first();
        if ($resellerPayoutRequest->status == 'Accepted') {
            if (empty($passbook)) {
                $passbook = new Passbook;
            }
            $passbook->entity = "reseller_payout_request";
            $passbook->entity_id = $resellerPayoutRequest->id;
            $passbook->user_id = $resellerPayoutRequest->user_id;
            $passbook->date = Carbon::today()->toDateString();
            $passbook->from_date = $resellerPayoutRequest->created_at;
            $passbook->to_date = $resellerPayoutRequest->created_at;
            $passbook->amount = -1 * $resellerPayoutRequest->amount;
            $passbook->description = "Withdrawal request accepted dated on " . Carbon::parse($resellerPayoutRequest->created_at)->toDateString();
            $passbook->save();
        } else {
            if (!empty($passbook))
                $passbook->delete();
        }

        $resellerPayoutRequest->save();

        return $resellerPayoutRequest;
    }

    public function delete(Request $req, $id)
    {
        $resellerPayoutRequest = ResellerPayoutRequest::find($id);
        if (empty($resellerPayoutRequest)) Utils::error('Reseller payout request not found!', 404);
        $resellerPayoutRequest->delete();
        return $resellerPayoutRequest;
    }

    public function export(Request $req)
    {
        if ($req->input('from_date') == '' || $req->input('from_date') == 'null') {
            $req->request->remove('from_date');
            $req->request->remove('to_date');
        }
        $type = $req->query('format', 'excel');
        if ($type == 'excel') {
            $payoutRequests = ($this->index($req));
            $showUserName = true;
            $fileName = "ResellerPayoutRequest";
            if ($req->user_id) {
                $showUserName = false;
                $user = User::select('id', 'first_name', 'last_name')->find($req->user_id);
                $fileName = $user->first_name. "_" . $user->last_name."ResellerPayoutRequest";
            }
            return Excel::download(new ResellerPayoutRequestExport($payoutRequests, $showUserName), "$fileName.xls", \Maatwebsite\Excel\Excel::XLS);
        } else {
            Utils::error("Invalid export format", 400);
        }
    }
}
