<?php

namespace App\Http\Controllers;

use App\Helpers\Utils;
use App\ServiceEmail;
use Exception;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use Illuminate\Http\Response;

class ServiceEmailController extends Controller
{
    function __construct()
    {
    }

    /**
     * List email-services.
     * GET /api/email-services/
     */
    public function index(Request $req)
    {

        $mails = ServiceEmail::orderByRaw('created_at DESC');

        if ($query = $req->query('query', false)) {
            $mails->whereRaw('(name LIKE ? OR email LIKE ? )', ["%$query%", "%$query%"]);
        }
        if ($req->query('nopaginate', false) == '1') {
            $mails = $mails->get();
        } else {
            $mails = $mails->paginate($req->query('per_page', 20));
        }

        if ($mails->isEmpty()) Utils::error('No mails found!', 404);

        return ($mails);
    }

    /**
     * Create email-service.
     * POST /api/email-services
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'name' => 'required|string|max:255',
            'email' => 'required|email|unique:service_emails,email',
        ]);

        // create mail
        $mail = new ServiceEmail;
        $mail->fill($req->all());
        $mail->save();

        return $mail;
    }

    /**
     * Show email-service.
     * GET /api/email-services/{id}
     */
    public function show(Request $req, $id)
    {
        $mail = ServiceEmail::find($id);
        if (empty($mail)) Utils::error('Mail not found!', 404);
        return $mail;
    }

    /**
     * Show sent email-service.
     * GET /api/email-services/sent/{id}
     */
    public function show_sent(Request $req, $id)
    {
        $endPoint = 'show-sent-mail/' . $id;

        $client = new \GuzzleHttp\Client();
        $data = $client->request('GET', env('MAIL_API_URL') . $endPoint, [
            $req,
        ]);

        if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
            $data = json_decode($data->getBody()->getContents());
            $response = (array)$data->data;
            return $response;
        } else {
            Utils::error('No mail found!', 404);
        }
    }

    /**
     * Show receive email-service.
     * GET /api/email-services/receive/{id}
     */
    public function show_receive(Request $req, $id)
    {
        $endPoint = 'show-received-mail/' . $id;
        $client = new \GuzzleHttp\Client();
        $data = $client->request('GET', env('MAIL_API_URL') . $endPoint, [
            $req,
        ]);

        if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
            $data = json_decode($data->getBody()->getContents());
            $response = (array)$data->data;
            return $response;
        } else {
            Utils::error('No mail found!', 404);
        }
    }

    /**
     * Update email-service.
     * POST /api/email-services/{id}
     */
    public function update(Request $req, $id)
    {
        $mail = ServiceEmail::find($id);
        if (empty($mail)) Utils::error('Mail not found!', 404);

        $this->validate($req, [
            'name' => 'required|string|max:255',
            'email' => ['email', Rule::unique('service_emails')->ignore($mail->id)],
        ]);

        // update mail
        if ($req->has('name')) $mail->name = $req->input('name');
        if ($req->has('email')) $mail->email = $req->input('email');

        $mail->save();

        return $mail;
    }


    /**
     * List receive email-services.
     * GET /api/email-services/receive
     */
    public function index_receive(Request $req)
    {
        $email = $req->query('email');
        $endPoint = 'get-received-mail/' . $email;

        $client = new \GuzzleHttp\Client();
        $data = $client->request('GET', env('MAIL_API_URL') . $endPoint, [
            $req,
        ]);
        if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
            $data = json_decode($data->getBody()->getContents());
            $response = (array)$data->data;
            return $response;
        } else {
            Utils::error('No mails found!', 404);
        }
    }

    /**
     * List Send email-services.
     * GET /api/email-services/send
     */
    public function index_send(Request $req)
    {
        $email = $req->query('email');
        $endPoint = 'get-sent-mail/' . $email;

        $client = new \GuzzleHttp\Client();
        $data = $client->request('GET', env('MAIL_API_URL') . $endPoint, [
            $req,
        ]);

        if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
            $data = json_decode($data->getBody()->getContents());
            $response = (array)$data->data;
            return $response;
        } else {
            Utils::error('No mails found!', 404);
        }
    }

    public function send_mail(Request $req)
    {
        $this->validate($req, [
            'to_addresses' => 'required',
            'from_address' => 'email',
            'from_name' => 'string',
            'subject' => 'required|string',
            'body' => 'required|string',
            'body_type' => 'required|in:html,text',
            'attachment' => 'file|mimes:pdf,doc,docx,xls,xlsx,ppt,pptx,txt,rtf,html,zip,mp3,wav,mp4,mov,wmv,avi,mpg,jpg,jpeg,png,gif,svg|max:2048'
        ]);

        $toAddresses = $req->input('to_addresses');
        $toAddresses = preg_split("/\,/", $toAddresses);

        try {
            $apiURL = env('MAIL_API_URL') . 'send-mail';
            $postInput = [
                'to_addresses' => $toAddresses,
                'attachments' => $req->input('attachments'),
                'subject' => $req->input('subject'),
                'reply_to' => $req->input('reply_to'),
                'from_address' => $req->input('from_address'),
                'from_name' => $req->input('from_name'),
                'body' => $req->input('body'),
                'body_type' => $req->input('body_type')
            ];

            $client = new Client();
            $response = $client->request('POST', $apiURL, ['form_params' => $postInput]);
        } catch (Exception $e) {
            Utils::error($e->getMessage(), 500);
        }

        $statusCode = $response->getStatusCode();
        $responseBody = json_decode($response->getBody(), true);
        return $responseBody;
    }


    /**
     * Show email-service.
     * GET /api/email-services/change-status/{id}
     */
    public function change_status(Request $req, $id)
    {
        $endPoint = 'update-received-mail-status/' . $id;

        $client = new \GuzzleHttp\Client();
        $postInput = [
            'status' => $req->input('status'),
        ];

        $data = $client->request(
            'POST',
            env('MAIL_API_URL') . $endPoint,
            ['form_params' => $postInput],
        );

        if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
            $data = json_decode($data->getBody()->getContents());

            return [
                'message' => 'Status change successfully'
            ];
        } else {
            Utils::error('No mails found!', 404);
        }
    }

    /**
     * Delete email-service by id.
     * DELETE /api/email-services/receive/{id}
     */
    public function delete(Request $req, $id)
    {
        $mail = ServiceEmail::find($id);
        if (empty($mail)) Utils::error('Mail not found!', 404);
        $mail->delete();
        return $mail;
    }

    /**
     * Delete email-service by id.
     * DELETE /api/email-services/{id}
     */
    public function delete_receive(Request $req, $id)
    {
        $endPoint = 'delete-received-mail/' . $id;
        $client = new \GuzzleHttp\Client();

        $data = $client->request('POST', env('MAIL_API_URL') . $endPoint);
        if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
            return [
                'message' => 'Mail deleted successfully'
            ];
        } else {
            Utils::error('something want to wrong', 404);
        }
    }

    /**
     * Delete email-service by id.
     * DELETE /api/email-services/{id}/send
     */
    public function delete_send(Request $req, $id)
    {
        $endPoint = 'delete-sent-mail/' . $id;
        $client = new \GuzzleHttp\Client();

        $data = $client->request('POST', env('MAIL_API_URL') . $endPoint);
        if ($data->getStatusCode() >= 200 && $data->getStatusCode() <= 210) {
            return [
                'message' => 'Mail deleted successfully'
            ];
        } else {
            Utils::error('something want to wrong', 404);
        }
    }
}