<?php

namespace App\Http\Controllers;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\SiteSetting;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class SiteSettingController extends Controller
{
    /**
     * List siteSettings.
     * GET /api/site-settings
     */
    public function index(Request $req)
    {
        $siteSettings = SiteSetting::orderByRaw('position ASC, created_at ASC');
        if ($req->query('nopaginate', false) == '1') {
            $siteSettings = $siteSettings->get();
        } else {
            $siteSettings = $siteSettings->paginate($req->query('per_page', 20));
        }

        if ($siteSettings->isEmpty()) Utils::error('No siteSettings found!', 404);

        return ($siteSettings);
    }

    /**
     * Create siteSettings.
     * POST /api/site-settings
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'key' => 'required|alpha_dash|string|unique',
            'value' => 'required',
            'comment' => 'string',
            'hint' => 'string',
            'setting_group' => 'string',
            'data_type' => 'in:enum,string,int,double,bool,json',
        ]);

        // create siteSettings
        $siteSettings = new SiteSetting;
        $siteSettings->fill($req->all());

        $siteSettings->save();

        return $siteSettings;
    }

    /**
     * Show siteSettings.
     * GET /api/site-settings/{id}
     */
    public function show(Request $req, $id)
    {
        $siteSettings = SiteSetting::find($id);
        if (empty($siteSettings)) Utils::error('SiteSetting not found!', 404);
        return $siteSettings;
    }

    public function find_bank_string($id)
    {
        $siteSettings = SiteSetting::where('key', $id)->first();
        return $siteSettings;
    }

    public static function findByKey($key)
    {
        $siteSettings = SiteSetting::where('key', $key)->first();
        return $siteSettings;
    }
    /**
     * Update siteSettings.
     * POST /api/site-settings/{id}
     */
    public function update(Request $req, $id)
    {
        $siteSettings = SiteSetting::find($id);
        if (empty($siteSettings)) Utils::error('SiteSetting not found!', 404);

        $this->validate($req, [
            'key' => ['alpha_dash', 'string', Rule::unique('site_settings')->ignore($siteSettings->id)],
            'data_type' => 'in:enum,string,int,double,bool,json',
            'comment' => 'string',
            'hint' => 'string',
            'setting_group' => 'string',
        ]);

        // update siteSettings
        if ($req->has('key')) $siteSettings->key = $req->input('key');
        if ($req->has('value')) $siteSettings->value = $req->input('value');
        if ($req->has('data_type')) $siteSettings->data_type = $req->input('data_type');
        if ($req->has('comment')) $siteSettings->comment = $req->input('comment');
        if ($req->has('hint')) $siteSettings->hint = $req->input('hint');
        if ($req->has('setting_group')) $siteSettings->setting_group = $req->input('setting_group');

        $siteSettings->save();

        return $siteSettings;
    }


    /**
     * Delete siteSettings by id.
     * DELETE /api/site-settings/{id}
     */
    public function delete(Request $req, $id)
    {
        $siteSettings = SiteSetting::find($id);
        if (empty($siteSettings)) Utils::error('SiteSetting not found!', 404);
        $siteSettings->delete();
        return $siteSettings;
    }
    public function load_settings(Request $req)
    {
        $tables[] = [
            "name" => "Credit Card Mode",
            "key" => "stripe_mode",
            "comment" => "Credit card mode",
            "hint" => "test or live",
            "setting_group" => "Payment Method",
            "data_type" => "enum",
            "expected_values" => "live,test",
            "value" => 'test',
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Credit Card Public Key for Test Mode",
            "key" => "stripe_public_key_test",
            "comment" => "Credit Card Public Key for Test mode",
            "hint" => "Enter public key for Test mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => 'pk_test_AdQkfvDPjhh5OBqmlJ5DGCub',
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Credit Card Private Key for Test Mode",
            "key" => "stripe_private_key_test",
            "comment" => "Credit Card Private Key for Test mode",
            "hint" => "Enter private key for Test mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "sk_test_5J87rf5MsZJYIK36Jt9bveuI",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Credit Card Public Key for Live Mode",
            "key" => "stripe_public_key_live",
            "comment" => "Card Public Key for Live mode",
            "hint" => "Enter public key for Live mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "pk_live_1vbUyNpmwlvwkoLmpZtn5Gk6",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Credit Card Private Key for Live Mode",
            "key" => "stripe_private_key_live",
            "comment" => "Card Private Key for Live mode",
            "hint" => "Enter private key for Live mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "sk_live_M3enKM9GcFqbfhMGNkUU32oO",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Paypal Mode",
            "key" => "paypal_mode",
            "comment" => "Paypal mode",
            "hint" => "test or live",
            "setting_group" => "Payment Method",
            "data_type" => "enum",
            "expected_values" => "live,test",
            "value" => 'test',
            "position" => "5",
        ];
        $tables[] = [
            "name" => "Paypal Public Key for Test Mode",
            "key" => "paypal_public_key_test",
            "comment" => "Paypal Public Key for Test mode",
            "hint" => "Enter public key for test mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "AY_5fJxxSwWtvcQfVcCqTgngHv9wmc1WEdgi0njaLoTEsS9oouuC2akeT78cEPFavcglH7LC2ynDWTcd",
            "position" => "5",
        ];
        $tables[] = [
            "name" => "Paypal Private Key for Test Mode",
            "key" => "paypal_private_key_test",
            "comment" => "Paypal Private Key for Test mode",
            "hint" => "Enter private key for test mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "EDFMZIw9RhyKoYvh10vSq2KKhr6fD0SMJRFDcOWAQAZ-wEBBqvqUaa6v983oERQ6mq6-26PnC26gEpcQ",
            "position" => "5",
        ];

        $tables[] = [
            "name" => "Paypal Public Key for Live Mode",
            "key" => "paypal_public_key_live",
            "comment" => "Paypal Public Key for Live mode",
            "hint" => "Enter public key for live mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "ARFbYu4nOwxgdMytHfC0btbilHBtuJTumJJUOY7OEcCTQgxdRPFuzd1Fn48EKpB2WbLckaSqodLJivXL",
            "position" => "5",
        ];
        $tables[] = [
            "name" => "Paypal Private Key for Live Mode",
            "key" => "paypal_private_key_live",
            "comment" => "Paypal Private Key for Live mode",
            "hint" => "Enter private key for live mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "EFZGDsFgHF4ceSxhLJmQijmEL48sgVon_ciPTxeciyAnLCRWYut7-eZx4bD-llYL_g69-rQOxuZmd1e2",
            "position" => "5",
        ];
        $tables[] = [
            "name" => "Google Api key",
            "key" => "google_map_api_key",
            "comment" => "Google map api key",
            "hint" => "Enter google api key",
            "setting_group" => "Api Key",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "",
            "position" => "6",
        ];
        $tables[] = [
            "name" => "Connect Mode",
            "key" => "connect_stripe_mode",
            "comment" => "Connect mode",
            "hint" => "test or live",
            "setting_group" => "Payment Method",
            "data_type" => "enum",
            "expected_values" => "live,test",
            "value" => 'test',
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Connect Public Key for Test Mode",
            "key" => "connect_stripe_public_key_test",
            "comment" => "Connect Public Key for Test mode",
            "hint" => "Enter Connect public key for Test mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => 'pk_test_AdQkfvDPjhh5OBqmlJ5DGCub',
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Connect Private Key for Test Mode",
            "key" => "connect_stripe_private_key_test",
            "comment" => "Connect Private Key for Test mode",
            "hint" => "Enter Connect private key for Test mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "sk_test_5J87rf5MsZJYIK36Jt9bveuI",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Connect Public Key for Live Mode",
            "key" => "connect_stripe_public_key_live",
            "comment" => "Connect Public Key for Live mode",
            "hint" => "Enter Connect public key for Live mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "pk_live_1vbUyNpmwlvwkoLmpZtn5Gk6",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Connect Private Key for Live Mode",
            "key" => "connect_stripe_private_key_live",
            "comment" => "Connect Private Key for Live mode",
            "hint" => "Enter Connect private key for Live mode",
            "setting_group" => "Payment Method",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "sk_live_M3enKM9GcFqbfhMGNkUU32oO",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "FCM Api Key",
            "key" => "fcm_api_key",
            "comment" => "FCM Api Key",
            "hint" => "Enter FCM Api Key",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "AAAA5kKzELc:APA91bHAb-xGBmz20Ki3nX5-w-Ir0qpYIyHbUcaZlGsyFk_kGDUjQdAevEhfEAtw7n0Kn0M2mrxL69Qq2zsed5oJW8iezu4X5BlNccFMMMrq-QyrVnYWO8uFqCnNLwkskQGahZXSMJEg",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Epos Android App Version",
            "key" => "epos_android_app_version",
            "comment" => "Epos Android App Version",
            "hint" => "Enter Epos Android App Version",
            "setting_group" => "",
            "data_type" => "int",
            "expected_values" => null,
            "value" => "1",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Epos Ios App Version",
            "key" => "epos_ios_app_version",
            "comment" => "Epos Ios App Version",
            "hint" => "Enter Epos Ios App Version",
            "setting_group" => "",
            "data_type" => "int",
            "expected_values" => null,
            "value" => "1",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Bank String",
            "key" => "bank_string",
            "comment" => "Bank String",
            "hint" => "Enter Bank String",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "UBSIDI LIMITED,Tiffin Tom,TIFFINTOMUK LTD,Tiffin Tom Limited,Ubsidi Ltd,TIFFINTOM LIMITED,TiffinTomuk Ltd,Tiffintomuk Ltd n,tiffintomuk ltd c,Tiffintomuk Ltd cp",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Printing App Url Link",
            "key" => "printing_app_url_link",
            "comment" => "Printing App Url Link",
            "hint" => "Enter Printing App Url Link",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "https://cust.smartmps.co.uk/epos/Epos_Receiver_Sunmi.apk",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Default Connect Account",
            "key" => "default_connect_account",
            "comment" => "Default Connect Account",
            "hint" => "Enter Default Connect Account",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "acct_1J6bcrROMJ5OEf2b",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Caxton Fx Email",
            "key" => "caxton_email",
            "comment" => "Caxton Fx Email",
            "hint" => "Enter Caxton Fx Email",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "admin@ubsidi.com",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Caxton Fx Username",
            "key" => "caxton_username",
            "comment" => "Caxton Fx Username",
            "hint" => "Enter Caxton Fx Username",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "4d94fc2a-c7c4-4f2f-af9e-ce21a7988b3e",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Caxton Fx Password",
            "key" => "caxton_password",
            "comment" => "Caxton Fx Password",
            "hint" => "Enter Caxton Fx Password",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "HjF1mI5E53JlaieGAlCMJA==",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Caxton Fx Token",
            "key" => "caxton_token",
            "comment" => "Caxton Fx Token",
            "hint" => "Enter Caxton Fx Token",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "hGV0OHFDbzaiOxVAvrJGyftZh1cr81KL4q2UvnSzzOl8AT9NGnECO4vSaHKFEqdz7_c6ZOmNhA_6iWuvQU_zbD1rHfpax38ZLIy-fySGk_AqR8Qpt4OzxbZ8b77UsiqLXpdEei3IKVZu0MmTnVPtxxPj-JPixd-f2hxrDaD9ku4qut_trr-YnyxcdTwa8qEkSY1OdpCM9oDaiNb3wjV6xtJo5vG7LekBPrbk2T8EMia7rgjSII8GHl4eucsaDJHbfOmFht4u0lfT2g4FL7vFExlNWLbxB2HAJQPE5yoVj7aIluz6x9qlX9EZeEq2B0RSmHB46muPMObTe62UV5dGJ1Uw6rbbYot8H1rUP6lS4pJhL_uW-vg8_-c34VUkxVSU",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Caxton Fx User Id",
            "key" => "caxton_user_id",
            "comment" => "Caxton Fx User Id",
            "hint" => "Enter Caxton Fx User Id",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "C00979729",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Caxton Fx Public Key",
            "key" => "caxton_public_key",
            "comment" => "Caxton Fx Public Key",
            "hint" => "Enter Caxton Fx Public Key",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "rNoCzg8H0fmBYOwS",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Login Method",
            "key" => "admin_login_method",
            "comment" => "Login Method",
            "hint" => "Enter Login Method",
            "setting_group" => "",
            "data_type" => "enum",
            "expected_values" => 'authenticator,otp,none',
            "value" => "authenticator",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Wallet Amount",
            "key" => "pouch_weight",
            "comment" => "Wallet Amount",
            "hint" => "Enter Wallet Amount",
            "setting_group" => "",
            "data_type" => "double",
            "expected_values" => null,
            "value" => "100000",
            "position" => "0",
        ];
        $tables[] = [
            "name" => "Webhook Key",
            "key" => "webhook_key",
            "comment" => "Webhook Key",
            "hint" => "Enter Webhook Key",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "",
            "position" => "0",
        ];

        $tables[] = [
            "name" => "Aamarpay Sandbox Store Id",
            "key" => "aamarpay_sandbox_store_id",
            "comment" => "Aamarpay Sandbox Store Id",
            "hint" => "Enter Sandbox Store Id",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "aamarpaytest",
            "position" => "0",
        ];

        $tables[] = [
            "name" => "Aamarpay Sandbox Signature Key",
            "key" => "aamarpay_sandbox_signature_key",
            "comment" => "Aamarpay Sandbox Signature Key",
            "hint" => "Enter Sandbox Signature Key",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "dbb74894e82415a2f7ff0ec3a97e4183",
            "position" => "0",
        ];

        $tables[] = [
            "name" => "Aamarpay Sandbox Url",
            "key" => "aamarpay_sandbox_url",
            "comment" => "Aamarpay Sandbox Url",
            "hint" => "Enter Sandbox Url",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "https://sandbox.aamarpay.com",
            "position" => "0",
        ];

        $tables[] = [
            "name" => "Aamarpay Payment Mode",
            "key" => "aamarpay_mode",
            "comment" => "Aamarpay Payment Mode",
            "hint" => "test or live",
            "setting_group" => "Payment Method",
            "data_type" => "enum",
            "expected_values" => "live,test",
            "value" => 'test',
            "position" => "0",
        ];

        $tables[] = [
            "name" => "Aamarpay live Store Id",
            "key" => "aamarpay_live_store_id",
            "comment" => "Aamarpay live Store Id",
            "hint" => "Enter live Store Id",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "aamarpaytest",
            "position" => "0",
        ];

        $tables[] = [
            "name" => "Aamarpay live Signature Key",
            "key" => "aamarpay_live_signature_key",
            "comment" => "Aamarpay live Signature Key",
            "hint" => "Enter live Signature Key",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "dbb74894e82415a2f7ff0ec3a97e4183",
            "position" => "0",
        ];

        $tables[] = [
            "name" => "Aamarpay live Url",
            "key" => "aamarpay_live_url",
            "comment" => "Aamarpay live Url",
            "hint" => "Enter live Url",
            "setting_group" => "",
            "data_type" => "string",
            "expected_values" => null,
            "value" => "https://sandbox.aamarpay.com",
            "position" => "0",
        ];
        // $tables[] = [
        //     "name" => "Caxton Fx Main Password",
        //     "key" => "caxton_main_password",
        //     "comment" => "Caxton Fx Main Password",
        //     "hint" => "Enter Caxton Fx Main Password",
        //     "setting_group" => "",
        //     "data_type" => "string",
        //     "expected_values" => null,
        //     "value" => "guKIxlEmCND0TLX2vX+HPw==",
        //     "position" => "0",
        // ];
            
        foreach ($tables as $table) {

            $siteSetting = $this->findByKey($table['key']);
            if (empty($siteSetting)) {
                $siteSetting = new SiteSetting($table);
                $siteSetting->save();
            } else {
                error_log("Settings found for " . $table['key']);
            }
        }
        return SiteSetting::get();
    }

    public function stripe_public_key(Request $req)
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_public_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_public_key_live';
        }
        $siteSetting = SiteSettingController::findByKey($columns);
        return $siteSetting;
    }

    public function stripe_connect_public_key(Request $req)
    {
        $mode = SiteSettingController::findByKey('connect_stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'connect_stripe_public_key_test';
        }
        if ($mode == 'live') {
            $columns = 'connect_stripe_public_key_live';
        }
        $siteSetting = SiteSettingController::findByKey($columns);
        return $siteSetting;
    }

    public function default_account(Request $req)
    {
        $siteSetting = SiteSettingController::findByKey('default_connect_account');
        return $siteSetting;
    }
}
