<?php

namespace App\Http\Controllers\Tiffintom;

use App\Helpers\Auth;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomCategory;
use App\ModelsTiffintom\TiffintomMainAddon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomCategoryController extends Controller
{


    /**
     * List categories
     * GET /tiffintom/categories
     */
    public function index(Request $req)
    {
        $categories = TiffintomCategory::selectRaw("categories.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'restaurant_id')->orderBy('categories.sortorder', 'ASC');

        if ($query = $req->query("query", false)) {
            $categories->whereRaw('(category_name LIKE ? )', ["%$query%"]);
        }
        if ($query = $req->query("restaurant_id", false)) {
            $categories->where('restaurant_id', $query);
        }

        if ($status = $req->query("status", false)) {
            if ($status != 'all') {
                $categories->where('categories.status', $status);
            }
        } else {
            if (Auth::role() == 'guest' || Auth::role() == 'tiffintom_user') {
                $categories->where('categories.status', '1');
            }
        }
        $categories->where('categories.delete_status', 'N');

        if ($query = $req->query('print_block_id', false)) {
            if ($requirement = $req->query('requirement', false)) {
                $categories->whereRaw('(categories.id NOT IN (SELECT category_id from print_block_assign where print_block_id = ?))', [$req->input('print_block_id')]);
            }
        }

        if ($req->query('nopaginate', false) == '1') {
            $categories = $categories->get();
        } else {
            $categories = $categories->paginate($req->query('per_page', 20));
        }

        if ($categories->isEmpty()) Utils::error('No categories found!', 404);

        if ($query = $req->query("prefilled", false) && $req->query("is_suggested", false) != 1) {
            $categories->each(function ($items) {
                $items->append('menu');
            });
            $categoried = $categories;
            $categories = [];
            foreach ($categoried as $key => $value) {
                if (!$value->menu->isEmpty()) {
                    $categories[] = $value;
                }
            }
        }
        if ($query = $req->query("prefilled", false) && $req->query("is_suggested", false) == 1) {
            $categories->each(function ($items) {
                $items->append('menu_suggested');
            });
        }

        return $categories;
    }

    /**
     * Create category.
     * POST /tiffintom/categories
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'category_name' => 'required|string',
            'category_seo' => 'string',
            'sortorder' => 'numeric',
            'description' => 'string',
            'delete_status' => 'in:Y,N',
            'status' => 'boolean',
        ]);

        // create category
        $category = new TiffintomCategory;
        $category->fill($req->all());
        if ($req->has('category_name')) $category->category_name = (new ExtraController())->convertToTitleCase($req->input('category_name'));;
        if ($req->has('category_name')) $category->category_seo = $this->seoUrl($category->category_name);
        $category->save();
        return $category;
    }

    /**
     * List category
     * GET /tiffintom/categories/id
     */
    public function show(Request $req, $id)
    {
        $categories = TiffintomCategory::find($id);
        if (empty($categories)) Utils::error("categories not found", 404);
        return $categories;
    }

    /**
     * List category
     * GET /tiffintom/categories/id/get-addon
     */
    public function get_addon(Request $req, $id)
    {
        $categories = TiffintomCategory::find($id);
        if (empty($categories)) Utils::error("categories not found", 404);

        $categories->main_addon = TiffintomMainAddon::where('category_id', $categories->id)->orderBy('sortorder', 'ASC')->get();

        return $categories;
    }

    /**
     * Update category.
     * POST  /tiffintom/categories/id
     */
    public function update(Request $req, $id)
    {
        $category = TiffintomCategory::find($id);
        if (empty($category)) Utils::error('categories not found!', 404);

        $this->validate($req, [
            'restaurant_id' => 'required|exists:tiffintom_connection.restaurants,id',
            'category_name' => 'required|string',
            'category_seo' => 'string',
            'sortorder' => 'numeric',
            'description' => 'string',
            'delete_status' => 'in:Y,N',
            'status' => 'boolean',
        ]);

        // update category
        if ($req->has('restaurant_id')) $category->restaurant_id = $req->input('restaurant_id');
        if ($req->has('category_name')) $category->category_name = (new ExtraController())->convertToTitleCase($req->input('category_name'));
        if ($req->has('category_name')) $category->category_seo = $this->seoUrl($category->category_name);
        if ($req->has('description')) $category->description = $req->input('description');
        if ($req->has('delete_status')) $category->delete_status = $req->input('delete_status');
        $category->sortorder = $req->input('sortorder');
        $category->status = $req->input('status');
        $category->save();

        return $category;
    }

    /**
     * Delete category by id.
     * DELETE /tiffintom/categories/{id}
     */
    public function delete(Request $req, $id)
    {
        $category = TiffintomCategory::find($id);
        if (empty($category)) Utils::error('categories not found!', 404);
        $category->delete();
        return $category;
    }

    /**
     * Enable category.
     * POST /tiffintom/categories/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $category = TiffintomCategory::find($id);
        if (empty($category)) Utils::error('categories not found!', 404);
        $category->status = '1';
        $category->save();
        return $category;
    }

    /**
     * Disable category.
     * POST /tiffintom/categories/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $category = TiffintomCategory::find($id);
        if (empty($category)) Utils::error('categories not found!', 404);
        $category->status = '0';
        $category->save();
        return $category;
    }

    //Convert to seo url name
    public function seoUrl($string)
    {
        $text = preg_replace('~[^\\pL\d]+~u', '-', $string);

        // trim
        $text = trim($text, '-');

        // lowercase
        $text = strtolower($text);

        // remove unwanted characters
        $text = preg_replace('~[^-\w]+~', '', $text);

        if (strlen($text) > 70) {
            $text = substr($text, 0, 70);
        }

        if (empty($text)) {
            //return 'n-a';
            return time();
        }

        return $text;
    }

    /**
     * Bulk Upload category.
     * POST /tiffintom/categories/bulk
     */
    public function bulk_category(Request $req)
    {
        if ($req->has('categories') && !is_array($req->input('categories')))
            $req->merge(['categories' => json_decode($req->input('categories'))]);
        $this->validate($req, [
            'categories' => 'required|array'
        ]);

        $restaurantCategories = ($req->input('categories'));
        $count = 1;
        foreach ($restaurantCategories as $category) {
            $categoryObj = TiffintomCategory::find($category->id);
            $categoryObj->sortorder = $count;
            $categoryObj->save();
            $count++;
        }
        return $restaurantCategories;
    }
}
