<?php

namespace App\Http\Controllers\Tiffintom;

use App\Business;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ExtraController;
use App\ModelsTiffintom\TiffintomOffer;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TiffintomOfferController extends Controller
{
    /**
     * List offers
     * GET /tiffintom/offers
     */
    public function index(Request $req)
    {
        $offers = TiffintomOffer::selectRaw("offers.*,restaurants.restaurant_name as restaurant_name")->leftJoin('restaurants', 'restaurants.id', '=', 'resid')->orderBy('id', 'ASC');

        if ($query = $req->query("query", false)) {
            $offers->whereRaw('(normal_percentage)', ["%$query%"]);
        }
        if ($query = $req->query("restaurant_id", false)) {
            $offers->where('resid', $query);
            // User wise check
            if ($req->hasHeader('business-id')) {
                $business = Business::where('restaurant_id', $query)->first();
                if (empty($business)) Utils::error('User not authenticated.', 404);
                (new ExtraController())->userCheck($req->header('business-id'), $business->id);
            }
        }
        if ($status = $req->query("status", false)) {
            $offers->where('status', $status);
        }

        if ($req->query('nopaginate', false) == '1') {
            $offers = $offers->get();
        } else {
            $offers = $offers->paginate($req->query('per_page', 20));
        }

        if ($offers->isEmpty()) Utils::error('TiffintomOffer not found!', 404);

        return $offers;
    }

    /**
     * Create offer.
     * POST /tiffintom/offers
     */
    public function create(Request $req)
    {
        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'first_user' => 'in:Y,N',
            'normal' => 'in:Y,N',
            'free_percentage' => 'required_if:first_user,Y|numeric|gt:0',
            'free_price' => 'required_if:first_user,Y|numeric|gt:0',
            'normal_percentage' => 'required_if:normal,Y|numeric|gt:0',
            'normal_price' => 'required_if:normal,Y|numeric|gt:0',
            'offer_from' => 'required|date_format:Y-m-d',
            'offer_to' => 'required|date_format:Y-m-d|after:offer_from',
            'offer_mode' => 'required|in:flat,percentage',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $req->input('resid'))->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        // create offer
        $offer = new TiffintomOffer;
        $offer->fill($req->all());
        $offer->save();
        return $offer;
    }

    /**
     * List offers
     * GET /tiffintom/offers/id
     */
    public function show(Request $req, $id)
    {
        $offer = TiffintomOffer::find($id);
        if (empty($offer)) Utils::error("TiffintomOffer not found", 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $offer->resid)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        return $offer;
    }

    /**
     * Update offer.
     * POST  /tiffintom/offers/id
     */
    public function update(Request $req, $id)
    {
        $offer = TiffintomOffer::find($id);
        if (empty($offer)) Utils::error('TiffintomOffer not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $offer->resid)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $this->validate($req, [
            'resid' => 'required|exists:tiffintom_connection.restaurants,id',
            'first_user' => 'in:Y,N',
            'normal' => 'in:Y,N',
            'free_percentage' => 'required_if:first_user,Y|numeric|gt:0',
            'free_price' => 'required_if:first_user,Y|numeric|gt:0',
            'normal_percentage' => 'required_if:normal,Y|numeric|gt:0',
            'normal_price' => 'required_if:normal,Y|numeric|gt:0',
            'offer_from' => 'required|date_format:Y-m-d',
            'offer_to' => 'required|date_format:Y-m-d|after:offer_from',
            'offer_mode' => 'required|in:flat,percentage',
            'status' => 'boolean',
            'delivery_type' => 'boolean',
            'pickup_type' => 'boolean',
            'dinein_type' => 'boolean',
        ]);

        // update offer
        if ($req->has('resid')) $offer->resid = $req->input('resid');
        if ($req->has('first_user')) $offer->first_user = $req->input('first_user');
        if ($req->has('normal')) $offer->normal = $req->input('normal');
        if ($req->has('free_percentage')) $offer->free_percentage = $req->input('free_percentage');
        if ($req->has('free_price')) $offer->free_price = $req->input('free_price');
        if ($req->has('normal_percentage')) $offer->normal_percentage = $req->input('normal_percentage');
        if ($req->has('normal_price')) $offer->normal_price = $req->input('normal_price');
        if ($req->has('offer_from')) $offer->offer_from = $req->input('offer_from');
        if ($req->has('offer_to')) $offer->offer_to = $req->input('offer_to');
        if ($req->has('offer_mode')) $offer->offer_mode = $req->input('offer_mode');
        $offer->status = $req->input('status');
        $offer->delivery_type = $req->input('delivery_type');
        $offer->pickup_type = $req->input('pickup_type');
        $offer->dinein_type = $req->input('dinein_type');
        $offer->save();

        return $offer;
    }

    /**
     * Delete offer by id.
     * DELETE /tiffintom/offers/{id}
     */
    public function delete(Request $req, $id)
    {
        $offer = TiffintomOffer::find($id);
        if (empty($offer)) Utils::error('TiffintomOffer not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $offer->resid)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $offer->delete();
        return $offer;
    }

    /**
     * Enable offer.
     * POST /tiffintom/offers/{id}/enable
     */
    public function enable(Request $req, $id)
    {
        $offer = TiffintomOffer::find($id);
        if (empty($offer)) Utils::error('TiffintomOffer not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $offer->resid)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $offer->status = '1';
        $offer->save();
        return $offer;
    }

    /**
     * Disable offer.
     * POST /tiffintom/offers/{id}/disable
     */
    public function disable(Request $req, $id)
    {
        $offer = TiffintomOffer::find($id);
        if (empty($offer)) Utils::error('TiffintomOffer not found!', 404);

        // User wise check
        if ($req->hasHeader('business-id')) {
            $business = Business::where('restaurant_id', $offer->resid)->first();
            if (empty($business)) Utils::error('User not authenticated.', 404);
            (new ExtraController())->userCheck($req->header('business-id'), $business->id);
        }

        $offer->status = '0';
        $offer->save();
        return $offer;
    }

    /**
     * Bulk Upload offer.
     * POST /tiffintom/offers/{id}/disable
     */
    public function bulk_offer(Request $req)
    {
        if ($req->has('offers') && !is_array($req->input('offers')))
            $req->merge(['offers' => json_decode($req->input('offers'))]);
        $this->validate($req, [
            'offers' => 'required|array'
        ]);

        $offers = ($req->input('offers'));

        foreach ($offers as $offer) {
            $offerItem = null;
            if (isset($offer->id)) {
                $offerItem = TiffintomOffer::find($offer->id);
            }
            if (empty($offerItem)) {
                $offerItem = new TiffintomOffer((array)$offer);
                $offerItem->save();
            } else {
                $offerItem->resid = $offer->resid;
                $offerItem->first_user = $offer->first_user;
                $offerItem->normal = $offer->normal;
                $offerItem->free_percentage = $offer->free_percentage;
                $offerItem->free_price = $offer->free_price;
                $offerItem->normal_percentage = $offer->normal_percentage;
                $offerItem->normal_price = $offer->normal_price;
                $offerItem->offer_from = $offer->offer_from;
                $offerItem->offer_to = $offer->offer_to;
                $offerItem->offer_mode = $offer->offer_mode;
                $offerItem->status = (int) $offer->status;
                $offerItem->delivery_type = $offer->delivery_type;
                $offerItem->pickup_type = $offer->pickup_type;
                $offerItem->dinein_type = $offer->dinein_type;
                $offerItem->save();
            }
        }

        return  $offers;
    }
}
