<?php

namespace App\Http\Controllers;

use App\Business;
use App\Traits\ResponseTrait;
use App\UbsidiPaymentGatewayKey;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Validator;
use Stripe\StripeClient;
use Illuminate\Support\Str;

class UbsidiPaymentGatewayController extends Controller
{
    use ResponseTrait;
    private $stripe;

    public function __construct()
    {
        $mode = SiteSettingController::findByKey('stripe_mode')->value;
        if ($mode == 'test') {
            $columns = 'stripe_private_key_test';
        }
        if ($mode == 'live') {
            $columns = 'stripe_private_key_live';
        }
        $key = SiteSettingController::findByKey($columns)->value;
        $this->stripe = new StripeClient([
            'api_key' => $key,
            'stripe_version' => '2020-08-27', // Replace with the version you want to use
        ]);
    }

    public function generateKeys(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'business_id' => 'required|exists:businesses,id',
            'is_test_keys' => 'required|boolean',
        ], [
            'is_test_keys.boolean' => 'The is test keys field must be 0 or 1.'
        ]);

        if ($validator->fails()) {
            return $this->error([], $validator->errors()->first());
        }

        $keys = UbsidiPaymentGatewayKey::where('business_id', $request->business_id)->first();
        if (!$keys) {
            $keys = new UbsidiPaymentGatewayKey();
            $keys->business_id = $request->business_id;
        }

        if ((bool)$request->is_test_keys) {
            $keys->test_public_key = 'tpk_' . Str::random(32);
            $keys->test_secret_key = 'tsk_' . Str::random(32);
        } else {
            $keys->public_key = Str::random(32);
            $keys->secret_key = Str::random(32);
        }
        $keys->save();

        return $this->success(['keys' => $keys], 'Keys generated successfully.');
    }

    public function paymentSchema(Request $request)
    {
        $testMode = true;
        if ($request->headers->has('api-key') && $request->header('api-key') != null) {
            $key = UbsidiPaymentGatewayKey::where('test_secret_key', $request->header('api-key'))->first();
            if (!$key) {
                $key = UbsidiPaymentGatewayKey::where('secret_key', $request->header('api-key'))->first();
                if (!$key) {
                    return $this->error([], 'API key not exists.');
                }
                $testMode = false;
            }
        } else {
            return $this->error([], 'API authentication is required.');
        }

        $validator = Validator::make($request->all(), [
            'amount' => 'required|numeric|min:1',
            'card_number' => 'required|digits:16',
            'expiry_month' => 'required|digits:2',
            'expiry_year' => 'required|digits:2',
            'cvv' => 'required|digits:3',
        ]);

        if ($validator->fails()) {
            return $this->error([], $validator->errors()->first());
        }
        try {
            $token = $this->stripe->tokens->create([
                'card' => [
                    'number' => $request->card_number,
                    'exp_month' => $request->expiry_month,
                    'exp_year' => $request->expiry_year,
                    'cvc' => $request->cvv,
                ],
            ]);
            dd($token);

            $paymentMethod = $this->stripe->paymentMethods->create([
                'type' => 'card',
                'card' => [
                    'number' => $request->card_number,
                    'exp_month' => $request->expiry_month,
                    'exp_year' => $request->expiry_year,
                    'cvc' => $request->cvv,
                ],
            ]);

            $paymentIntent = $this->stripe->paymentIntents->create([
                'amount' => (float)$request->amount * 100,
                'currency' => 'gbp',
                'payment_method' => $paymentMethod->id,
                'confirm' => true,
                'capture_method' => 'automatic',
                'payment_method_options' => ['card' => ['capture_method' => 'manual']],
                'payment_method_types' => ['card']
            ]);

            $paymentSchema = [
                'payment_schema_id' => $paymentIntent->id,
                'schema_secret' => $paymentIntent->client_secret,
                'amount' => (float)$paymentIntent->amount / 100,
                'status' => 'requires_confirmation',
                'created' => Carbon::parse($paymentIntent->created)->format('d-m-Y h:i A'),
            ];

            if ($paymentIntent) {
                return $this->success($paymentSchema, 'Payment Schema created successfully.');
            } else {
                return $this->error();
            }
        } catch (\Exception $e) {
            return $this->error([], $e->getMessage());
        }
    }

    public function refund(Request $request)
    {
        $testMode = 1;
        if ($request->headers->has('api-key') && $request->header('api-key') != null) {
            $key = UbsidiPaymentGatewayKey::where('test_secret_key', $request->header('api-key'))->first();
            if (!$key) {
                $key = UbsidiPaymentGatewayKey::where('secret_key', $request->header('api-key'))->first();
                if (!$key) {
                    return $this->error([], 'API key not exists.');
                }
                $testMode = 0;
            }
        } else {
            return $this->error([], 'API authentication is required.');
        }

        $validator = Validator::make($request->all(), [
            'amount' => 'required|numeric|min:1',
            'charge_id' => 'required'
        ]);

        if ($validator->fails()) {
            return $this->error([], $validator->errors()->first());
        }

        try {
            $refund = $this->stripe->refunds->create([
                'charge' => $request->charge_id,
                'amount' => (float)$request->amount * 100
            ]);

            $refundData = [
                'refund_id' => $refund->id,
                'transaction_id' => $refund->balance_transaction,
                'amount' => (float)$refund->amount / 100,
                'status' => $refund->status
            ];

            if ($refund->status == 'succeeded') {
                return $this->success($refundData, 'Payment refunded successfully.');
            } else {
                return $this->error();
            }
        } catch (\Exception $e) {
            return $this->error([], $e->getMessage());
        }
    }

    public function transactions(Request $request)
    {
        $testMode = 1;
        if ($request->headers->has('api-key') && $request->header('api-key') != null) {
            $key = UbsidiPaymentGatewayKey::where('test_secret_key', $request->header('api-key'))->first();
            if (!$key) {
                $key = UbsidiPaymentGatewayKey::where('secret_key', $request->header('api-key'))->first();
                if (!$key) {
                    return $this->error([], 'API key not exists.');
                }
                $testMode = 0;
            }
        } else {
            return $this->error([], 'API authentication is required.');
        }

        $business = $key->business;
        $request->merge(["business_id" => $business->id]);
        $request->merge(["gateway_mode_test" => $testMode]);

        return (new StripeTransactionController())->index($request);
    }

    public function confirmSchema(Request $request)
    {
        $testMode = 1;
        if ($request->headers->has('api-key') && $request->header('api-key') != null) {
            $key = UbsidiPaymentGatewayKey::where('test_secret_key', $request->header('api-key'))->first();
            if (!$key) {
                $key = UbsidiPaymentGatewayKey::where('secret_key', $request->header('api-key'))->first();
                if (!$key) {
                    return $this->error([], 'API key not exists.');
                }
                $testMode = 0;
            }
        } else {
            return $this->error([], 'API authentication is required.');
        }

        $validator = Validator::make($request->all(), [
            'payment_intent_id' => 'required',
        ]);

        if ($validator->fails()) {
            return $this->error([], $validator->errors()->first());
        }

        try {
            $paymentIntent = $this->stripe->paymentIntents->capture($request->payment_intent_id, []);

            dd($paymentIntent);

            if ($paymentIntent->status == 'succeeded') {
                return $this->success([], 'Payment confirmed successfully.');
            } else {
                return $this->error();
            }
        } catch (\Exception $e) {
            return $this->error([], $e->getMessage());
        }
    }

    public function getStripeKey(Request $request)
    {
        $testMode = 1;
        if ($request->headers->has('api-key') && $request->header('api-key') != null) {
            $key = UbsidiPaymentGatewayKey::where('test_public_key', $request->header('api-key'))->first();
            if (!$key) {
                $key = UbsidiPaymentGatewayKey::where('public_key', $request->header('api-key'))->first();
                if (!$key) {
                    return $this->error([], 'API key not exists.');
                }
                $testMode = 0;
            }
        } else {
            return $this->error([], 'API authentication is required.');
        }


        $business = $key->business;
        $stripePublicKey = null;

        if ($business->stripe_mode == 'Live') {
            $stripePublicKey = $business->stripe_public_key_live;
            if ($testMode && $stripePublicKey != null) {
                return $this->error([], 'You are using Test mode key, But your business mode is Live.');
            }
        } elseif ($business->stripe_mode == 'Test') {
            $stripePublicKey = $business->stripe_public_key_test;
            if (!$testMode && $stripePublicKey != null) {
                return $this->error([], 'You are using Live mode key, But your business mode is Test.');
            }
        }

        if ($stripePublicKey == null) {
            $mode = SiteSettingController::findByKey('stripe_mode')->value;
            if ($mode == 'live') {
                $columns = 'stripe_public_key_live';
                $stripePublicKey = SiteSettingController::findByKey($columns)->value;
                if ($testMode && $stripePublicKey != null) {
                    return $this->error([], 'You are using Test mode key, But your business mode is Live.');
                }
            }
            if ($mode == 'test') {
                $columns = 'stripe_public_key_test';
                $stripePublicKey = SiteSettingController::findByKey($columns)->value;
                if (!$testMode && $stripePublicKey != null) {
                    return $this->error([], 'You are using Live mode key, But your business mode is Test.');
                }
            }
        }

        if ($stripePublicKey != null) {
            return $this->success(['key' => $stripePublicKey], 'Key fetched successfully.');
        } else {
            return $this->error();
        }
    }

    public function createPaymentSchema(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'amount' => 'required|numeric|min:1',
            'card_number' => 'required|digits:16',
            'expiry_month' => 'required|digits:2',
            'expiry_year' => 'required|digits:2',
            'cvv' => 'required|digits:3',
        ]);

        if ($validator->fails()) {
            return $this->error([], $validator->errors()->first());
        }

        $apiData = [
            'amount' => $request->amount,
            'card_number' => $request->card_number,
            'expiry_month' => $request->expiry_month,
            'expiry_year' => $request->expiry_year,
            'cvv' => $request->cvv,
        ];
        // $apiData = [
        //     'name' => 'Sneh',
        //     'job' => 'Software Engineer',
        // ];

        // $curl = curl_init();

        // curl_setopt_array($curl, array(
        //     CURLOPT_URL => "http://192.168.1.42:8000/api/ubsidi/payment-schema", // your preferred url
        //     CURLOPT_RETURNTRANSFER => true,
        //     CURLOPT_ENCODING => "",
        //     CURLOPT_MAXREDIRS => 10,
        //     CURLOPT_TIMEOUT => 3000,
        //     CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        //     CURLOPT_CUSTOMREQUEST => "POST",
        //     CURLOPT_POSTFIELDS => json_encode($apiData),
        //     CURLOPT_HTTPHEADER => array(
        //         // Set here requred headers
        //         "accept: */*",
        //         "accept-language: en-US,en;q=0.8",
        //         "content-type: application/json",
        //         "api-key: tsk_jDul3pj7qME9KJXat1TbjOx4Fr7JUvxl"
        //     ),
        // ));

        // $response = curl_exec($curl);
        // $err = curl_error($curl);

        // curl_close($curl);

        // if ($err) {
        //     echo "cURL Error #:" . $err;
        // } else {
        //     print_r(json_decode($response));
        // }

        dd('test');

        // $jsonData = $response->json();
        // dd($jsonData);
    }
}
