<?php

namespace App\Http\Controllers;

use App\Helpers\ImageHelper;
use App\UserBusiness;
use App\Helpers\Utils;
use App\Http\Controllers\Controller;
use App\ResellerCommission;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class UserBusinessController extends Controller
{
    /**
     * List userBusinesses.
     * GET /api/user-businesses
     */
    public function index(Request $req)
    {
        $userBusinesses = UserBusiness::selectRaw('businesses.joining_date,businesses.email,businesses.name,user_businesses.*,users.*,user_businesses.id as id')->join('businesses', 'businesses.id', '=', 'user_businesses.business_id')->join('users', 'users.id', '=', 'user_businesses.user_id')->orderByRaw('user_businesses.created_at DESC');

        if ($query = $req->query('user_id', false)) {
            $userBusinesses->where('user_id', $query);
        }
        if ($query = $req->query('business_id', false)) {
            $userBusinesses->where('business_id', $query);
        }
        $userBusinesses = $userBusinesses->get();

        if ($userBusinesses->isEmpty()) Utils::error('No userBusinesses found!', 404);

        return ($userBusinesses);
    }

    /**
     * Create userBusiness.
     * POST /api/user-businesses
     */
    public function create(Request $req)
    {
        $user = User::find($req->input('user_id'));
        $req->merge(['reseller' => $user->reseller]);
        if ($req->has('reseller_commissions') && is_array($req->input('reseller_commission'))) {
            $req->merge(['reseller_commissions', json_encode($req->input('reseller_commission'))]);
        }
        $this->validate($req, [

            'user_id' =>  ['required', 'exists:users,id', Rule::unique('user_businesses')->where(function ($query) use ($req) {
                return $query->where('user_id', $req->input('user_id'))->where('business_id', $req->input('business_id'));
            })],
            'business_id' =>  ['required', 'exists:businesses,id', Rule::unique('user_businesses')->where(function ($query) use ($req) {
                return $query->where('user_id', $req->input('user_id'))->where('business_id', $req->input('business_id'));
            })],
            'reseller_commissions' => 'required_if:reseller,1|json',

        ], [
            'user_id.unique' => "Business already assigned with the user",
            'business_id.unique' => "User already assigned with the business",
        ]);
        if ($user->reseller) {
            $userBusinessesResellers = UserBusiness::where('business_id', $req->input('business_id'))->whereRaw('(user_id IN (SELECT id FROM users where reseller=1))')->count();
            if ($userBusinessesResellers > 0) Utils::error("Reseller is already assigned with this business", 400);
        }
        // create userBusiness
        $userBusiness = new UserBusiness;
        $userBusiness->fill($req->all());

        $userBusiness->save();

        if ($req->has('reseller_commissions')) {
            $req = $req->merge(json_decode($req->reseller_commissions, true));
            (new ResellerCommissionController())->create($req);
        }

        return $userBusiness;
    }

    /**
     * Show userBusiness.
     * GET /api/user-businesses/{id}
     */
    public function show(Request $req, $id)
    {
        $userBusiness = UserBusiness::find($id);
        if (empty($userBusiness)) Utils::error('UserBusiness not found!', 404);
        return $userBusiness;
    }

    /**
     * Update userBusiness.
     * POST /api/user-businesses/{id}
     */
    public function update(Request $req, $id)
    {
        $userBusiness = UserBusiness::find($id);
        if (empty($userBusiness)) Utils::error('UserBusiness not found!', 404);
        $user = User::find($userBusiness->user_id);
        $req->merge(['reseller' => $user->reseller]);
        if ($req->has('reseller_commissions') && is_array($req->input('reseller_commission'))) {
            $req->merge(['reseller_commissions', json_encode($req->input('reseller_commission'))]);
        }
        $this->validate($req, [

            'user_id' =>  ['exists:users,id', Rule::unique('user_businesses')->where(function ($query) use ($req, $id) {
                return $query->where('user_id', $req->input('user_id'))->where('business_id', $req->input('business_id'))->whereNotIn('id', [$id]);
            })],
            'business_id' =>  ['exists:businesses,id', Rule::unique('user_businesses')->where(function ($query) use ($req, $id) {
                return $query->where('user_id', $req->input('user_id'))->where('business_id', $req->input('business_id'))->whereNotIn('id', [$id]);
            })],
            'reseller_commissions' => 'required_if:reseller,1|json',

        ], [
            'user_id.unique' => "Business already assigned with the user",
            'business_id.unique' => "User already assigned with the business",
        ]);
        if ($user->reseller) {
            $userBusinessesResellers = UserBusiness::where('business_id', $req->input('business_id'))->whereRaw('(user_id IN (SELECT id FROM users where reseller=1))')->where('id', '!=', $id)->count();
            if ($user->reseller && $userBusinessesResellers > 0) Utils::error("Reseller is already assigned with this business", 400);
        }
        // update userBusiness
        if ($req->has('user_id')) $userBusiness->user_id = $req->input('user_id');
        if ($req->has('business_id')) $userBusiness->business_id = $req->input('business_id');

        $userBusiness->save();
        if ($req->has('reseller_commissions')) {
            $req = $req->merge(json_decode($req->reseller_commissions, true));
            $resellerCommission = ResellerCommission::where('user_id', $userBusiness->user_id)->where('business_id', $userBusiness->business_id)->latest()->first();
            if (empty($resellerCommission)) {
                (new ResellerCommissionController())->create($req);
            } else (new ResellerCommissionController())->update($req, $resellerCommission->id);
        }

        return $userBusiness;
    }

    /**
     * Delete userBusiness by id.
     * DELETE /api/user-businesses/{id}
     */
    public function delete(Request $req, $id)
    {
        $userBusiness = UserBusiness::find($id);
        if (empty($userBusiness)) Utils::error('UserBusiness not found!', 404);
        $userBusiness->delete();
        if ($userBusiness->image != null) ImageHelper::delete(ImageHelper::$TYPE_BRAND, $userBusiness->image);
        ResellerCommission::where('user_id', $userBusiness->user_id)->where('business_id', $userBusiness->business_id)->delete();
        return $userBusiness;
    }
}
