<?php

namespace App;

use App\Http\Controllers\SiteSettingController;
use Carbon\Carbon;
use DateTime;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Log;

class StripeTransaction extends Model
{
    protected $fillable = [
        'txn_id',
        'payment_intent_id',
        'service',
        'object',
        'description',
        'descriptor',
        'amount',
        'fee',
        'net',
        'currency',
        'reporting_category',
        'source',
        'status',
        'type',
        'available_on',
        'created',

        'cash_handling_fee_type',
        'cash_handling_fee',

        'management_fee_type',
        'management_fee',

        'connect_card_processing_fee_type',
        'connect_card_processing_fee',
        'connect_card_auth_fee_type',
        'connect_card_auth_fee',

        'merchant_card_processing_fee_type',
        'merchant_card_processing_fee',
        'merchant_card_auth_fee_type',
        'merchant_card_auth_fee',

        'charge_back_fee_type',
        'charge_back_fee',

        'card_brand',
        'card_number',
        'card_type',
        'card_country',
        'email',

        'non_domestic',
        'charges_refundable',  //no, yes

        'mode', //live,test
        'account_type', //normal, connect

        'refunded_amount',
        'transfer_status',
        'dispute_status',
        'payout_status',
        'metadata',
    ];
    // protected $hidden = ['updated_at'];
    protected $appends = ['transaction_date_time'];
    protected $casts = ['non_domestic' => 'bool', 'archived' => 'bool', 'refunded' => 'bool', 'metadata' => 'json',];

    public function getTransactionDateTimeAttribute()
    {
        return Carbon::createFromTimestamp($this->created)->format('Y-m-d H:i:s');
    }
    public function getOnlineOrderAttribute()
    {
        return OnlineOrder::where('txn_id', $this->payment_intent_id)->first();
    }
    public function getCalculatedFeesAttribute()
    {
        $order = OnlineOrder::leftJoin('order_commissions', 'order_commissions.online_order_id', '=', 'online_orders.id')->where('txn_id', $this->payment_intent_id)->first();
        if (!empty($order)) {
            return $this->calculate_order_fees($this, $order);
        } else {
            if ($this->service == 'Connect') {
                return $this->calculate_connect_fees($this);
            } else {
                return $this->calculate_merchant_fees($this);
            }
        }
    }
    public function getBusinessAttribute()
    {
        $business = Business::select('id', 'name', 'connect_service')->find($this->business_id);
        $business->makeHidden('business_commissions');
        $business->makeHidden('user_counts');

        return $business;
    }
    public function getResellerStripeTransactionsAttribute()
    {
        return ResellerStripeCommission::where('stripe_transaction_id', $this->id)->first();
    }
    public function getCountryCommissionHistoryAttribute()
    {
        return CountryCommissionHistory::where('stripe_transaction_id', $this->id)->first();
    }
    public function getCardBrandCommissionAttribute()
    {
        return CardBrandCommissionHistory::where('stripe_transaction_id', $this->id)->first();
    }
    public function getDirectStripeTransactionAttribute()
    {
        return $this->direct_stripe_transaction();
    }
    // Attributes
    //
    public function calculate_connect_fees($transaction)
    {
        $cardProcessingFee = 0.00;
        $cardAuthFee = 0.00;
        $managementFee = 0.00;
        $handlingFee = 0.00;
        $chargeBackFee = 0.00;
        $cardCountry = $transaction->card_country;
        $brand = $transaction->card_brand;
        $cardBrandWiseCommission = 0.00;
        $countryWiseCommission = 0.00;

        $humanAmount = $transaction->amount / 100;
        if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
            $cardAuthFee  = $this->calculateFee($humanAmount, $transaction->connect_card_auth_fee, $transaction->connect_card_auth_fee_type);
            $cardProcessingFee  = $this->calculateFee($humanAmount, $transaction->connect_card_processing_fee, $transaction->connect_card_processing_fee_type);

            $countryCommission = CountryCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();
            if (!empty($countryCommission)) {
                $countryWiseCommission = abs($this->calculateFee($transaction->amount / 100, $countryCommission->stripe_percentage, 'percentage'));
                $countryWiseCommission = $countryWiseCommission + $countryCommission->stripe_fixed;
            }

            $cardBrandCommission = CardBrandCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();
            if (!empty($cardBrandCommission)) {
                $cardBrandWiseCommission = abs($this->calculateFee($transaction->amount / 100, $cardBrandCommission->stripe_percentage, 'percentage'));
                $cardBrandWiseCommission = $cardBrandWiseCommission + $cardBrandCommission->stripe_fixed;
            }
            if ($transaction->reporting_category == 'refund') {
                $cardAuthFee = -$cardAuthFee;
                $cardProcessingFee = -$cardProcessingFee;
                $cardBrandWiseCommission = -$cardBrandWiseCommission;
                $countryWiseCommission = -$countryWiseCommission;
            }
        }

        $merchantTransction['order_id'] = $transaction->txn_id;
        $merchantTransction['restaurant_id'] = null;
        $merchantTransction['txn_id'] = $transaction->txn_id;
        $merchantTransction['order_date'] = (new DateTime('@' . $transaction->created))->format('Y-m-d H:i:s');
        $merchantTransction['order_type'] = $transaction->service;
        $merchantTransction['order_status'] = $transaction->reporting_category;
        $merchantTransction['payment_method'] = $transaction->service;
        $merchantTransction['service'] = $transaction->service;
        $merchantTransction['order_grand_total'] = $transaction->amount / 100;

        $merchantTransction['processing_fee'] = $cardProcessingFee;
        $merchantTransction['country'] = $cardCountry;
        $merchantTransction['country_charges'] = $this->convertToDecimal($countryWiseCommission);
        $merchantTransction['card_brand'] = $brand;
        $merchantTransction['card_brand_charge'] = $this->convertToDecimal($cardBrandWiseCommission);
        $merchantTransction['auth_fee'] = $cardAuthFee;
        $merchantTransction['charge_back_fee'] = $chargeBackFee;
        $merchantTransction['management_fee'] = $managementFee;
        $merchantTransction['handling_fee'] = $handlingFee;
        $merchantTransction['total_fee'] =  ($this->convertToDecimal($managementFee + $handlingFee + $cardAuthFee + $cardProcessingFee + $cardBrandWiseCommission + $countryWiseCommission + $chargeBackFee + $handlingFee));

        if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
            $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $merchantTransction['total_fee']);
            if (abs($merchantTransction['total_fee']) > ($transaction->amount / 100)) $merchantTransction['net_total'] = 0;
        } else {
            $merchantTransction['total_fee'] = 0;
            $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 + $merchantTransction['total_fee']);
        }
        return $merchantTransction;
    }
    public function calculate_merchant_fees($transaction)
    {
        if ($transaction->service == 'Other') {
            $transaction->service = 'Merchant';
            $transaction->save();
        }
        $cardProcessingFee = 0.00;
        $cardAuthFee = 0.00;
        $managementFee = 0.00;
        $handlingFee = 0.00;
        $chargeBackFee = 0.00;
        $cardCountry = $transaction->card_country;
        $brand = $transaction->card_brand;
        $cardBrandWiseCommission = 0.00;
        $countryWiseCommission = 0.00;
        if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes') {
            $cardAuthFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_auth_fee, $transaction->merchant_card_auth_fee_type));
            $cardProcessingFee  = abs($this->calculateFee($transaction->amount / 100, $transaction->merchant_card_processing_fee, $transaction->merchant_card_processing_fee_type));

            $countryCommission = CountryCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();
            if (!empty($countryCommission)) {
                $countryWiseCommission = abs($this->calculateFee($transaction->amount / 100, $countryCommission->stripe_percentage, 'percentage'));
                $countryWiseCommission = $countryWiseCommission + $countryCommission->stripe_fixed;
            }
            $cardBrandCommission = CardBrandCommissionHistory::where('stripe_transaction_id', $transaction->id)->first();

            if (!empty($cardBrandCommission)) {
                $cardBrandWiseCommission = abs($this->calculateFee($transaction->amount / 100, $cardBrandCommission->stripe_percentage, 'percentage'));
                $cardBrandWiseCommission = $cardBrandWiseCommission + $cardBrandCommission->stripe_fixed;
            }

            if ($transaction->reporting_category == 'refund') {
                $cardAuthFee = -$cardAuthFee;
                $cardProcessingFee = -$cardProcessingFee;
                $cardBrandWiseCommission = -$cardBrandWiseCommission;
                $countryWiseCommission = -$countryWiseCommission;
            }
        }

        if ($transaction->reporting_category == 'dispute') {
            $chargeBackFee  = $this->calculateFee($transaction->amount / 100, $transaction->charge_back_fee, $transaction->charge_back_fee_type);
        }

        $merchantTransction['order_id'] = $transaction->txn_id;
        $merchantTransction['restaurant_id'] = null;
        $merchantTransction['txn_id'] = $transaction->txn_id;
        $merchantTransction['order_date'] = (new DateTime('@' . $transaction->created))->format('Y-m-d H:i:s');
        $merchantTransction['order_type'] = $transaction->service;
        $merchantTransction['order_status'] = $transaction->reporting_category;
        $merchantTransction['payment_method'] = $transaction->service;
        $merchantTransction['service'] = $transaction->service;
        $merchantTransction['order_grand_total'] = $transaction->amount / 100;

        $merchantTransction['processing_fee'] = $cardProcessingFee;
        $merchantTransction['country'] = $cardCountry;
        $merchantTransction['country_charges'] = $this->convertToDecimal($countryWiseCommission);
        $merchantTransction['card_brand'] = $brand;
        $merchantTransction['card_brand_charge'] = $this->convertToDecimal($cardBrandWiseCommission);
        $merchantTransction['auth_fee'] = $cardAuthFee;
        $merchantTransction['charge_back_fee'] = $chargeBackFee;
        $merchantTransction['management_fee'] = $managementFee;
        $merchantTransction['handling_fee'] = $handlingFee;
        $merchantTransction['total_fee'] =  ($this->convertToDecimal($managementFee + $handlingFee + $cardAuthFee + $cardProcessingFee + $cardBrandWiseCommission + $countryWiseCommission + $chargeBackFee + $handlingFee));

        if ($transaction->reporting_category == 'charge' || $transaction->charges_refundable == 'yes')
            $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 - $merchantTransction['total_fee']);
        else {
            $merchantTransction['total_fee'] = 0;
            $merchantTransction['net_total'] = $this->convertToDecimal($transaction->amount / 100 + $merchantTransction['total_fee']);
        }
        return $merchantTransction;
    }
    public function calculate_order_fees($stripeTransaction, $order)
    {
        $orderTypeWiseCommission = 0.00;
        $orderTypeWiseServiceCharge = 0.00;
        $orderBasedOrTransactionWiseCharge = 0.00;
        $countryWiseCommission = 0.0;
        $cardBrandWiseCommission = 0.0;
        $payment_method_fees = 0;
        $chargeBackFee = 0;
        $handlingFee = 0;
        $cardCountry = "";
        $brand = "";

        $serviceCharge = $order->user_service_charge;
        $orderGrandTotal = $order->grand_total - $serviceCharge;

        if ($order->payment_method == 'paypal') {
            $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->online_paypal_fee, $order->online_paypal_fee_type);
        }
        if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Online') {
            $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->online_card_fee, $order->online_card_fee_type);
        }
        if ($order->payment_method == 'cod')
            $payment_method_fees  = $this->calculateFee($orderGrandTotal, $order->online_cod_fee, $order->online_cod_fee_type);

        if ($order->order_type == 'dinein') {
            $orderTypeWiseCommission = $this->calculateFee($orderGrandTotal, $order->dinein_commission_fee, $order->dinein_commission_fee_type);
            $orderTypeWiseServiceCharge = $this->calculateFee($orderGrandTotal, $order->dinein_service_fee, $order->dinein_service_fee_type);
        } else {
            $orderTypeWiseCommission = $this->calculateFee($orderGrandTotal, $order->online_commission_fee, $order->online_commission_fee_type);
            $orderTypeWiseServiceCharge = $this->calculateFee($orderGrandTotal, $order->online_service_fee, $order->online_service_fee_type);
        }
        if ($order->order_charge_type == 'order_based') {
            $orderBasedOrTransactionWiseCharge = $this->calculateFee($orderGrandTotal, $order->per_order_charge, $order->per_order_charge_type);
        } else {
            if ($order->payment_method == 'paypal')
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->paypal_charge, $order->paypal_charge_type);
            if ($order->payment_method == 'Credit Card' || $order->payment_method == 'Credit Card')
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->card_charge, $order->card_charge_type);
            if ($order->payment_method == 'cod')
                $orderBasedOrTransactionWiseCharge  = $this->calculateFee($orderGrandTotal, $order->cod_charge, $order->cod_charge_type);
        }

        $transaction['order_id'] = "#ODR000" . $order->order_id;
        $transaction['restaurant_id'] = $order->restaurant_id;
        if (empty($order->stripe_txn_id)) {
            $transaction['txn_id'] = $order->online_txn_id;
        } else {
            $transaction['txn_id'] = $order->stripe_txn_id;
        }
        $transaction['order_date'] = $order->order_date;
        $transaction['order_type'] = $order->order_type;
        $transaction['order_status'] = $order->order_status;
        $transaction['payment_method'] = $order->payment_method;
        $transaction['service'] = 'Online';

        $transaction['payment_method_fees'] = $payment_method_fees;
        $transaction['order_type_wise_commission'] = $orderTypeWiseCommission;
        $transaction['order_type_wise_service_charge'] = $orderTypeWiseServiceCharge;
        $transaction['order_based_or_transaction_wise_charge'] = $orderBasedOrTransactionWiseCharge;
        $transaction['country'] = $cardCountry;
        $transaction['country_charges'] = $countryWiseCommission;
        $transaction['card_brand'] = $brand;
        $transaction['card_brand_charge'] = $cardBrandWiseCommission;
        $transaction['charge_back_fee'] = $chargeBackFee;
        $transaction['handling_fee'] = $handlingFee;

        $transaction['total_fee'] = $this->convertToDecimal($payment_method_fees + $orderTypeWiseCommission + $orderTypeWiseServiceCharge + $orderBasedOrTransactionWiseCharge + $countryWiseCommission + $cardBrandWiseCommission + $chargeBackFee + $handlingFee);
        if ($order->order_status == 'Failed') {
            $transaction['order_grand_total'] = -$this->convertToDecimal($order->grand_total);
            $transaction['total_fee'] = 0.0;
        } else {
            $transaction['order_grand_total'] = $this->convertToDecimal($orderGrandTotal);
        }
        $transaction['net_total'] = $this->convertToDecimal($transaction['order_grand_total'] - $transaction['total_fee']);
        return $transaction;
    }
    public function calculateFee($amount, $fee, $type)
    {
        $calculatedFee = 0.00;
        if ($type == 'percentage') {
            if ($fee != 0)
                $calculatedFee = ($amount * $fee / 100);
        } else {
            $calculatedFee = $fee;
        }
        return  $this->convertToDecimal($calculatedFee);
    }
    public function direct_stripe_transaction()
    {
        $stripeMode = "test";
        $stripeMode = $this->mode;
        $stripeKey = "";
        if ($stripeMode == 'test') {
            if ($this->account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_test');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_test');
        }
        if ($stripeMode == 'live') {
            if ($this->account_type == 'connect')
                $keySetting = SiteSettingController::findByKey('connect_stripe_private_key_live');
            else $keySetting = SiteSettingController::findByKey('stripe_private_key_live');
        }

        if (!empty($keySetting)) {
            $stripeKey = trim($keySetting->value);
        }

        if ($this->account_type == 'normal') {
            $business = Business::find($this->business_id);
            if (!empty($business)) {
                $mode = $this->mode;
                if (strtolower($mode) == 'test' && $business->stripe_private_key_test != null) {
                    $stripeKey = $business->stripe_private_key_test;
                }
                if (strtolower($mode) == 'live' && $business->stripe_private_key_live != null) {
                    $stripeKey = $business->stripe_private_key_live;
                }
            }
        }

        $stripe = new \Stripe\StripeClient($stripeKey);
        try {
            $disputes = $stripe->disputes->retrieve($this->source, []);
            if (!empty($disputes)) {
                $diff = strtotime(date('Y-m-d')) - $disputes->balance_transactions[0]->available_on;
                $disputes->days_to_response = abs(round($diff / 86400)) . ' days to respond';
            }
            return $disputes;
        } catch (\Stripe\Exception\InvalidRequestException $e) {
            return null;
        }
    }
    public function convertToDecimal($value)
    {
        return floatval(number_format($value, 2, '.', ''));
    }
}